<?php

namespace hji\membership;

use hji\common\ajax\AjaxCreator;
use hji\common\gravityform\import\GravityFormImport;
use hji\common\utils\GFHelper;
use hji\common\pluginUpdate\PluginUpdate;
use hji\common\utils\GravityFormsFilters;
use hji\common\utils\PluginHelper;
use hji\common\utils\WPHelper;
use hji\membership\abstracts\Component;
use hji\membership\advancedOptions\ListingStatusesOptions;
use hji\membership\advancedOptions\ListingStatusesOptionsSubSection;
use hji\membership\advancedOptions\MembershipAdvancedOptions;
use hji\membership\advancedOptions\GravityFormsMappingOptions;
use hji\membership\components\affordability\AffordabilityCalc;
use hji\membership\controllers\APIClient as APIClientController;
use hji\membership\controllers\Dispatcher;
use hji\membership\controllers\PluginAutoUpdateManager;
use hji\membership\extension\twig\AlphabetAgentIndexExtension;
use hji\membership\extension\twig\EmailValidateExtension;
use hji\membership\extension\twig\GridExtension;
use hji\membership\extension\twig\HeroBannerImageExtension;
use hji\membership\extension\twig\ImageExtension;
use hji\membership\extension\twig\NoImageExtension;
use hji\membership\extension\twig\OptimizationHeadExtension;
use hji\membership\extension\twig\RidxExtention;
use hji\membership\extension\twig\SliderResponsiveBreakPointsExtension;
use hji\membership\extension\twig\tests\ListingJustSoldTest;
use hji\membership\controllers\CommonScripts;
use hji\membership\controllers\GForms;
use hji\membership\controllers\ProductManager;
use hji\membership\controllers\RequireJs;
use hji\membership\extension\twig\AgentContactFormExtension;
use hji\membership\extension\twig\AgentImageExtension;
use hji\membership\extension\twig\ListingContactFormExtension;
use hji\membership\extension\twig\NumberFormatExtension;
use hji\membership\extension\twig\ResizeImageExtension;
use hji\membership\extension\twig\ShareButtonsExtension;
use hji\membership\extension\twig\TourThisHomeFormExtension;
use hji\membership\extension\twig\tests\ListingSoldTest;
use hji\membership\extension\twig\URLExtension;
use hji\membership\includes\PHPVersionRequirement;
use hji\membership\models\Customer;
use hji\common\models\JsVars;
use hji\common\utils\Cookies;
use hji\membership\models\Settings;
use hji\membership\models\UserSession;
use hji\membership\services\MembershipRequirePlugins;
use hji\membership\services\Request;
use hji\membership\services\Route;
use hji\membership\services\seo\HJIOpenGraph;
use hji\membership\services\seo\SocialGraph;
use hji\membership\utils\Environment;
use hji\membership\advancedOptions\AdvancedOptions;
use hji\membership\models\MostPopulatedCity;
use hji\membership\utils\URL;
use hji\ResponsiveIDX\controllers\MapSearch;
use Redux;
use hji\membership\utils\URLHelper;

class Membership extends Component
{
    const NAME = 'HJI Membership';
    const VERSION = '2.24.5';

    const SUPPORT_EMAIL = 'help@homejunction.com';
    const SUPPORT_PHONE = '(858) 777-9533 Ext. 4';
    const DEFAULT_APP_HOST = 'https://sm.homejunction.com';
    const DEFAULT_STAGING_HOST = 'https://sm-test.homejunction.com';
    const DEFAULT_MARKET_TRENDS_WIDGET_PATH = '/app/js/market-trends-widget/marketTrendsWidget';
    const DEFAULT_APP_VERSION = '1.4';

    private static $_instance;

    private $_authHasRan = false;
    /**
     * Plugin main file
     * by default must be WP_PLUGIN_DIR . '/hji-membership/Membership.php'
     * @var string
     */
    public static $file;
    public static $slug;
    public static $dir;
    public static $url;
    public static $activeTheme;

    private $hji_mce_buttons = array();

    public $eventDispatcher;
    public $eventLogger;
    public $post_types;
    protected $cssVars = [];

    /**
     * @var APIClientController
     */
    public $apiController;

    public $customerModel;

    public $gravityFormsActive = true;

    /**
     * @var AdvancedOptions
     */
    private $advancedOptions;

    public static $noImgUrl;
    public static $noAgentImgUrl;

    /**
     * @var null
     */
    protected static $hjiListingImagesThemeConfig = null;

    const  PLUGIN_RELATIVE_PATH = 'hji-membership/Membership.php';

    const HJI_GF_MAPPING_OPTIONS_KEY = 'hji_gf_mapping';

    const HJI_GF_MAPPING_PREIMPORTED_OPTIONS_KEY = 'hji_gf_mapping_preimported';

    const NO_IMG_FILENAME = 'photo_na-full.jpg';
    const NO_AGENT_IMG_FILENAME = 'profile-placeholder.jpg';

    function __construct($mainFile)
    {

        //prevent empty main file initialization
        if (empty($mainFile)) {
            $mainFile = WP_PLUGIN_DIR . '/' . self::PLUGIN_RELATIVE_PATH;
        }

        self::$file = $mainFile;
        self::$slug = basename(dirname(self::$file));
        self::$dir  = dirname(self::$file);

        $this->setDiRules();
        new MembershipRequirePlugins();
        // Using this as action
        add_action('upgrader_process_complete', [new PluginUpdate(), 'update']);
        add_action('plugins_loaded', [new PluginUpdate(), 'update']);
        add_filter('tiny_mce_plugins', array($this, 'consolidateHjiMceButtons'));
        add_filter('mce_buttons_3', array($this, 'addTinyMceButtons'), 99);

        add_action('admin_enqueue_scripts', array($this, 'enqueue_scripts'));
//        add_action( 'wp_default_scripts', array($this, 'replace_scripts' ), -1 );

        add_action('admin_print_footer_scripts', array($this, 'adminFooterScripts'));

        add_action('admin_menu', array($this, 'admin_menu'));
        add_action('init', array($this, 'init'));

        // Registering scripts for admin and front-end at once
        // TODO: refactor script loading via correct hooks
        // TODO: eliminate theme customizer hooks usage
        add_action('admin_enqueue_scripts', [$this, 'register_scripts'], 999998);
        add_action('customize_controls_enqueue_scripts', array($this, 'register_scripts'));

        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'), 999999);
        add_action('admin_enqueue_scripts', [$this, 'enqueue_scripts'], 999999);
        add_action('customize_preview_init', array($this, 'enqueue_scripts'));

        add_action('activated_plugin', array($this, 'loadThisPluginFirst'), 100);

        add_action('plugins_loaded', function () {self::$url = URL::getPluginURIBySlug(self::$slug);}, 8);
        add_action('plugins_loaded', array($this, 'registerRequireJs'), 9);
        add_action('plugins_loaded', array($this, 'loadThirdPartyPlugins'));
        add_action('hji_plugin_version_update' . self::$file, array($this, 'membershipUpdate'));
        add_action('plugins_loaded', [$this, 'loadComponents'], 15);

        // !!! Don't change hook and orders. this part must be first loaded before 'register_scripts'
        add_action('plugins_loaded', [$this, 'loadOptions'], 6);
        add_action('plugins_loaded', function () {
            HJIOpenGraph::initActions();
        });


        register_activation_hook(self::$file, array($this, 'pluginActivation'));
        register_deactivation_hook(self::$file, array($this, 'pluginDeactivation'));

        add_filter('script_loader_tag', [$this, 'addDeferToScript'], 9999, 2 );
        add_filter('style_loader_tag', [$this, 'addPreloadLink'], 9999, 4 );

        add_action('init', array($this, 'registerImageSizes'));
        add_filter('image_size_names_choose', array($this, 'addImageSizeNames'));
        add_filter('wp_mail', array($this, 'addSendGridEmailHeaders'));
        add_filter('wp_mail_from_name', array($this, 'removeWPDefaultFromName'));
        add_filter('timber/loader/paths', array($this, 'setDefaultTemplatePaths'));
        /** following WP pattern when author is not set wp-includes/comment-template.php::get_comment_author() */
        add_filter('timber/user/name', function ($displayName, $user) {
            return $displayName ?? __('Anonymous');
        }, 10, 2);
        add_filter('http_request_host_is_external', array('\hji\membership\controllers\Downloads', 'allow_wp_catalog_host'), 10, 3);
        add_filter('redux/hji-boulevard-settings/field/class/address', [$this, 'registerPathToReduxExtension'], 10, 2);
        (new GravityFormImport())->initHooks();
        $this->componentsInit();


        rendering\twigRenderer::setActionHooks();

        $phpRequirement = new PHPVersionRequirement();
        $phpRequirement->setActionHooks();

        (new SocialGraph())->init();

        // this string prevent redux update notification
        $GLOBALS['redux_update_check'] = 1;

        //hide plugins in network plugins
        if (is_network_admin()) {
            add_action('plugins_loaded', [$this, 'hidePluginsActivateInNetwork']);
        }

        self::$noAgentImgUrl = self::copyDefImages(self::NO_AGENT_IMG_FILENAME);
        self::$noImgUrl = self::copyDefImages(self::NO_IMG_FILENAME);
    }

    /**
     * @param string $imageFileName
     * @return string|null
     */
    private static function copyDefImages(string $imageFileName)
    {
        $uploadsDir = wp_get_upload_dir();
        $destFilePath = $uploadsDir['basedir'] . '/' . $imageFileName;
        $uploadsDirMulti = URLHelper::getUploadDir();
        if (file_exists($destFilePath)) {
            return $uploadsDirMulti . '/' . $imageFileName;
        }

        if (!copy(self::$dir . '/resources/images/' . $imageFileName, $destFilePath ) ) {
            return null;
        }

        return $uploadsDirMulti . '/' . $imageFileName;
    }

    /**
     * @return mixed
     */
    public function getHjiListingImagesThemeConfig()
    {
        if (self::$hjiListingImagesThemeConfig === null && current_theme_supports('hji-listing-images')) {
            self::$hjiListingImagesThemeConfig = get_theme_support('hji-listing-images');
        }

        return self::$hjiListingImagesThemeConfig;
    }

    private function componentsInit(): void
    {
        (AffordabilityCalc::getInstance())->init();
    }

    /**
     * @param $filterPath
     * @param $field
     * @return mixed|string
     */
    public function registerPathToReduxExtension($filterPath, $field)
    {
        if ($field['type'] === 'address') {
            $filterPath = self::$dir . '/classes/extension/redux/address/address/class-redux-address.php';
        }

        return $filterPath;

    }

    /**
     * @param $src
     * @param $handle
     * @return false|string
     */
    public function filteringScriptsSrc($src, $handle)
    {
        $handlers = [
            'jquery-core'
        ];

        foreach ($handlers as $scriptHandle) {
            if ($scriptHandle === $handle && strpos( $src, '?ver=')) {
                return remove_query_arg( 'ver', $src );
            }
        }

        return  $src;
    }

    /**
     * @param $src
     * @param $handle
     * @return false|string
     */
    public function filteringStyleSrc($src, $handle)
    {
        $handlers = [
            'blvd/main-style'
        ];

        foreach ($handlers as $scriptHandle) {
            if ($scriptHandle === $handle && strpos( $src, '?ver=')) {
                return remove_query_arg( 'ver', $src );
            }
        }

        return  $src;
    }

    public function addPreloadLink($tag, $handle, $href, $media)
    {
        $defPreload = '<link rel="preload" id="link-%s" href="%s" as="style" onload="this.onload=null;this.rel=\'stylesheet\'">
            <noscript>%s</noscript>'
        ;

//        $defPreload = '<link rel="stylesheet" href="%s" media="print" onload="this.media=\'%s\'"><noscript>%s</noscript>';

        $defPrefetch = '<link rel="prefetch" href="%s" as="style" >';

        $handles = [
            'idx-open-house-widget' => 'preload',
            'ridx-open-house' => 'preload',
            'mortgage-calculator' => 'preload',
            'hji-recent-sales' => 'preload',
            'hji-search-alert-frequency' => 'preload',
            'hji-user-login' => 'preload',
//            'blvd/font-awesome',
//            'blvd/swiper-slider',
            'blvd/hji-icons' => 'preload',
            'blvd/animatecss' => 'preload',
            'font-awesome' => 'font',
            'hji-accessibility' => 'preload',
            'membership-gl' => 'preload',
//            'wp-jquery-ui-dialog' => 'preload',
//            'blvd/main-style' => 'preload',
            'hji-icons' => 'font',
            'blvd/swiper-slider' => 'preload',
            'blvd/theme-font' => 'font'
        ];

        foreach( $handles as $scriptHandle => $linkRel) {
            if ($linkRel === 'font' && $scriptHandle === $handle) {
                return '<link rel="preload" as="style" href="' . $href . '">
                    <link rel="stylesheet" media="print" onload="this.onload=null;this.removeAttribute(\'media\');" href="' . $href . '">
                    <noscript><link ' . $scriptHandle . ' rel="stylesheet" href="' . $href . '" /></noscript>'
                ;
            }

            if ($linkRel === 'prefetch' && $scriptHandle === $handle) {
                return sprintf($defPrefetch, $href);
            }

            if ($linkRel === 'exclude' && $scriptHandle === $handle) {
                return sprintf($defPrefetch, $href);
            }

            if ($linkRel === 'preload' && $scriptHandle === $handle) {
                return sprintf($defPreload, $handle, $href, $tag);
            }

        }

        return $tag;
    }

    public function addDeferToScript($tag, $handle)
    {
        $handles = [
//            'jquery-core',
//            'hji-accessibility' => 'async',
//            'require-js' => 'defer',
            'blvd/modernizr' => 'defer',
            'hji-search-alert-frequency' => 'defer',
//            'hji-lazysizes' => 'async',
//            'jquery-migrate' => 'async',
//            'jquery-ui-mouse',
//            'jquery-ui-resizable',
//            'jquery-ui-draggable',
//            'jquery-ui-controlgroup',
//            'jquery-ui-checkboxradio',
//            'jquery-ui-button',
//            'jquery-ui-dialog',
//            'blvd/main-script',
//            'hji-mortgage-calculator' => 'async',
//            'backbone' => 'defer',
//            'underscore' => 'async',
//            'idx-open-house-widget' => 'defer',
//            'wp-polyfill' => 'defer',
//            'wp-i18n' => 'defer'
        ];

        $searchString = ' %s="%s" src';
        foreach( $handles as $scriptHandle => $asyncType) {
            if ($scriptHandle === $handle) {
                $needle = sprintf($searchString, $asyncType, $asyncType);
                return str_replace(' src', $needle, $tag);
            }
        }

        return $tag;
    }

    /**
     * Instantiating classes separately only once
     *
     * @since 1.0.4
     */
    function initAccessibleClasses()
    {
        $this->settingsController   = new controllers\Settings();

        if (is_admin()) {
            $this->settingsController->setActionHooks();
        }

        $this->customerModel        = Customer::getInstance();
        $this->apiController        = new APIClientController($this->customerModel);
        $this->productManager       = ProductManager::getInstance();
        $this->gFormsController     = new GForms();
        $pluginAutoUpdateManager = new PluginAutoUpdateManager();
        $pluginAutoUpdateManager->switchDisableAutoUpdate();

        if ($this->isLicensed())
        {
            if (is_admin())
            {
                $googleApiSettingsController = new controllers\GoogleApiSettings();
                $googleApiSettingsController->setActionHooks();
                $this->downloadsController = new controllers\Downloads();
            }

            $this->productManager->initActionHooks();


            // Register Membership First
            $this->productManager->registerProduct(self::$file, 'plugin');

            // Start event logger
            $this->_startEventLogger();
        }

        $this->requireJs = RequireJs::getInstance();

        $this->gFormsController->setWPActions();
    }


    /**
     * Returns instance of the Membership Settings Model
     *
     * @since 2.6.0
     * @return \hji\membership\models\Settings  Settings Model
     */
    public static function settings()
    {
        return Settings::getInstance();
    }


    /**
     * Instance of the Membership Class
     *
     * @param string $file
     * @return Membership
     * @since   1.0.4
     */
    static function getInstance($file = '')
    {
        if (!( self::$_instance instanceof self))
        {
            self::$_instance = new self($file);
        }

        return self::$_instance;
    }


    /**
     * Checks if membership is licensed
     * based on last available authentication token.
     *
     * @return bool
     *
     * @since 1.0.4
     */
    function isLicensed()
    {
        // If authentication has ran in this runtime, token is set
        // and customer model has markets schema - we're good to go.

        if (UserSession::getToken() && $this->customerModel->markets)
        {
            return true;
        }
        else if (!$this->_authHasRan)
        {
            $authenticated = false;

            // Avoid running authentication while options being saved via options.php page
            if (!isset($_REQUEST['option_page']))
            {
                $authenticated = $this->apiController->authenticate(true);
                $this->_authHasRan = true;
            }

            return $authenticated;
        }

        return false;
    }


    /**
     * Returns array of licensed markets
     * or false
     *
     * @return array
     */
    function getMarkets()
    {
        $markets = (array) $this->customerModel->markets;
        return (!empty($markets)) ? array_keys($markets) : array();
    }


    /**
     * Forces Membership plugin to load first
     * Important, since we're calling common membership methods
     * from other HJI plugins.
     */
    function loadThisPluginFirst()
    {
        // ensure path to this file is via main wp plugin path
        $wp_path_to_this_file = preg_replace('/(.*)plugins\/(.*)$/', WP_PLUGIN_DIR."/$2", self::$file);
        $this_plugin = plugin_basename(trim($wp_path_to_this_file));
        $active_plugins = get_option('active_plugins');
        $this_plugin_key = array_search($this_plugin, $active_plugins);

        if ($this_plugin_key)
        {
            // If it's 0 it's the first plugin already, no need to continue

            array_splice($active_plugins, $this_plugin_key, 1);
            array_unshift($active_plugins, $this_plugin);
            update_option('active_plugins', $active_plugins);
        }
    }


    function admin_menu()
    {
        add_menu_page(
            self::NAME,
            self::NAME,
            'manage_options',
            self::$slug,
            false,
            self::$url . '/resources/images/admin-menu.png'
        );
    }


    function init()
    {
        $theme = wp_get_theme();
        self::$activeTheme = $theme;

        WPHelper::registerPluginVersion(self::$file);
        add_action('wp_print_footer_scripts', '\hji\common\models\JsVars::loadVars', 0);
        add_action('admin_print_scripts', '\hji\common\models\JsVars::loadVars', 0);
        (new AjaxCreator)->init();
    }

    public function loadComponents()
    {
        if (!GFHelper::gravityFormsExists() || !GFHelper::gravityFormsAPIEnable()) {
            //throw new \RuntimeException('Gravity Forms plugin is not active');
            error_log('Wordpress Notice: Gravity Forms plugin is not active');
            add_action('admin_notices', function () {
                echo '<div class="notice notice-error  is-dismissible"><p>The following required plugin is currently inactive: <em>Gravity Forms</em></p></div>';
            }, 40);

            $this->gravityFormsActive = false;
        } elseif (!GFHelper::isGravityFormExportInitialized()) {
            //throw new \RuntimeException('Gravity Forms plugin is not active');
            error_log('Wordpress Notice: Gravity Forms broken (gravityforms/export.php not exist)');
            add_action('admin_notices', function () {
                echo '<div class="notice notice-error  is-dismissible"><p>Gravity Forms broken (gravityforms/export.php not exist). Please re-install plugin.</p></div>';
            }, 40);

            $this->gravityFormsActive = false;
        }

        if ($this->gravityFormsActive) {
            $components = [
                GravityFormsFilters::class,
            ];
            $components = apply_filters('hji_membership_components', $components);

            foreach ($components as $component) {
                $this->di()->create($component);
            }
        }

        add_filter('gform_confirmation_anchor', function () { return false; });
    }

    public function loadOptions()
    {
        if (WPHelper::is_heartbeat()) {
            return;
        }

        $advancedOptions =  AdvancedOptions::getInstance();
        $advancedOptions->addSection(new MembershipAdvancedOptions());

        if (is_admin() && !wp_doing_ajax() && is_user_logged_in()) {
            if (PluginHelper::hji_is_plugin_active('hji-responsive-idx/responsive-idx.php')) {
                $markets = $this->getMarkets();
                sort($markets);
                if (is_array($markets) && count($markets) > 0) {
                    $advancedOptions->addSection(new ListingStatusesOptions());
                    $params = ['subsection' => true];
                    foreach ($markets as $market) {
                        $advancedOptions->addSection(new ListingStatusesOptionsSubSection($market, '', '', $params));
                    }
                }
            }

            if ($this->gravityFormsActive && !empty($_GET['page']) && $_GET['page'] === 'hji-advanced-options') {
                $advancedOptions->addSection(new GravityFormsMappingOptions());
            }

            // must be on one (or more) step after current action
            add_action('init', function () use ($advancedOptions) {
                $advancedOptions->initSettings();
            }, 7);
        }
    }

    function loadThirdPartyPlugins()
    {
        add_filter('timber/twig',  function(\Twig\Environment $twig) {
            $twig->addExtension(new NumberFormatExtension());
            $twig->addExtension(new ResizeImageExtension());
            $twig->addExtension(new URLExtension());
            $twig->addExtension(new AgentImageExtension());
            $twig->addExtension(new ShareButtonsExtension());
            $twig->addExtension(new AgentContactFormExtension());
            $twig->addExtension(new ListingContactFormExtension());
            $twig->addExtension(new AlphabetAgentIndexExtension());
            $twig->addExtension(new TourThisHomeFormExtension());
            $twig->addExtension(new EmailValidateExtension());
            $twig->addExtension(new OptimizationHeadExtension());
            $twig->addExtension(new SliderResponsiveBreakPointsExtension());
            $twig->addExtension(new NoImageExtension());
            $twig->addExtension(new ImageExtension());
            $twig->addExtension(new HeroBannerImageExtension());
            $twig->addExtension(new ImageExtension());
            $twig->addExtension(new GridExtension());

            $twig->addTest(new ListingSoldTest());
            $twig->addTest(new ListingJustSoldTest());
            $twig->addExtension(new RidxExtention());

            return $twig;
        });

        if (!class_exists('ReduxFramework')) {
            require_once(self::$dir . '/third-party/redux-framework/redux-core/framework.php');
        }

        if (is_admin() && !wp_doing_ajax() && is_user_logged_in()) {
            add_action('redux/loaded', function ($redux) {
                if ($redux === null) {
                    return;
                }

                if (!empty($redux->args['opt_name'])
                    && ($optName = $redux->args['opt_name']) === 'advanced_options'
                ) {
                    Redux::set_extensions($optName , __DIR__ . '/extension/redux/address/', true);
                }
            });
        }
    }

    static function marketTrendsWidgetPath()
    {
        $mapSearchAppVersion = '/' . self::DEFAULT_APP_VERSION;

        if (defined('HJI_MARKET_TRENDS_WIDGET_URL') && URL::isValidURL(HJI_MARKET_TRENDS_WIDGET_URL)) {
            $path = HJI_MARKET_TRENDS_WIDGET_URL;
        } elseif (Environment::is(Environment::DEV)) {
            $path = self::DEFAULT_STAGING_HOST . self::DEFAULT_MARKET_TRENDS_WIDGET_PATH;
        } else {
            $path = self::DEFAULT_APP_HOST . $mapSearchAppVersion . self::DEFAULT_MARKET_TRENDS_WIDGET_PATH;
        }

        return $path;
    }

    function registerRequireJs()
    {
        $this->requireJs->addPath('hji-membership', self::$url . '/resources/scripts/build');
        $this->requireJs->addPath('hji-membership/third-party', self::$url . '/third-party');

        $slipstreamUrl = Environment::is(Environment::DEV) ? 'https://slipstream-test-cdn.homejunction.com' :
            'https://slipstream-cdn.homejunction.com';
        $this->requireJs->addPath('slipstream', $slipstreamUrl . '/js/1.5.3', true);
        $this->requireJs->addPath('hji-common/widgets/poi', 'components/widgets/poi/poi');

        $mapboxDraw = 'https://api.mapbox.com/mapbox.js/plugins/leaflet-draw/v0.4.10/leaflet.draw';
        $this->requireJs->addPath('mapbox/draw', $mapboxDraw, true);

        $this->requireJs->addPath('hji-market-trends', rtrim(static::marketTrendsWidgetPath(), '.js'), true);
    }


    function register_scripts()
    {
        CommonScripts::register();
        $min = (SCRIPT_DEBUG || Environment::is('dev')) ? '' : '.min';

        wp_register_script('require-js', self::$url . "/resources/scripts/build/hji.js", array('jquery', 'underscore', 'backbone'), self::VERSION, true);
        if (!is_admin()) {
            wp_register_script('hji-accessibility', self::$url . "/resources/scripts/build/accessibility{$min}.js", ['jquery'], '', true);
            wp_enqueue_script('hji-accessibility');
            wp_enqueue_script('hji-swiper-slider', 'https://cdnjs.cloudflare.com/ajax/libs/Swiper/8.0.6/swiper-bundle.min.js', [], '', true);
        }

        // Do not load on themes pages (conflict with Redux Framework).
        if (!stripos($_SERVER['REQUEST_URI'], 'themes.php'))
        {
            wp_enqueue_script('require-js');
            wp_register_style('hji-membership-admin', self::$url . "/resources/styles/admin$min.css");
        }

        wp_register_style('hji-membership-mandatory-admin', self::$url . "/resources/styles/mandatory-admin$min.css");
        // prevent removing hji-membership-mandatory-admin in gravity forms admin
        add_filter( "gform_noconflict_styles", function ($required_objects) {
            return array_merge($required_objects, ['hji-membership-mandatory-admin']);
        });
        wp_register_style('hji-membership', self::$url . "/resources/styles/membership$min.css");

        $settings = $this->di()->get('hji\membership\models\Settings');
        $api = $this->apiController->api;

        JsVars::setVar('common', 'debug',       ((defined('SCRIPT_DEBUG') && SCRIPT_DEBUG === true)) || Environment::is(Environment::DEV));
        JsVars::setVar('common', 'licenseKey',  $settings->licenseKey);
        JsVars::setVar('common', 'areaBias',    $settings->areaBias);
        JsVars::setVar('common', 'apiVersion',  $api::$api_version);
        JsVars::setVar('common', 'siteUrl',     home_url());
        JsVars::setVar('common', 'ajaxUrl',     admin_url('admin-ajax.php'));
        JsVars::setVar('common', 'apiBaseUrl',  $api->getApiBaseUrl());
        JsVars::setVar('common', 'env',         Environment::environment());
        JsVars::setVar('common', 'cookiePath',  Cookies::COOKIEPATH);
        JsVars::setVar('common', 'cookieDomain',Cookies::COOKIE_DOMAIN);
        JsVars::setVar('common', 'customer',    $settings->customer);
        JsVars::setVar('common', 'gMapsApiKey', static::settings()->googleApi->googleApiKey);
        JsVars::setVar('common', 'hjgMapsApiKey','AIzaSyB2NnYBBeSC8UVq1OcRA5SLgfNyj1cYAd8');
        JsVars::setVar('common', 'isAdmin', is_admin() && !wp_doing_ajax() && is_user_logged_in());
        JsVars::setVar('common', 'hjiAdvancedOptions', AdvancedOptions::getInstance()->getOptions());
        JsVars::setVar('common', 'marketTrendsWidgetUrl', static::marketTrendsWidgetPath());

        if (UserSession::getToken())
        {
            JsVars::setVar('common', 'token',  UserSession::getToken());
            JsVars::setVar('common', 'tokenExpires',  UserSession::getTokenExpires());
        }

        // Add markets and their centroids
        // Since we're calling customerModel before checking if customer is licensed
        // we need to check if markets data exists

        if ($this->customerModel->markets)
        {
            $markets = array();

            foreach($this->customerModel->markets as $k => $market)
            {
                $m = new \stdClass();
                $m->centroid = @$market->metadata->centroid;
                $markets[$k] = $m;
            }

            JsVars::setVar('common', 'markets', $markets);

            $mostPopulatedCity= new MostPopulatedCity();
            JsVars::setVar('common', 'mostPopulatedCity', $mostPopulatedCity->jsVar());
        }
    }

    private function getSupportedThemes(): array
    {
        return [
            'twenty-fourteen',
            'twenty-nineteen',
            'twenty-twenty-one',
            'twenty-twenty-two',
            'twenty-twenty',
            'astra',
            'avada',
            'divi',
            'hello-elementor',
            'beaver-builder'
        ];
    }

    /**
     * @param string $themeName
     * @return bool
     */
    private function isSupportedThemes(string $themeName): bool
    {
        return in_array($themeName, $this->getSupportedThemes());
    }

    /**
     * @param \WP_Theme $theme
     * @return void
     */
    private function gridSizesStyles(\WP_Theme $theme)
    {
        $min = (SCRIPT_DEBUG || Environment::is('dev')) ? '' : '.min';

        if ($theme->exists()) {
            $themeName = str_replace(' ', '-', strtolower($theme->get('Name')));

            if ($this->isSupportedThemes($themeName)) {
                wp_enqueue_style("hji-{$themeName}", self::$url . "/resources/styles/supported-themes/{$themeName}{$min}.css", [], '', 'all');
            } else {
                wp_enqueue_style('hji-default-grid-sizes', self::$url . "/resources/styles/default-grid-sizes$min.css", [], '', 'all');
            }
        }
    }

    public static function getAdditionalCssClass(\WP_Theme $theme): string
    {
        $additionalClasses = [];
        $themeName = $theme->get('Name');
        if ($themeName && strtolower($themeName) !== 'avada') {
            $additionalClasses[] = 'alignwide';
        }

        $additionalClasses = apply_filters('get_additional_css_for_non_hji_themes', $additionalClasses, $theme);

        return implode(' ', $additionalClasses);
    }

    private function enqueueGravityFormsStyles()
    {
        $disableStyles = apply_filters('disable_default_gravity_forms_styles', false);

        if (empty($disableStyles)) {
            wp_enqueue_style('gforms_reset_css');
            wp_enqueue_style('gforms_datepicker_css');
            wp_enqueue_style('gforms_formsmain_css');
            wp_enqueue_style('gforms_ready_class_css');
            wp_enqueue_style('gforms_browsers_css');
        }
    }

    function enqueue_scripts()
    {
        $this->register_scripts();
        $script_suffix = Environment::is('dev') ? '' : '.min';

        if (!current_theme_supports('hji-twitter-bootstrap')) {
            wp_enqueue_style('hji-bootstrap');
            wp_enqueue_script('hji-bootstrap');
            wp_enqueue_style('hji-common');
            wp_enqueue_style('font-awesome');

            $additionalStyles = 'body {
                --hj-bg-color-light: rgb(255 255 255 / 70%);
                --hj-bg-color-dark: rgb(0 0 0 / 5%);
                --hj-border-color-dark: rgb(0 0 0 / 6%);
                --hj-border: none;
                --hj-box-shadow: 0px 1px 3px 0px var(--hj-shadow-color-dark);
                --hj-shadow-color-dark: rgb(0 0 0 / 15%);
                --hj-fg-color-light-grey: rgb(0 0 0 / 30%);
                --hj-fg-color-dark:rgb(0 0 0 / 63%);
                --hj-bg-color-light-grey: rgb(0 0 0 / 6%);
                --hj-fg-alert-error-color: rgb(255 0 0);
                }'
            ;

            wp_add_inline_style('hji-bootstrap', $additionalStyles);

            $this->gridSizesStyles(self::$activeTheme);

            $this->enqueueGravityFormsStyles();
        }

        if (is_admin()) {
            wp_enqueue_script('hji-membership-admin');
            wp_enqueue_style('hji-membership-admin');
            wp_enqueue_style('hji-membership-mandatory-admin');
            wp_enqueue_script('hji-membership-main-admin', self::$url . '/resources/scripts/build/main-admin.js', ['jquery'], '', true);
        } else {
            /**
             * Remember, you did this not because you're a dumb ass,
             * but because Slipstream is not compatible with the older versions
             * of the scripts that WP comes with. Sigh...
             */
            wp_deregister_script('underscore');
            wp_deregister_script('backbone');
            wp_register_script('underscore', 'https://cdnjs.cloudflare.com/ajax/libs/underscore.js/1.9.1/underscore-min.js', array('jquery'), true);
            wp_register_script('backbone', 'https://cdnjs.cloudflare.com/ajax/libs/backbone.js/1.4.0/backbone-min.js', array('underscore'), true);

            wp_enqueue_style('hji-accessibility', self::$url . "/resources/styles/accessibility/main{$script_suffix}.css");
        }

        wp_enqueue_style('hji-icons', self::$url . '/resources/fonts/hji-icons/stylesheets/hji-icons.css', array(), '1.0.0', 'all');
        wp_enqueue_style('membership-gl', self::$url . "/resources/styles/membership{$script_suffix}.css", array(), '', 'all');
//        wp_enqueue_script('jquery-ui-dialog');
        wp_enqueue_script('underscore');
//        wp_enqueue_style('wp-jquery-ui-dialog');
        wp_enqueue_script('hji-lazysizes', 'https://cdnjs.cloudflare.com/ajax/libs/lazysizes/5.3.2/lazysizes.min.js', [], '', true);

        $loading = "
            if ('loading' in HTMLImageElement.prototype) {
                const images = document.querySelectorAll('img[loading=\"lazy\"]');
                images.forEach(img => {
                    if (typeof img.dataset.src !== 'undefined') {
                        img.src = img.dataset.src;
                    }
                });
            }"
        ;

        $lazyConfig = "
            window.lazySizesConfig = window.lazySizesConfig || {};
            window.lazySizesConfig.expFactor = 0;
            window.lazySizesConfig.init = false;
            window.lazySizesConfig.loadMode = 1;
        ";

        if (!is_admin()) {
            wp_add_inline_script('require-js', $loading);
            wp_add_inline_script('hji-lazysizes', $lazyConfig, 'before');
        }
    }

    public static function replace_scripts($scripts)
    {
        self::set_script( $scripts, 'jquery-migrate', URL::getPluginURIBySlug(self::$slug) . '/resources/scripts/build/jquery-migrate/jquery-migrate-1.4.1.js', array(), '1.4.1-wp' );
        self::set_script( $scripts, 'jquery', false, array( 'jquery-core', 'jquery-migrate' ), '1.12.4-wp' );
    }

    private static function set_script( $scripts, $handle, $src, $deps = array(), $ver = false, $in_footer = false ) {
        $script = $scripts->query( $handle, 'registered' );

        if ( $script ) {
            // If already added
            $script->src  = $src;
            $script->deps = $deps;
            $script->ver  = $ver;
            $script->args = $in_footer;

            unset( $script->extra['group'] );

            if ( $in_footer ) {
                $script->add_data( 'group', 1 );
            }
        } else {
            // Add the script
            if ( $in_footer ) {
                $scripts->add( $handle, $src, $deps, $ver, 1 );
            } else {
                $scripts->add( $handle, $src, $deps, $ver );
            }
        }
    }

    /**
     * Invoke js in the footer for common usage
     */
    function adminFooterScripts()
    {
        if (stristr($_SERVER['REQUEST_URI'], 'widgets.php'))
        {
            ?>
            <script>
                jQuery(document).ready(function(){
                    HJI.Membership.Admin.autoSaveWidgets();
                });
            </script>
            <?php
        }
    }


    /**
     * Consolidating HJI MCE buttons into a single array.
     *
     * Before TinyMCE gets initialized, we're running custom filter
     * to gather all HJI buttons, so they can be added
     * to MCE panel at once.
     *
     * @param $plugins
     * @return mixed
     */
    function consolidateHjiMceButtons($plugins)
    {
        $this->hji_mce_buttons = apply_filters('hji_mce_buttons', $this->hji_mce_buttons);

        return $plugins;
    }


    function addTinyMceButtons($buttons)
    {
        if (empty($this->hji_mce_buttons))
        {
            return $buttons;
        }

        return array_merge($this->hji_mce_buttons, $buttons);
    }


    function pluginActivation()
    {
        // deactivate Redux plugin if activated...
        $redux_slug = 'redux-framework/redux-framework.php';
        if (is_plugin_active($redux_slug))
        {
            deactivate_plugins($redux_slug);
        }

        WPHelper::registerPluginVersion(self::$file);

        /**
         * Useful to other plugins to run their activation processes
         * like settings setup and rewrite rules flushes.
         */
        do_action('hji_membership_activation');
    }


    function pluginDeactivation(){}


    function membershipUpdate($params)
    {
        if (isset($params['plugin']) && $params['plugin'] == self::$file)
        {
            // Force re-authentication after the update

            $this->_authHasRan = false;
            $this->apiController->authenticate(true);
        }
    }


    /**
     * @deprecated
     * @return string
     */
    public static function environment()
    {
        _deprecated_function(__FUNCTION__, '2.9.12', 'Environment::environment()');

        return Environment::environment();
    }


    /**
     * Registers default image dimensions.
     */
    function registerImageSizes()
    {
        // Used for agent headshots in email templates

        add_image_size('hji/150w150h', 150, 150, array( 'center', 'top' ));

        // Used for agent logo in email templates

        add_image_size('hji/150h', 0, 150, false);
    }


    /**
     * Adds image sizes to some selector
     *
     * Allows media manager to pass custom media sizes
     * into select image (JS) object,
     * which is used in the License Key admin UI to select the correct size.
     *
     * @param $sizes
     *
     * @return array
     */
    function addImageSizeNames($sizes )
    {
        return array_merge( $sizes, array(
            'hji/150w150h' => __( 'HJI 150x150' ),
            'hji/150h' => __( 'HJI 150 height' ),
        ) );
    }


    /**
     * Adds SendGrid API-specific header to track email activity by license key
     *
     * @param $params
     *
     * @return mixed
     */
    function addSendGridEmailHeaders($params)
    {
        $settings = new Settings();

        if ($settings->licenseKey)
        {
            $obj = new \stdClass();
            $obj->category = array($settings->licenseKey);
            $headers['X-SMTPAPI'] = json_encode($obj);

            if (isset($params['headers']))
            {
                if (is_string($params['headers']))
                {
                    $params['headers'] = trim($params['headers']) . "\r\n" . 'X-SMTPAPI: ' . json_encode($obj) . "\r\n";
                }
                else if (is_array($params['headers']))
                {
                    $params['headers'] = array_merge($params['headers'], $headers);
                }
            }
            else
            {
                $params['headers'] = $headers;
            }
        }

        return $params;
    }


    /**
     * Removes "From WordPress" from email headers.
     *
     * Implements wp_mail_from_name filter hook.
     * When wp_mail function is used directly and From Name is not set,
     * WP will use FROM: WordPress by default.
     *
     * @param $fromName string
     *
     * @return string
     */
    function removeWPDefaultFromName($fromName)
    {
        if ($fromName == 'WordPress' || !$fromName)
        {
            $settings = new Settings();
            $fromName = ($settings->customer['officeName']) ? $settings->customer['officeName'] : '';
        }

        return $fromName;
    }


    private function _startEventLogger()
    {
        $userActionLogger = $this->di()->get('hji\\membership\\models\\UserActionLogger');
        $this->eventLogger = $this->di()->create('hji\\membership\\observers\\EventLogger', [$userActionLogger]);
        $this->getServiceContainer()->getDispatcher()->attachListener($this->eventLogger);
        $this->getServiceContainer()->getDispatcher()->attachListener($this->eventLogger);
    }


    /**
     * Sets default Twig paths in the following order:
     *  Child Theme
     *  Parent Theme
     *  Plugins
     *
     * @param $paths
     *
     * @return array
     */
    function setDefaultTemplatePaths($paths)
    {
        $childThemePath  = get_stylesheet_directory();
        $parentThemePath = get_template_directory();

        if (is_child_theme())
        {
            if (false !== $path = $this->getHJITemplatesPath($childThemePath))
                $default[] = $path;

            $default[] = $childThemePath;
        }

        if (false !== $path = $this->getHJITemplatesPath($parentThemePath))
            $default[] = $path;

        $default[] = $parentThemePath;

        $paths = array_merge($default, $paths);
        $uniquePaths = array_unique($paths);

        return $uniquePaths;
    }


    protected function getHJITemplatesPath($themePath)
    {
        $hjiTemplatesDir = 'hji-templates';
        $path = trailingslashit($themePath) . $hjiTemplatesDir;

        if (file_exists($path))
            return $path;

        return false;
    }

    /**
     * @return array Post Object classes
     */
    public function getPostTypes()
    {
        return $this->post_types;
    }


    private function setDiRules()
    {
        $this->di()->addRule(extension\postTypes\PostTypeManager::class, ['shared' => true]);
        $this->di()->addRule(Request::class, ['shared' => true]);
        $this->di()->addRule(Route::class, ['shared' => true]);
        $this->di()->addRule(Dispatcher::class, ['shared' => true]);

        add_filter('parse_request', function ($query) {
//            var_dump($query);die('err');
            $this->di()->get(Request::class)->setQuery($query);
        });
    }

    /**
     * Array of plugins, which must be hide "Network Activate" in network plugins list
     * @return array
     */
    private function getPluginsHidingActivateInNetwork()
    {
        return [
            'hji-blvd-framework/BlvdBase.php',
        ];
    }

    /**
     * @param $plugins
     * @return mixed
     */
    public function hidePluginsActivateInNetwork($plugins)
    {
        $pluginsHidingActivateInNetwork = apply_filters('hji_membership_hide_plugins_in_network', $this->getPluginsHidingActivateInNetwork());
        foreach ($pluginsHidingActivateInNetwork as $pluginHidingActivateInNetwork) {
            // remove "Network Activate" link
            add_filter( "network_admin_plugin_action_links_" . $pluginHidingActivateInNetwork, function ($actions) {
                unset($actions['activate']);

                return $actions;
            });
            //prevent bulk action activation
            add_action('admin_action_activate-selected', function () use ($pluginHidingActivateInNetwork) {
                if (is_network_admin()) {
                    foreach ($_POST['checked'] as $key => $item) {
                        if ($item === $pluginHidingActivateInNetwork) {
                            unset($_POST['checked'][$key]);
                        }
                    }
                }
            });
        }
    }

    /**
     * @return array
     */
    public function getCssVars()
    {
        return $this->cssVars;
    }

    public function getCssVar($key)
    {
        return $this->cssVars[$key] ?? null;
    }

    /**
     * @param string $key
     * @param mixed $value
     * @return $this
     */
    public function addCssVar($key, $value)
    {
        if (!empty($key && $value !== null)) {
            $this->cssVars[$key] = $value;
        }

        return $this;
    }

    public function printCssVariables()
    {
        $output = '';
        if (count($this->cssVars > 0)) {
            foreach ($this->cssVars as $key => $value) {
                $output .= '--' . $key . ': ' . $value . ';';
            }

            $output = '<style>body{' . $output . '}</style>';
        }

        echo $output;
    }

    public function getBrokersData()
    {
        $markets = $this->getMarkets();

        $data = [];
        foreach ($markets as $market) {
            $data[$market] = $this->getBrokerDataByMarket($market);
        }

        return $data;
    }

    public function getBrokerDataByMarket($market)
    {
        if (class_exists(\Redux::class)) {
            return \Redux::get_option('advanced_options', $market . '-broker-of-record-item');
        }

        return '';
    }
}
