<?php

namespace hji\membership\advancedOptions;

use hji\membership\Membership;
use ReduxFramework;

class AdvancedOptions
{
    public $args = [];
    public $sections = [];
    public $theme;
    public $ReduxFramework;
    public $reduxOptionName = 'advanced_options';
    public $optionsObjects = [];
    public $avilabledFields = [];

    private static $instances = [];


    /**
     * @return AdvancedOptions
     */
    public static function getInstance()
    {
        $cls = static::class;

        if (!isset(self::$instances[$cls])) {
            self::$instances[$cls] = new static;
        }

        return self::$instances[$cls];
    }

    public function initSettings()
    {
        if (!class_exists('ReduxFramework')) {
            return;
        }

        // Set the default arguments
        $this->setArguments();

        if (!isset($this->args['opt_name'])) { // No errors please
            return;
        }

        $this->ReduxFramework = new ReduxFramework($this->sections, $this->args);
    }

    public function addSection(AdvancedOptionsInterface $section)
    {
        $this->optionsObjects[] = $section;
        $fields = $section->getFields();

         $sectionFileds = [
            'title'  => $section->getTitle(),
            'desc'   => $section->getDescription(),
            'fields' => $fields,
            'icon'   => $section->getIcon(),
             //'submenu' => false, // Setting submenu to false on a given section will hide it from the WordPress sidebar menu!
        ];

        $this->sections[] = array_merge($sectionFileds, $section->getParams());

        $this->addAvilabledFields($fields);

        $section->addActions();
    }

    /**
     * All the possible arguments for Redux.
     * For full documentation on arguments, please refer to: https://github.com/ReduxFramework/ReduxFramework/wiki/Arguments
     * */
    public function setArguments()
    {
        /**
         * !!! For now Advanced Options submenu item should no be display on admin menu. This should be hidden for all users,
         * but page need shown if user use url for this page
         *
         * On admin css added this style rule
         *
         * #adminmenu .wp-submenu a[href="admin.php?page=hji-advanced-options"] {
         *   display: none;
         * }
         */
        $this->args = [
            //'customizer_only' => true,
            // TYPICAL -> Change these values as you need/desire
            'opt_name'           => $this->reduxOptionName,
            // This is where your data is stored in the database and also becomes your global variable name.
            'display_name'       => 'Advanced Options',
            //'menu_type'          => 'menu',
            'menu_type'            => 'submenu',
            //'menu_type'            => 'hidden',
            //Specify if the admin menu should appear or not. Options: menu or submenu (Under appearance only)
            'allow_sub_menu'     => false,
            // Show the sections below the admin menu item or not
            'menu_title'         => __( 'Advanced Options', 'redux-framework-demo' ),
            'page_title'         => __( 'Advanced Options', 'redux-framework-demo' ),
            // You will need to generate a Google API key to use this feature.
            // Please visit: https://developers.google.com/fonts/docs/developer_api#Auth
            'google_api_key'     => '',
            // Must be defined to add google fonts to the typography module

            'async_typography'   => false,
            // Use a asynchronous font on the front end or font string
            'admin_bar'          => false,
            // Show the panel pages on the admin bar
            'global_variable'    => '',
            // Set a different name for your global variable other than the opt_name
            'dev_mode'           => false,
            // Show the time the page took to load, etc
            'customizer'         => true,
            // Enable basic customizer support

            // OPTIONAL -> Give you extra features
            'page_priority'      => null,
            // Order where the menu appears in the admin area. If there is any conflict, something will not show. Warning.
            //'page_parent'        => 'themes.php',
            'page_parent'        => Membership::$slug,
            // For a full list of options, visit: http://codex.wordpress.org/Function_Reference/add_submenu_page#Parameters
            'page_permissions'   => 'manage_options',
            // Permissions needed to access the options panel.
            'menu_icon'          => '',
            // Specify a custom URL to an icon
            'last_tab'           => '',
            // Force your panel to always open to a specific tab (by id)
            'page_icon'          => 'icon-themes',
            // Icon displayed in the admin panel next to your menu_title
            'page_slug'          => 'hji-advanced-options',
            // Page slug used to denote the panel
            'save_defaults'      => true,
            // On load save the defaults to DB before user clicks save or not
            'default_show'       => false,
            // If true, shows the default value next to each field that is not the default value.
            'default_mark'       => '',
            // What to print by the field's title if the value shown is default. Suggested: *
            'show_import_export' => true,
            // Shows the Import/Export panel when not used as a field.

            // CAREFUL -> These options are for advanced use only
            'transient_time'     => 60 * MINUTE_IN_SECONDS,
            'output'             => true,
            // Global shut-off for dynamic CSS output by the framework. Will also disable google fonts output
            'output_tag'         => true,
            // Allows dynamic CSS to be generated for customizer and google fonts, but stops the dynamic CSS from going to the head
            // 'footer_credit'     => '',                   // Disable the footer credit of Redux. Please leave if you can help it.

            // FUTURE -> Not in use yet, but reserved or partially implemented. Use at your own risk.
            'database'           => '',
            // possible: options, theme_mods, theme_mods_expanded, transient. Not fully functional, warning!

            // HINTS
            'hints'              => [
                'icon'          => 'icon-question-sign',
                'icon_position' => 'right',
                'icon_color'    => 'lightgray',
                'icon_size'     => 'normal',
                'tip_style'     => [
                    'color'   => 'light',
                    'shadow'  => true,
                    'rounded' => false,
                    'style'   => '',
                ],
                'tip_position'  => [
                    'my' => 'top left',
                    'at' => 'bottom right',
                ],
                'tip_effect'    => [
                    'show' => [
                        'effect'   => 'slide',
                        'duration' => '500',
                        'event'    => 'mouseover',
                    ],
                    'hide' => [
                        'effect'   => 'slide',
                        'duration' => '500',
                        'event'    => 'click mouseleave',
                    ],
                ],
            ]
        ];

        // Panel Intro text -> before the form
        if (!isset( $this->args['global_variable'] ) || $this->args['global_variable'] !== false) {
            if (!empty($this->args['global_variable'])) {
                $v = $this->args['global_variable'];
            } else {
                $v = str_replace('-', '_', $this->args['opt_name']);
            }
            $this->args['intro_text'] = '';
        } else {
            $this->args['intro_text'] = '';
        }

        // Add content after the form.
        $this->args['footer_text'] = '';
    }

    public function getOptions()
    {
        $advancedOptions = get_option($this->reduxOptionName);

        foreach ($this->avilabledFields as $field) {
            if (!isset($advancedOptions[$field])) {
                $advancedOptions[$field] = null;
            }
        }

        foreach($advancedOptions as $key => $options) {
            $getterMethod = sprintf('%sFilter', $key);

            foreach($this->optionsObjects as $optionsObject) {
                if (method_exists($optionsObject, $getterMethod)) {
                    $advancedOptions[$key] = $optionsObject->{$getterMethod}($options);
                }
            }
        }

        return $advancedOptions;
    }

    public function getOption($optionName)
    {
        return (isset($this->getOptions()[$optionName])) ? $this->getOptions()[$optionName] : null;
    }

    private function addAvilabledFields($fields)
    {
        foreach ($fields as $field) {
            $this->avilabledFields[] = $field['id'];
        }
    }

    /**
     * @param $pluginSlug
     * @param $gravityFormSlug
     * @return mixed|null
     * Example usage  $gfId = Options::getInstance()->getGfMapping('hjiagentroster', 'testimonials'));
     */
    public function getGfMapping($pluginSlug, $gravityFormSlug)
    {
        $key =  GravityFormsMappingOptions::OPTION_PREFIX . $pluginSlug . '_' . $gravityFormSlug;
        $options = $this->getOptions();

        return $options[$key] ?? null;
    }
}
