<?php

namespace hji\membership\advancedOptions;

use hji\common\gravityform\import\FormImportContextFactory;
use hji\common\gravityform\import\FormImporter;
use hji\common\gravityform\import\GravityFormImport;
use hji\common\utils\GFHelper;

class GravityFormsMappingOptions implements AdvancedOptionsInterface
{
    const PLUGIN_NAME_FIELD = 'pluginName';

    const OPTION_PREFIX = 'gf_form_';

    /**
     * @var false|mixed|void
     */
    public $preImportedFormsOptions;

    public function __construct()
    {
        $gravityFormImport = new GravityFormImport();
        $this->preImportedFormsOptions = $gravityFormImport->getPreimportedOptions();
    }

    public function getTitle()
    {
        $preImportedFormsOptionsTooltip = '';
        if (is_array($this->preImportedFormsOptions) && ($count = count($this->preImportedFormsOptions)) > 0) {
            $preImportedFormsOptionsTooltip = '<span title="Update Available" class="update-plugins count-1 redux-gf-mapping"><span class="update-count">' . $count . '</span></span>';
        }

        return 'Forms';
    }

    public function getDescription()
    {
        return '';
    }

    public function getIcon()
    {
        return 'el el-photo';
    }

    public function getParams(): array
    {
        return [];
    }

    /**
     * @return array
     */
    public function getFields()
    {
        $fields = [];
        $mappingFields = get_option(GravityFormImport::HJI_GF_MAPPING_OPTIONS, []);
        $gravityForms = $this->getGravityForms();

        $mappedFields = [];
        $class = '';
        $preImportedFormData = [];
        $updatePluginOption = false;
        foreach ($mappingFields as $mappingFieldsKey => $mappingFieldData) {
            $formSlugArray = explode(':', $mappingFieldsKey);
            $pluginFullPath = '';

            if (empty($mappingFieldData['pluginPath'])) {
                continue;
            }

            if (!file_exists($mappingFieldData['pluginPath'])
                || (strpos($mappingFieldData['pluginPath'], WP_PLUGIN_DIR) !== false)
            ) {
                $pluginFullPathArray = explode('/plugins/', $mappingFieldData['pluginPath']);
                if (is_array($pluginFullPathArray) && count($pluginFullPathArray) === 2) {
                    $mappingFields[$mappingFieldsKey]['pluginPath'] = $pluginFullPathArray['1'];
                    $updatePluginOption = true;
                } elseif ((is_array($pluginFullPathArray) && count($pluginFullPathArray) === 1)) {
                    $pluginFullPath = WP_PLUGIN_DIR . '/' . $mappingFieldData['pluginPath'];
                } else {
                    continue;
                }
            } else {
                $pluginFullPath = WP_PLUGIN_DIR . '/' . $mappingFieldData['pluginPath'];
            }

            $pluginData = empty($pluginFullPath) ? [] : get_plugin_data($pluginFullPath);
            if (is_array($this->preImportedFormsOptions) && isset($this->preImportedFormsOptions[$mappingFieldsKey])) {
                $class = 'hji-gf-form-update';
                $preImportedFormData[] = [
                    'formTitle' => $mappingFieldData['formTitle'] ?? '',
                    'formVersion' => $this->preImportedFormsOptions[$mappingFieldsKey]['formVersion'] ?? '',
                    'slug' => $mappingFieldsKey,
                    'fieldId' => 'advanced_options-' . self::OPTION_PREFIX . $formSlugArray[0] . '_' . $formSlugArray[1]
                ];
            }

            $pluginActive = true;
            if (!empty($pluginFullPath)) {
                $pluginPathInfo = pathinfo($pluginFullPath);
                $pluginBaseName = basename($pluginPathInfo['dirname']) . '/' . $pluginPathInfo['basename'];
                $pluginActive = is_plugin_active($pluginBaseName);
            }

            $mappedFields[$formSlugArray[0]][] = [
                'id' => self::OPTION_PREFIX . $formSlugArray[0] . '_' . $formSlugArray[1],
                'type' => 'select',
                'title' => $mappingFieldData['formTitle'] ?? '',
                'subtitle' => ($mappingFieldData['formTitle'] ?? '') . ' for ' . ($pluginData['Name'] ?? $formSlugArray[0]),
                'desc' => 'Please select gravity form...',
                'options' => $gravityForms,
                'class' => $class,
                'attributes' => [
                    'autocomplete' => 'off',
                ],
                'pluginActive' => $pluginActive,
                'pluginName' => ($pluginData['Name'] ?? $formSlugArray[0]),
            ];
        }

        if ($updatePluginOption) {
            update_option(GravityFormImport::HJI_GF_MAPPING_OPTIONS, $mappingFields);
        }

        foreach ($mappedFields as $mappingFieldsKey => $mappingFieldData) {
            if (!empty($mappingFieldData[0]['pluginActive'])) {
                $fields[] = [
                    'id' => 'section_' . $mappingFieldsKey,
                    'type' => 'section',
                    'class' => 'hji-gf-mapping-section',
                    'title' => $mappingFieldData[0]['pluginName'],
                    'subtitle' => 'List of gravity forms mapping for ' . $mappingFieldData[0]['pluginName'],
                    'indent' => true,
                ];

                foreach ($mappingFieldData as $formData) {
                    $fields[] = $formData;
                }
            }
        }

        add_action('admin_enqueue_scripts', function() use ($preImportedFormData) {
            $script = "jQuery(function() {
            window.gfFormDataToUpdate =" . json_encode($preImportedFormData) . "
        });";

            wp_add_inline_script('main-admin', $script);
        });

        return $fields;
    }

    /**
     * @return mixed|void
     */
    public function addActions()
    {
        add_action('redux/options/advanced_options/saved', [$this, 'saveOptionsHook'], 10, 3 );
        add_action('redux/options/advanced_options/reset', [$this, 'resetOptions'], 10, 1);
        add_action('redux/options/advanced_options/section/reset', [$this, 'resetOptions'], 10, 1);
    }

    public function resetOptions(\Redux_Panel $options): void
    {
        if (property_exists($options, 'parent') && property_exists($options->parent, 'options')) {
            $this->saveOptionsHook($options->parent->options);
        }
    }

    /**
     * @return array
     */
    protected function getGravityForms()
    {
        if (!GFHelper::gravityFormsExists()) {
            return [];
        }
        $forms = [];
        $formsData = \RGFormsModel::get_forms(true);
        foreach ($formsData as $form) {
            $forms[$form->id] = $form->title;
        }

        return $forms;
    }

    public function saveOptionsHook($values)
    {
        $gravityFormImport = new GravityFormImport();
        $preImportedForms = $gravityFormImport->getPreimportedOptions();

        $mappingFieldsData = get_option(GravityFormImport::HJI_GF_MAPPING_OPTIONS, []);

        if (
            !(is_array($mappingFieldsData) && count($mappingFieldsData) > 0)
            || !(is_array($preImportedForms) && count($preImportedForms) > 0)
        ) {
            return $values;
        }

        foreach ($mappingFieldsData as $pluginSlug => $pluginValues) {
            if (!isset($pluginValues['forms'])) {
                continue;
            }

            foreach ($pluginValues['forms'] as $key => $value) {
                $valuesKey = self::OPTION_PREFIX . $pluginSlug . '_' . $value['form_key'];
                if (array_key_exists($valuesKey, $values)) {
                    $mappingFieldsData[$pluginSlug]['forms'][$key]['form_id'] = $values[$valuesKey];
                }

                $preImportedKey = $pluginSlug . '_' . $value['form_key'];
                if (isset($values[$preImportedKey])) {
                    $action = $values[$preImportedKey];

                    switch ($action) {
                        case '1' : $preImportedFormId = $preImportedForms[$pluginSlug]['forms'][$value['form_key']];
                            $mappingFieldsData[$pluginSlug]['forms'][$key]['form_id'] = $preImportedFormId;
                        break;
                        case '2':
                            $contextFactory = new FormImportContextFactory();
                            $context  = $contextFactory->create($pluginValues[self::PLUGIN_NAME_FIELD]);
                            $importer = new FormImporter($context);
                            $newFormId = $importer->importFile($pluginValues[self::PLUGIN_NAME_FIELD], $preImportedForms[$preImportedKey]['filePath']);
                            if (!empty($newFormId)) {
                                $values[$valuesKey] = $newFormId;
                                unset($preImportedForms[$preImportedKey]);
                                $values[$preImportedKey] = 0;
                                $mappingFieldsData[$pluginSlug]['forms'][$key]['form_id'] = $newFormId;
                            }

                        break;
                    }
                }
            }
        }

        $gravityFormImport->setPreImportedOptions($preImportedForms);

        return $values;
    }
}

