<?php

namespace hji\membership\advancedOptions;

use hji\membership\Membership;
use hji\ResponsiveIDX\models\MarketMetadata;

class ListingStatusesOptions implements AdvancedOptionsInterface
{
    /**
     * @inheritDoc
     */
    public function getTitle()
    {
        return 'IDX Search Filters';
    }

    /**
     * @inheritDoc
     */
    public function getDescription()
    {
        return '';
    }

    /**
     * @inheritDoc
     */
    public function getIcon()
    {
        return '';
    }

    /**
     * @inheritDoc
     */
    public function getFields()
    {
        $fields = [
            [
                'id' => 'listings-options-general-settings',
                'type' => 'section',
                'title' => 'General Settings',
                'subtitle' => 'IDX Search Filters allow to configure relevant search filters available to your website visitors in the front-end. If nothig selected, all options are available by default.<br><br>Market-specific filters can be configured on their respective setting pages on the left hand side.',
                'indent' => false
            ],
            [
                'id' => 'listing-options-listing-types-section',
                'type'=> 'section',
                'title' => 'Listing Types',
                'subtitle' => 'Select listing types you would like to support across all your markets.<br>If nothing selected, all listing types will be supported by default.',
                'indent' => true
            ]
        ];

        $markets = Membership::getInstance()->getMarkets();
        $resultListingTypes = [];
        if (is_array($markets) && count($markets) > 0) {

            foreach ($markets as $market) {
                $listingTypes = MarketMetadata::getInstance($market)->getListingTypes();
                $resultListingTypes = array_merge($resultListingTypes, $listingTypes);
            }
        }

        foreach ($resultListingTypes as $key => $listingType) {
            $fields[] = [
                'id' =>  'global_' . $key .'_listing-type-switch',
                'type' => 'switch',
                'title' => $listingType,
                'subtitle' => '',
                'default'  => '1',
            ];
        }

        $fields[] = [
            'id' => 'listing-options-listing-types-section-end',
            'type'=> 'section',
            'title' => '',
            'subtitle' => '',
            'indent' => false
        ];

        return $fields;

    }

    /**
     * @inheritDoc
     */
    public function addActions()
    {
        add_action('redux/options/advanced_options/saved', [$this, 'saveOptions'], 10, 1);
        add_action('redux/options/advanced_options/reset', [$this, 'resetOptions'], 10, 1);
        add_action('redux/options/advanced_options/section/reset', [$this, 'resetOptions'], 10, 1);
    }

    public function resetOptions(\Redux_Panel $options): void
    {
        if (property_exists($options, 'parent') && property_exists($options->parent, 'options')) {
            foreach ($options->parent->options as $key => $option) {
                $keyParts = explode('_', $key);
                if (isset($keyParts['3']) && $keyParts['3'] === 'listing-type-property') {
                    foreach ($option as $status => $statusValue) {
                        if (strpos(strtolower($status), 'pending') === false) {
                            $options->parent->options[$key][$status] = 1;
                        } else {
                            $options->parent->options[$key][$status] = 0;
                        }
                    }
                }
            }

            $this->saveOptions($options->parent->options);
        }
    }

    public function getParams(): array
    {
        return [];
    }

    /**
     * @param array $values
     */
    public function saveOptions($values): void
    {
        $valueTemp = $values;

        $options = Membership::getInstance()::settings()->getListingFilteringOptions();
        foreach ($valueTemp as $key => $value) {
            if (strpos($key, 'global_') === false) {
                continue;
            }

            $keyParts = explode('_', $key);
            if (is_array($keyParts) && count($keyParts) === 3 && $keyParts[0] === 'global' && $keyParts['2'] === 'listing-type-switch') {
                $options['global-listing-types'][$keyParts[1]] = $value;
            }
        }

        Membership::getInstance()::settings()->setGlobalListingOptions($options);
    }
}
