<?php

namespace hji\membership\advancedOptions;


use hji\membership\Membership;

class MembershipAdvancedOptions implements AdvancedOptionsInterface
{
    public $availableLookupAreaList = [
        'place' => 'Place',
        'township' => 'Township',
        'county' => 'County',
        'neighborhood' => 'Neighborhood',
        'zipcode' => 'Zip Code',
        'school' => 'School',
        'address' => 'Address'
    ];

    public $areaBasisList = [
        'postal-city' => 'Postal Cities',
        'census-place' => 'Census Places',
    ];

    public function getTitle()
    {
        return 'Membership Settings';
    }

    public function getDescription()
    {
        return '';
    }

    public function getIcon()
    {
        return 'el el-home';
    }

    public function getParams(): array
    {
        return [];
    }

    public function getFields()
    {
        return [
            [
                'id' => 'lookupAreaList',
                'type' => 'sortable',
                'title' => 'IDX Area Search',
                'subtitle' => 'Define and reorder these however you want.',
                'desc' => '',
                'mode' => 'checkbox',
                'options' => $this->availableLookupAreaList,
                'default' => [
                    'place' => true,
                    'township' => true,
                    'county' => true,
                    'neighborhood' => true,
                    'zipcode' => true,
                    'school' => true,
                    'address' => true,
                ]
            ],
            [
                'id' => 'areaBias',
                'type' => 'select',
                'title' => 'Area Bias',
                'subtitle' => 'For details see <a href="https://slipstream.homejunction.com/#/ws/areas?id=bias">Area Bias</a> documentation.',
                'desc' => '',
                'options' => $this->areaBasisList,
                'default' => 'postal-city',
            ],
            [
                'id' => 'lookupAreaDefaultPlaceholder',
                'type' => 'text',
                'title' => 'Default Lookup Area Placeholder',
                'subtitle' => 'Set default lookup area placeholder if you need. Optional field',
                'desc' => '',
                'mode' => 'text',
                'default' => 'Neighborhood, City, School, Zip...',
            ],
            [
                'id' => 'licensedInStates',
                'type' => 'select',
                'multi' => true,
                'title' => 'States You’re Licensed In',
                'subtitle' => 'Optional',
                'desc' => 'IDX property search will be limited to these states. This may be useful when MLS coverage exceeds your area of service.',
                'default' => '',
                'options' => $this->getStatesAndDistricts(),
            ],
        ];
    }

    public function addActions()
    {
        add_action('redux/options/advanced_options/saved', [$this, 'saveOptions'], 10, 3);
        add_action('redux/options/advanced_options/reset', [$this, 'resetOptions'], 10, 1);
        add_action('redux/options/advanced_options/section/reset', [$this, 'resetOptions'], 10, 1);
    }

    public function saveOptions($value)
    {
        switch ($value['areaBias']) {
            case 'postal-city':
                Membership::getInstance()->settings()->setPostalCityBias();
                break;
            case 'census-place':
                Membership::getInstance()->settings()->setCensusPlaceBias();
                break;
        }

        $statesLicensedIn = [];
        if (isset($value['licensedInStates']) && is_array($value['licensedInStates'])) {
            $statesLicensedIn = $value['licensedInStates'];
        }

        Membership::getInstance()->settings()->setStateLicensedIn($statesLicensedIn);
    }

    public function resetOptions(\Redux_Panel $options): void
    {
        if (property_exists($options, 'parent') && property_exists($options->parent, 'options')) {
            $this->saveOptions($options->parent->options);
        }
    }

    /**
     * @param array $options
     * @return array
     */
    public function lookupAreaListFilter($options)
    {
        $result = [];

        if (!empty($options)) {
            foreach ($options as $key => $value) {
                if ($value === '1' && isset($this->availableLookupAreaList[$key])) {
                    $result[$key] = $this->availableLookupAreaList[$key];
                }
            }
        }

        // If not selected lookupAreaList then use all available Lookup Area
        if (count($result) < 1) {
            $result = $this->availableLookupAreaList;
        }

        return $result;
    }

    private function getStatesAndDistricts()
    {
        return [
            'AL' => 'Alabama',
            'AK' => 'Alaska',
            'AZ' => 'Arizona',
            'AR' => 'Arkansas',
            'CA' => 'California',
            'CO' => 'Colorado',
            'CT' => 'Connecticut',
            'DE' => 'Delaware',
            'DC' => 'District of Columbia',
            'FL' => 'Florida',
            'GA' => 'Georgia',
            'HI' => 'Hawaii',
            'ID' => 'Idaho',
            'IL' => 'Illinois',
            'IN' => 'Indiana',
            'IA' => 'Iowa',
            'KS' => 'Kansas',
            'KY' => 'Kentucky',
            'LA' => 'Louisiana',
            'ME' => 'Maine',
            'MD' => 'Maryland',
            'MA' => 'Massachusetts',
            'MI' => 'Michigan',
            'MN' => 'Minnesota',
            'MS' =>'Mississippi',
            'MO' => 'Missouri',
            'MT' => 'Montana',
            'NE' => 'Nebraska',
            'NV' => 'Nevada',
            'NH' => 'New Hampshire',
            'NJ' => 'New Jersey',
            'NM' => 'New Mexico',
            'NY' => 'New York',
            'NC' => 'North Carolina',
            'ND' => 'North Dakota',
            'OH' => 'Ohio',
            'OK' => 'Oklahoma',
            'OR' => 'Oregon',
            'PA' => 'Pennsylvania',
            'RI' => 'Rhode Island',
            'SC' => 'South Carolina',
            'SD' => 'South Dakota',
            'TN' => 'Tennessee',
            'TX' => 'Texas',
            'UT' => 'Utah',
            'VT' => 'Vermont',
            'VA' => 'Virginia',
            'WA' => 'Washington',
            'WV' => 'West Virginia',
            'WI' => 'Wisconsin',
            'WY' =>'Wyoming'
        ];
    }
}
