<?php

namespace hji\membership\components\affordability;

class AffordabilityCalcWidget extends \WP_Widget
{
    private $usedWidgetNumber = [];

    public function __construct()
    {
        $widgetOps = [
            'classname' => 'srp_affordabilitycalc',
            'description' => __('HJI Affordability Calculator Widget')
        ];

        $controlOps = [
            'width' => 280,
            'id_base' => 'srp_affordabilitycalc'
        ];

        parent::__construct('srp_affordabilitycalc', __('HJI Affordability Calculator'), $widgetOps, $controlOps);
    }

    private function getRandomWidgetId(): int
    {
        do {
            $number = mt_rand(1, 1000000);
        } while (in_array($number, $this->usedWidgetNumber, true));

        return $number;
    }

    public function widget($args, $instance)
    {
        if (empty($this->number) || in_array($this->number, $this->usedWidgetNumber, true)) {
            $number = $this->getRandomWidgetId();

            $this->usedWidgetNumber[] = $number;
            $this->_set($number);
        } else {
            $this->usedWidgetNumber[] = $this->number;
        }

        //check widget-related variables
        //since we allow to embed widgets into content where these vars don't exist
        $beforeTitle = $args['before_title'] ?? null;
        $afterTitle = $args['after_title'] ?? null;
        $beforeWidget = $args['before_widget'] ?? null;
        $afterWidget = $args['after_widget'] ?? null;

        $width = $args['width'] ?? null;

        $title = apply_filters('srp_AffordabilityCalc', empty($instance['title']) ? '' : $instance['title']);

        if (!empty($title)) {
            $title = $beforeTitle . $title . $afterTitle;
        }

        if (isset($instance['width']) && !empty($instance['width'])) {
            $width = 'style="width:' . $instance['width'] . 'px"';
        }

        $rate = !empty($instance['interest_rate']) ? $instance['interest_rate'] : null;
        $interestRate = $this->getOption('annual_interest_rate', $rate);

        if (!$options = get_option('srp_mortgage_calc_options')) {
            $options = $this->defaultSettingsMortgageCalc();
        }

        $output = $beforeWidget . $title . '
            <div class="srp_AffordabilityCalcwidget" ' . $width . '>
                <table class="srp_table">
                    <tr>
                        <td>
                        Monthly Gross Income </td>
                        <td>$</td>
                        <td>
                            <input 
                                id="' . $this->get_field_id('mo_gross_income') . '"
                                name="' . $this->get_field_name('mo_gross_income') . '"
                                type="text" size="8"
                            >
                        </td>
                    </tr>
                    <tr>
                        <td>Monthly Debt Expenses 
                            <sup>
                                <a 
                                    href="#TB_inline?height=300&width=400&inlineId=' . $this->get_field_id('sac_help') . '"
                                    class="thickbox" 
                                    title="What are Monthly Debt and Obligations?"
                                >
                                    [?]
                                </a>
                            </sup>
                            
                            <div
                                id="' . $this->get_field_id('sac_help') . '"
                                style="display: none"
                            >
                                <strong>Monthly Debt and Obligations Should Include:</strong>
                                <ol>
                                    <li>Monthly Credit Card Payments</li>
                                    <li>Monthly Auto Payments</li>
                                    <li>Monthly Child Support</li>
                                    <li>Monthly Association Fees</li>
                                    <li>Other Monthly Obligations, but NOT utility bills.</li>
                                </ol>
                            </div>
                        </td>
                        <td>$</td>
                        <td>
                            <input
                                id="' . $this->get_field_id('mo_debt_expences') . '"
                                name="' . $this->get_field_name('mo_debt_expences') . '"
                                type="text" size="8"
                            >
                        </td>
                    </tr>
                    <tr>
                        <td>Down Payment:</td>
                        <td>$
                        </td>
                        <td>
                            <input
                                id="' . $this->get_field_id('down_payment') . '"
                                name="' . $this->get_field_name('down_payment') . '"
                                type="text"
                                size="8"
                                value="0"
                            >
                        </td>
                    </tr>
                    <tr>
                        <td>Interest Rate:</td>
                        <td>%</td>
                        <td>
                            <input
                                id="' . $this->get_field_id('interest_rate') . '"
                                name="' . $this->get_field_name('interest_rate') . '2"
                                type="text"
                                size="8"
                                value="' . ($interestRate ?? $options['annual_interest_rate']) . '" 
                            />
                            <input
                                id="' . $this->get_field_id('property_tax') . '"
                                name="' . $this->get_field_name('property_tax') . '2"
                                type="hidden"
                                value="' . $options['property_tax_rate'] . '" 
                            />
                            <input
                                id="' . $this->get_field_id('home_insurance') . '"
                                name="' . $this->get_field_name('home_insurance') . '2"
                                type="hidden"
                                value="' . $options['home_insurance_rate'] . '" 
                            />
                            <input
                                id="' . $this->get_field_id('pmi') . '"
                                name="' . $this->get_field_name('pmi') . '2"
                                type="hidden"
                                value="' . $options['pmi'] . '"
                            />
                        </td>
                    </tr>
                    <tr>
                        <td colspan="3">
                            <div 
                                id="' . $this->get_field_id('result') . '"
                                class="srp_additional-info"
                                style="display: none"
                            >
                            </div>
                        </td>
                    </tr>
                    <tr><td colspan="3">
                  ';
        $output = apply_filters('widget', $output);
        $output .= '</td></tr></table></div> ' . $afterWidget;

        if (isset($instance['return']) && $instance['return'] === true) {
            return $output;
        } else {
            echo $output;
        }
    }

    private function getOption($option, $instance = null)
    {
        $mortgageRatesOptions = get_option('srp_mortgage_rates', null);
        /*--BEGIN return zillow rate--*/
        if ($option === 'annual_interest_rate') {
            if (
                !empty($mortgageRatesOptions['use_rates_in_calcs'])
                && !empty($mortgageRatesOptions['getratesummary_api_key'])
            ) {
                $rate = $this->getZillowMortgageRates(true);
                if ($rate) {
                    add_filter('widget', 'srp_mortgage_rates_branding', 9);

                    return $rate;
                }
            }
        } elseif ($instance !== null) {
            return $instance;
        }

        /*--END return zillow rate--*/

        return null;
    }

    private function getZillowMortgageRates($return_rate = false, $width = '100%')
    {
        $opt = get_option('srp_mortgage_rates');
        $ZWSID = $opt['getratesummary_api_key'];
        $state = $opt['getratesummary_state'];

        $url = "http://www.zillow.com/webservice/GetRateSummary.htm?zws-id=";

        if ($state) {
            $requestUrl = $url . $ZWSID . '&state=' . $state;
        } else {
            $requestUrl = $url . $ZWSID;
        }

        $xml = $this->wpHttpXml($requestUrl);

        if (isset($xml->message->code) && $xml->message->code != 0) {
            exit($xml->message->text);
        } else {
            $loanTypes = [
                '30 Year Fixed',
                '15 Year Fixed',
                '5/1 ARM',
            ];
            $srpDisplayRates = (int)$opt['display_rates'];

            $output = '
        <div class="simpleMortgageCalcwidget" style="width: ' . $width . '">
        <table class="srp_table">';

            if ($srpDisplayRates === 1) {
                $output .= '
                    <tr>
                        <td>&nbsp;</td>
                        <td><div align="right">Today</div></td>
                        <td><div align="right">Last Week</div></td>
                    </tr>
                ';
            }

            foreach ($loanTypes as $k => $v) {
                if (
                    isset($xml->response->today->rate[$k])
                    && $xml->response->today->rate[$k] > $xml->response->lastWeek->rate[$k]
                ) {
                    $change = ' class="srp_rte_up"';
                } else {
                    $change = ' class="srp_rte_down"';
                }

                $rate = '';
                if (isset($xml->response->today->rate[$k])) {
                    $rate = number_format((float)$xml->response->today->rate[$k], 2);
                }
                $output .= '
            <tr>
                <td>' . $v . '</td>
                <td class="srp_mrtg_rte"><span' . $change . '>'
                    . $rate
                    . '%</span></td>';
                //Do not show change (up/down) for the last week rates, since we have nothng to compare to.
                if ($srpDisplayRates === 1 && isset($xml->response->lastWeek->rate[$k])) {
                    $output .= '<td class="srp_mrtg_rte"><span>'
                        . number_format((float)$xml->response->lastWeek->rate[$k], 2) . '%</span></td>';
                }

                $output .= '</tr>';
            }

            $output .= '
          <tr>
            <td colspan="3">
                ' . $this->mortgageRatesBranding() . '
            </td>
          </tr>
        </table>
        </div>';

            //add disclaimer to the footer
            add_action('srp_footer_disclaimers', 'srp_zillow_disclaimer');

            if ($return_rate) {
                return $xml->response->today->rate[0];
            }

            return $output;
        }
    }

    private function wpHttpXml($url)
    {
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_TIMEOUT, 5);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_ENCODING, 'gzip');

        $result = curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        curl_close($ch);

        if (!$code) {
            preg_match('@^(?:http://)?([^/]+)@i', $url, $matches);
            $host = $matches[1];
            preg_match('/[^.]+\.[^.]+$/', $host, $matches);
            $this->debug(__('Something went wrong. No data is being returned from ' . $matches[0] . '.'), $result);

            return null;
        }

        if ($code != 200) {
            $message = 'Request to URL: "' . $url . '" failed. Response code: ' . $code;
            $this->debug(__($message), $result);

            return null;
        }

        $xml = '';

        if (is_string($result)) {
            simplexml_load_string($result);
        }

        return $xml;
    }

    private function debug($message = '', $data = null)
    {

        if (defined('HJI_DEBUG') && HJI_DEBUG && defined('WP_DEBUG_LOG')) {
            if ($message) {
                echo WP_DEBUG_LOG ? $message : "<div class='error'>{$message}</div>";
            }

            /* TODO: should be implement our debug system */
//            if(isset($_GET('hjdebug')) && $data) {
//                pa($data);
//            }
        }
    }

    private function mortgageRatesBranding($output = false)
    {
        $output .= '<div class="srp_attrib zillow" align="center">
                    <a href="http://www.zillow.com/Mortgage_Rates/">See more mortgage rates at Zillow.com</a><br />
                    <a href="http://www.zillow.com/Mortgage_Rates/"><img src="http://www.zillow.com/static/logos/zmm_logo_small.gif" width="145" height="15" alt="Zillow Mortgages" /></a>
                </div>'."\n";

        return $output;
    }

    private function defaultSettingsMortgageCalc(): array
    {
        //All rates are in %
        $options['annual_interest_rate'] = 6;
        $options['property_tax_rate'] = 1;
        $options['home_insurance_rate'] = 0.5;
        $options['pmi'] = 0.5;
        $options['origination_fee'] = 1; //%
        $options['mortgage_term'] = 30;
        $options['lender_fees'] = 600;
        $options['credit_report_fee'] = 50;
        $options['appraisal'] = 300;
        $options['title_insurance'] = 800;
        $options['reconveyance_fee'] = 75;
        $options['recording_fee'] = 45;
        $options['wire_courier_fee'] = 55;
        $options['endorsement_fee'] = 75;
        $options['title_closing_fee'] = 125;
        $options['title_doc_prep_fee'] = 30;

        return $options;
    }

    public function update($new_instance, $old_instance)
    {
        $instance = $old_instance;
        $instance['title'] = strip_tags($new_instance['title']);

        return $instance;
    }

    public function form($instance)
    {
        $instance = wp_parse_args((array)$instance, ['title' => 'Affordability Calculator']);
        $title = strip_tags($instance['title']);
        ?>
        <p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
            <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>"
                   name="<?php echo $this->get_field_name('title'); ?>" type="text"
                   value="<?php echo esc_attr($title); ?>"/></p>

        <?php
    }

    private function pa($x)
    {
        print '<textarea cols="80" rows="10">';
        //print '<pre>';
        print_r($x);
        //print '</pre>';
        print '</textarea>';
    }
}
