<?php
namespace hji\membership\controllers;


use hji\common\utils\GFAccessibility;
use hji\common\utils\GFHelper;
use hji\common\models\GForms as GFormsModel;


class GForms
{
    protected $gFormsModel;


    function __construct()
    {
        $this->gFormsModel = new GFormsModel();
    }


    function setWPActions()
    {
        add_action('init', array($this, 'setActions'));
    }


    function setActions()
    {
        if (!GFHelper::gravityFormsExists())
        {
            return;
        }

        // Admin Filters/Actions

        if (is_admin() && !wp_doing_ajax() && is_user_logged_in())
        {
            add_filter('gform_form_settings', array($this, 'gformFormSettings'), 10, 2);
            add_filter('gform_pre_form_settings_save', array($this, 'saveCustomFormSetting'));
            add_filter('gform_tooltips', array($this, 'formSettingsTooltips'));
            add_action('gform_field_advanced_settings',array($this, "fieldsSettings"), 10, 2);
            add_action('gform_editor_js', array($this, "gfEditorScript"));
        }
        else
        {
            add_filter("gform_field_content",   array($this, "parseDataTagsViaPostObject"), 10, 5);
            add_filter('gform_form_tag', [GFAccessibility::class, 'updateTagForm'], 10, 2);
        }
    }


    function gformFormSettings($settings, $form)
    {
        $settings['Form Basics'][GFormsModel::HJI_FORM_KEY] = "\n" .
            "<tr>\n" .
            "   <th>" .
            "       <label for=\"" . GFormsModel::HJI_FORM_KEY . "\">Form Key</label> \n" .
                    gform_tooltip(GFormsModel::HJI_FORM_KEY, '', true) . "\n" .
            "   </th>\n" .
            "   <td>\n" .
            '        <input value="' . rgar($form, GFormsModel::HJI_FORM_KEY) . '" name="' .
                    GFormsModel::HJI_FORM_KEY . '" class="fieldwidth-3">' . "\n" .
            "    </td>\n" .
            "</tr>\n";
    
        $settings['Form Basics'][GFormsModel::HJI_FORM_VERSION] = "\n" .
            "<tr>\n" .
            "   <th>" .
            "       <label for=\"" . GFormsModel::HJI_FORM_VERSION . "\">Form Version</label> \n" .
            gform_tooltip(GFormsModel::HJI_FORM_VERSION, '', true) . "\n" .
            "   </th>\n" .
            "   <td>\n" .
            '        <input value="' . rgar($form, GFormsModel::HJI_FORM_VERSION) . '" name="' .
            GFormsModel::HJI_FORM_VERSION . '" class="fieldwidth-3" >' . "\n" .
            "    </td>\n" .
            "</tr>\n";

        return $settings;
    }


    function saveCustomFormSetting($form)
    {
        $form[GFormsModel::HJI_FORM_KEY] = rgpost(GFormsModel::HJI_FORM_KEY);
        $form[GFormsModel::HJI_FORM_VERSION] = rgpost(GFormsModel::HJI_FORM_VERSION);

        return $form;
    }


    function formSettingsTooltips($tooltips)
    {
        $tooltips[GFormsModel::HJI_FORM_KEY] = "Form key by which HJI products identify forms. Changing existing key can stop HJI plugins from finding the right forms.";
        $tooltips["form_field_" . GFormsModel::HJI_FIELD_KEY] = "Field key is used by HJI plugins instead of field IDs to identify fields.";

        return $tooltips;
    }


    function fieldsSettings($position, $form_id)
    {
        $form = \GFFormsModel::get_form_meta($form_id);

        if ($this->gFormsModel->isHJIform($form))
        {
            //create settings on position 50 (right after Admin Label)
            if($position == 50){
                ?>
                <li class="<?php echo GFormsModel::HJI_FIELD_KEY; ?> field_setting">
                    <label for="field_<?php echo GFormsModel::HJI_FIELD_KEY; ?>">
                        <?php _e("HJI Field Key", "gravityforms"); ?>
                        <?php gform_tooltip("form_field_" . GFormsModel::HJI_FIELD_KEY) ?>
                    </label>
                    <input type="text" id="field_<?php echo GFormsModel::HJI_FIELD_KEY; ?>" size="30" onkeyup="SetFieldProperty('<?php echo GFormsModel::HJI_FIELD_KEY; ?>', this.value);"/>
                </li>
            <?php
            }
        }
    }


    function gfEditorScript()
    {
        ?>
        <script type='text/javascript'>
            //adding setting to fields of type "text"
            for(var i in fieldSettings)
            {
                fieldSettings[i] += ", .<?php echo GFormsModel::HJI_FIELD_KEY; ?>";
            }

            //binding to the load field settings event to initialize the checkbox
            jQuery(document).bind("gform_load_field_settings", function(event, field, form){
                if (field["<?php echo GFormsModel::HJI_FIELD_KEY; ?>"] != 'undefined')
                {
                    jQuery('#field_<?php echo GFormsModel::HJI_FIELD_KEY; ?>').val(field["<?php echo GFormsModel::HJI_FIELD_KEY; ?>"]);
                }
            });
        </script>
        <?php
    }


    /**
     * Replaces tags like {subdivision.name} or {listing.address.street}
     * with their respective object values as
     * global $post->subdivision->name or $post->listing->address->street etc
     *
     * @param $content
     * @param $field
     * @param $value
     * @param $lead_id
     * @param $form_id
     * @return mixed
     */
    function parseDataTagsViaPostObject($content, $field, $value, $lead_id, $form_id)
    {
        if (is_string($value))
        {
            // If value is JSON - return content
            $json = json_decode($value);

            if (json_last_error() == JSON_ERROR_NONE)
            {
                return $content;
            }
        }

        $content = preg_replace_callback(
            '/\{([-a-zA-z0-9_.^\}]+)\}/',
            function($matches) {
                global $post;

                $parts = explode('.', $matches[1]);

                if (count($parts) == 2)
                {
                    if (isset($post->{$parts[0]}->{$parts[1]}))
                    {
                        return $post->{$parts[0]}->{$parts[1]};
                    }
                }
                else if (count($parts) == 3)
                {
                    if (isset($post->{$parts[0]}->{$parts[1]}->{$parts[2]}))
                    {
                        return $post->{$parts[0]}->{$parts[1]}->{$parts[2]};
                    }
                }

                return $matches[0];

            }, $content);

        return $content;
    }
}
