<?php
namespace hji\membership\controllers;

use hji\common\models\Notices;
use hji\common\utils\HTTP;
use hji\common\utils\StringUtil;
use hji\common\utils\View;
use hji\membership\Membership;
use \hji\membership\models\GoogleApiSettings as M_Settings;


class GoogleApiSettings
{
    private $notices;

    public function __construct()
    {
        $this->notices = new Notices();
    }


    public function setActionHooks()
    {
        add_action('admin_menu',    array($this, 'admin_menu'));
        add_action('admin_init',    array($this, 'registerSettings'));
    }


    /**
     * Sets Page-Specific Action Hooks
     * which will executed only on the given page
     *
     * @param string $page Settings Page ID defined by WP
     */
    protected function setPageHooks($page)
    {
        add_action('load-' . $page, [$this, 'setOptionsUpdatedNotice']);
    }


    public function registerSettings()
    {
        register_setting(M_Settings::KEY, M_Settings::KEY);
    }


    public function admin_menu()
    {
        $page = add_submenu_page(Membership::$slug,
            Membership::NAME . ' | Settings',
            'Google API',
            'manage_options',
            Membership::$slug . '-google-api',
            [$this, 'renderSettingsPage']);

        $this->setPageHooks($page);
    }


    public function renderSettingsPage()
    {
        echo View::render(Membership::$dir . '/classes/views/google-api-settings.phtml',
            array(
                'settings' => new M_Settings(),
                'notices' => $this->notices)
        );
    }


    public function setOptionsUpdatedNotice()
    {
        $updated = HTTP::getParameter('settings-updated');

        if (StringUtil::toBoolean($updated) == true)
            $this->notices->addNotice('Your settings have been saved.');
    }
}
