<?php

namespace hji\membership\extension\postTypes;


use hji\membership\extension\rest\ToREST;


class PostTypeToREST extends ToREST
{
    protected $postType;


    public function setPostType(PostTypeModel $postType)
    {
        $this->postType = $postType;
        return $this;
    }


    public function registerFields()
    {
        if ($this->postType instanceof PostTypeModel)
        {
            foreach ($this->postType->getFields() as $field)
            {
                if ($this->shouldBeExcluded($field))
                    continue;

                $schema = array_merge([
                    'description'   => $field['desc'] ?? null,
                    'type'          => 'string'
                ], $field['rest_field_args'] ?? []);

                $this->registerField(
                    $this->postType->name,
                    $this->postType->removeFieldPrefix($field['id']),
                    $schema
                );
            }
        }
    }


    public function getFieldValue($object, $fieldId, $request, $object_type)
    {
        return get_post_meta($object['id'], $this->postType->applyFieldPrefix($fieldId), true);
    }


    public function updateFieldValue($value, $object, $fieldId, $request, $object_type)
    {
        return update_post_meta($object->ID, $this->postType->applyFieldPrefix($fieldId), $value);
    }


    /**
     * Filters out some field types automatically so they don't get
     * added to the rest fields for an object/post type.
     *
     * @param array $field
     *
     * @return bool
     */
    protected function shouldBeExcluded(array $field)
    {
        // we want a default to add the field, then logic below will remove the field if necessary
        $non_supported_types = [ 'title', 'group' ];
        $isUnsupportedField = in_array($field['type'], $non_supported_types);
        $shouldBeExcluded = (isset($field['show_in_rest']) && $field['show_in_rest'] === false);

        return ($isUnsupportedField || $shouldBeExcluded);
    }
}