<?php
/**
 * @package Redux Extentions
 * @class   Redux_Extension_Address
 * @version 1.0.0
 *
 */

defined( 'ABSPATH' ) || exit;

// Don't duplicate me!
if ( ! class_exists( 'Redux_address', false ) ) {

    /**
     * Main ReduxFramework_options_object class
     *
     * @since       1.0.0
     */
    class Redux_address extends Redux_Field {
        /**
         * Field Constructor.
         * Required - must call the parent constructor, then assign field and value to vars
         *
         * @param array  $field Field array.
         * @param mixed  $value Field values.
         * @param object $parent ReduxFramework pointer.
         */
        public function __construct( array $field, $value, $parent ) {
            parent::__construct( $field, $value, $parent );

            // Set default args for this field to avoid bad indexes. Change this to anything you use.
            $defaults = [
                'options'          => [],
                'stylesheet'       => '',
                'output'           => true,
                'enqueue'          => true,
                'enqueue_frontend' => true,
            ];

            $this->field = wp_parse_args( $this->field, $defaults );
        }

        /**
         * Field Render Function.
         * Takes the vars and outputs the HTML for the field in the settings
         *
         * @return      void
         */
        public function render()
        {
            $defaults = [
                'street' => '',
                'city'   => '',
                'state'  => '',
                'zip'    => ''
            ];

            $this->value = wp_parse_args( $this->value, $defaults );

            echo '<input type="text" id="' . $this->field['id'] . '-street" name="' . $this->field['name'] . $this->field['name_suffix'] . '[street]" placeholder="Street Address..." value="' . esc_attr( $this->value['street'] ) . '" class="regular-text ' . $this->field['class'] . '" style="width: 43em;" />&nbsp;';

            echo '<input type="text" id="' . $this->field['id'] . '-city" name="' . $this->field['name'] . $this->field['name_suffix'] . '[city]" placeholder="City..." value="' . esc_attr( $this->value['city'] ) . '" class="regular-text ' . $this->field['class'] . '" />&nbsp;';

            echo '<select id="' . $this->field['id'] . '-state" data-placeholder="State" name="' . $this->field['name'] . '[state]" class="redux-select-item ' . $this->field['class'] . '" >';
            echo '<option value="">State...</option>';

            foreach ( $this->states() as $k => $v ) {
                $this->stateDropdown($k, $v);
            }

            echo '</select>&nbsp;';

            echo '<input type="text" id="' . $this->field['id'] . '-zip" name="' . $this->field['name'] . $this->field['name_suffix'] . '[zip]" placeholder="Zip Code..." value="' . esc_attr( $this->value['zip'] ) . '" class="regular-text ' . $this->field['class'] . '" style="width: 7.3em;" />&nbsp;';
        }

        /**
         * Enqueue Function.
         * If this field requires any scripts, or css define this function and register/enqueue the scripts/css
         *
         * @return      void
         */
        public function enqueue()
        {

        }

        /**
         * @return array
         */
        protected function states()
        {
            return [
                'AL' => 'Alabama',
                'AK' => 'Alaska',
                'AZ' => 'Arizona',
                'AR' => 'Arkansas',
                'CA' => 'California',
                'CO' => 'Colorado',
                'CT' => 'Connecticut',
                'DE' => 'Delaware',
                'FL' => 'Florida',
                'GA' => 'Georgia',
                'HI' => 'Hawaii',
                'ID' => 'Idaho',
                'IL' => 'Illinois',
                'IN' => 'Indiana',
                'IA' => 'Iowa',
                'KS' => 'Kansas',
                'KY' => 'Kentucky',
                'LA' => 'Louisiana',
                'ME' => 'Maine',
                'MD' => 'Maryland',
                'MA' => 'Massachusetts',
                'MI' => 'Michigan',
                'MN' => 'Minnesota',
                'MS' => 'Mississippi',
                'MO' => 'Missouri',
                'MT' => 'Montana',
                'NE' => 'Nebraska',
                'NV' => 'Nevada',
                'NH' => 'New Hampshire',
                'NJ' => 'New Jersey',
                'NM' => 'New Mexico',
                'NY' => 'New York',
                'NC' => 'North Carolina',
                'ND' => 'North Dakota',
                'OH' => 'Ohio',
                'OK' => 'Oklahoma',
                'OR' => 'Oregon',
                'PA' => 'Pennsylvania',
                'RI' => 'Rhode Island',
                'SC' => 'South Carolina',
                'SD' => 'South Dakota',
                'TN' => 'Tennessee',
                'TX' => 'Texas',
                'UT' => 'Utah',
                'VT' => 'Vermont',
                'VA' => 'Virginia',
                'WA' => 'Washington',
                'WV' => 'West Virginia',
                'WI' => 'Wisconsin',
                'WY' => 'Wyoming'
            ];
        }

        /**
         * @param $id
         * @param $value
         */
        protected function stateDropdown($id, $value)
        {
            if (is_array($this->value)) {
                $selected = (is_array($this->value) && in_array($id, $this->value)) ? ' selected="selected"' : '';
            } else {
                $selected = selected($this->value, $id, false);
            }

            echo '<option value="' . $id . '"' . $selected . '>' . $value . '</option>';
        }
    }

    class_alias('Redux_Address', 'ReduxFramework_Address');
    class_alias('Redux_Address', 'ReduxFramework_Extension_address');
}

