<?php
namespace hji\membership\extension\taxonomies;


/**
 * Class TaxonomyManager (SINGLETON)
 *
 * @package hji\membership\extension\taxonomies
 */
class TaxonomyManager
{
    protected static $toRegister = []; // taxonomies to be registered
    protected static $registered = []; // registered taxonomies
    protected static $_instance;


    function __construct()
    {
        if (!( self::$_instance instanceof self))
            self::$_instance = $this;

        if (!has_action('init', [__CLASS__, 'registerAll']))
            add_action('init', [__CLASS__, 'registerAll']);
    }


    public function registerTaxonomy(Taxonomy $taxonomy, $objectType)
    {
        // check for duplicates
        if ($this->isRegistered($taxonomy, $objectType))
            return;

        // Execute registration if init action hook has been already triggered,
        // otherwise store for scheduled execution.

        if (did_action('init'))
        {
            $this->registerWithWP($taxonomy, $objectType);
        }
        else
        {
            self::$toRegister[$objectType][] = $taxonomy;
        }
    }


    public function register(string $name, string $objectType, array $schema)
    {
        $taxonomy = $this->createTaxonomy($name, $schema);
        $this->registerTaxonomy($taxonomy, $objectType);
    }


    public static function registerAll()
    {
        if (self::$_instance instanceof self)
            self::$_instance->registerTaxonomies();
    }


    public function createTaxonomy(string $name, array $schema)
    {
        return new Taxonomy($name, $schema);
    }


    protected function registerTaxonomies()
    {
        foreach (self::$toRegister as $objectType => $taxonomies)
        {
            foreach ($taxonomies as $taxonomy)
            {
                $this->registerWithWP($taxonomy, $objectType);
            }
        }
    }


    protected function registerWithWP(Taxonomy $taxonomy, $objectType)
    {
        $this->updateLedger($taxonomy, $objectType);

        register_taxonomy($taxonomy->getName(), $objectType, $taxonomy->getSchema());
        register_taxonomy_for_object_type($taxonomy->getName(), $objectType);

        if ($taxonomy->showInRest())
            $this->showInRest($taxonomy, $objectType);
    }

    /**
     * @codeCoverageIgnore
     */
    protected function showInRest($taxonomy, $objectType)
    {
        $schema = $taxonomy->getSchema();
        (new TaxonomyToREST())->registerField(
            $objectType,
            $taxonomy->getName(),
            $schema['rest_field_args'] ?? null);
    }


    protected function updateLedger($taxonomy, $objectType)
    {
        // Remove form $toRegister
        if ($taxonomies = self::$toRegister[$objectType] ?? null)
        {
            if (($i = array_search($taxonomy, $taxonomies)) !== false)
                unset(self::$toRegister[$objectType][$i]);
        }

        // Insert into $registered
        self::$registered[$objectType][] = $taxonomy;
    }


    protected function isRegistered($taxonomy, $objectType)
    {
        $registered = false;

        if ($taxonomies = self::$registered[$objectType] ?? null)
            $registered = (array_search($taxonomy, $taxonomies) !== false);

        return $registered;
    }
}