<?php

namespace hji\membership\extension\taxonomies;


use hji\membership\extension\rest\ToREST;


class TaxonomyToREST extends ToREST
{
    public function registerField(string $objectType, string $fieldId, array $args = null)
    {
        $defaultSchema = [
            'type'  => 'array',
            'items' => [
                'type' => 'string'
            ]
        ];

        $args['schema'] = array_merge($args['schema'] ?? [], $defaultSchema);

        parent::registerField($objectType, $fieldId, $args);
    }


    public function getFieldValue($object, $fieldId, $request, $object_type)
    {
        return get_the_terms($object['id'], $fieldId);
    }


    public function updateFieldValue($value, $object, $fieldId, $request, $object_type)
    {
        $taxonomy = $fieldId;
        $updated_term_ids = array(); // will contain an array of term IDs to be updated

        if (is_array($value))
        {
            foreach($value as $term)
            {
                // let's capitalize the title and turn it into a slug
                $term_name = ucwords($term);
                $term_slug = sanitize_title_with_dashes($term, '', 'save');

                // do we already have a term in the database?
                if ($term_id = get_term_by('slug', $term_slug, $taxonomy))
                {
                    $updated_term_ids[] = (int)$term_id->term_id;
                }
                // then let's create one
                else
                {
                    // @codeCoverageIgnoreStart
                    $new_term = wp_insert_term($term_name, $taxonomy, array('slug' => $term_slug));
                    // @codeCoverageIgnoreEnd
                    $updated_term_ids[] = $new_term['term_id'];
                }
            }
        }

        wp_set_post_terms($object->ID, $updated_term_ids, $taxonomy, false);
    }
}