<?php

namespace hji\membership\includes;


class PHPVersionRequirement
{
    public $requiredVersion = '7.1';
    public $currentVersion;
    protected $phpVersionOk;


    public function __construct()
    {
        $this->currentVersion = phpversion();
        $this->phpVersionOk = version_compare($this->currentVersion, $this->requiredVersion, '>=');
    }


    public function setActionHooks()
    {
        add_action('after_plugin_row', [$this, 'displayInlineWarningsInPluginsDashboard'], 10, 3);
        add_action('admin_notices', [$this, 'displayGlobalWarningAsAdminNotice']);
        add_filter('upgrader_pre_download', [$this, 'preventPluginUpgrade'], -1, 3);
    }


    public function displayInlineWarningsInPluginsDashboard($plugin_file, $plugin_data, $status)
    {
        if (substr($plugin_file, 0, 4) == 'hji-' && !$this->phpVersionOk)
        {
            $current = get_site_transient( 'update_plugins' );
            if ( ! isset( $current->response[ $plugin_file ] ) ) {
                return false;
            }

            $response = $current->response[ $plugin_file ];

            if ( is_network_admin() ) {
                $active_class = is_plugin_active_for_network( $plugin_file ) ? ' active' : '';
            } else {
                $active_class = is_plugin_active( $plugin_file ) ? ' active' : '';
            }

            echo '<tr class="plugin-update-tr' . $active_class . '" id="' . esc_attr( $response->slug . '-update' ) . '" data-slug="' . esc_attr( $response->slug ) . '" data-plugin="' . esc_attr( $plugin_file ) . '">';
            echo '<td colspan="3" class="plugin-update colspanchange"><div class="update-message notice inline notice-error notice-alt"><p><strong>';
            printf('PHP Version %1$s or newer is required to install latest Home Junction updates. Your current PHP version is %2$s.', $this->requiredVersion, $this->currentVersion);
            echo '</strong></p></div></td>';
            echo '</tr>';

            remove_action("after_plugin_row_{$plugin_file}", 'wp_plugin_update_row', 10, 2);
        }
    }


    public function displayGlobalWarningAsAdminNotice()
    {
        $screens = ['plugins', 'update-core', 'hji-membership_page_hji-membership-downloads', 'plugin-install'];
        $screen = get_current_screen();

        if (!in_array($screen->base, $screens) || $this->phpVersionOk)
            return;

        $class = 'notice notice-error';
        $message = sprintf('PHP Version %1$s or newer is required to install latest Home Junction updates. Your current PHP version is %2$s.', $this->requiredVersion, $this->currentVersion);

        printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));
    }


    public function preventPluginUpgrade($reply, $package, \WP_Upgrader $wp_upgrader)
    {
        if ((is_string($package) && stristr($package, '/hji-')) && !$this->phpVersionOk)
        {
            $message = sprintf('PHP Version %1$s or newer is required to install latest Home Junction updates. Your current PHP version is %2$s.', $this->requiredVersion, $this->currentVersion);
            return new \WP_Error(403, $message);
        }
        return $reply;
    }
}