<?php
namespace hji\membership\models;


use hji\common\utils\APIClient as APIClientUtil;
use hji\membership\Membership;
use hji\membership\utils\Environment;

class APIClientModel extends APIClientUtil
{
    protected $apiController;
    private static $nr_api_base = 'https://national-reports.homejunction.com';
    private static $nr_api_base_test = 'https://national-reports-test.homejunction.com';
    private static $tmp;


    function __construct(Membership $membership, \hji\membership\controllers\APIClient $apiController)
    {
        parent::__construct($membership);

        $this->apiController = $apiController;
    }


    private function swapAPI()
    {
        if ($this->api_base == $this->getNRApiUrl())
        {
            $this->api_base = self::$tmp;
        }
        else
        {
            self::$tmp = $this->api_base;
            $this->api_base = $this->getNRApiUrl();
        }
    }


    function createReport($args)
    {
        $args['object'] = true;
        $this->swapAPI();

        $result = $this->makeAPIRequest("GET", "/report/create",$args, array(), $auth = false);

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function getReport($args)
    {
        $this->swapAPI();

        $result = $this->makeAPIRequest("GET", "/report/" . $args['id'], array('object' => true), array(), $auth = false);

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function getUserReports($args)
    {
        $this->swapAPI();

        $result = $this->makeAPIRequest("GET", "/user/reports", array('object' => true), array(), $auth = false);

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function saveUserReport($args)
    {
        $this->swapAPI();

        $result = $this->makeAPIRequest("GET", "/report/{$args['reportId']}/associate", array('object' => true), array(), $auth = false);

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function deleteUserReport($args)
    {
        $this->swapAPI();

        $result = $this->makeAPIRequest("GET", "/report/{$args['reportId']}/dissociate", array('object' => true), array(), $auth = false);

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function subscribeToReport($args)
    {
        $this->swapAPI();

        $params = array('subscribed' => true, 'object' => true);

        if (isset($args['reportUrl']) && !empty($args['reportUrl']))
        {
            $params['reportUrl'] = $args['reportUrl'];
        }

        $result = $this->makeAPIRequest(
            "GET", "/report/{$args['reportId']}/subscribe", $params, array(), $auth = false
        );

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function unsubscribeFromReport($args)
    {
        $this->swapAPI();

        $result = $this->makeAPIRequest(
            "GET", "/report/{$args['reportId']}/unsubscribe",
            array('subscribed' => false, 'object' => true), array(), $auth = false
        );

        $this->swapAPI();

        if (!isset($result->success) || $result->success != true)
        {
            return false;
        }

        return $result->result;
    }


    function printReport($args)
    {
        $this->swapAPI();

        $result = $this->makeAPIRequest("GET", "/report/{$args['reportId']}/export", array('object' => true, 'token' => $this->last_token), array(), $auth = false);
        header('Content-type: application/pdf');
        echo $result;
        die();
    }


    function makeAPIRequest($method, $uri, $args = array(), $data = array(), $is_auth_request = false, $a_retry = false)
    {
        $success = false;
        $object = false;

        $_args = $args;

        if (isset($args['object']))
        {
            if ($args['object'] === true)
            {
                $object = true;
            }

            unset($_args['object']);
        }

        // check if token is active

        if (!$is_auth_request)
        {
            curl_setopt($this->ch, CURLOPT_HTTPHEADER,
                array("{$this->token_header}: {$this->last_token}"));
        }


        if (!is_array($_args))
        {
            $_args = array();
        }

        $http_parameters = http_build_query($_args);

        $post_body = '';

        if (is_array($data)) {
            $data = http_build_query($data);
        }

        if ($method == "POST" && !empty($data))
        {
            // the request is to post some JSON data back to the API (like adding a contact)
            $post_body = $data;
        }

        // start putting the URL parts together

        $full_url = $this->api_base . $uri;

        if (!empty($http_parameters))
        {
            $full_url .= '?' . $http_parameters;
        }

        $request_headers = "";

        curl_setopt($this->ch, CURLOPT_URL, $full_url);


        if ($method == "POST")
        {
            // put the built parameter key/values as the body of the POST request

            $request_headers .= "Content-Type: application/json\r\n";
            curl_setopt($this->ch, CURLOPT_POST, true);
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, $post_body);
        }
        elseif ($method == "DELETE")
        {
            curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, "DELETE");
            curl_setopt($this->ch, CURLOPT_POSTFIELDS, $post_body);
            curl_setopt($this->ch, CURLOPT_FOLLOWLOCATION, 1);
            //curl_setopt($this->ch, CURLOPT_HEADER, 0);
        }
        else
        {
            curl_setopt($this->ch, CURLOPT_POST, 0);
            curl_setopt($this->ch, CURLOPT_HTTPGET, true);
        }

        // clear previous error messages
        $this->last_error_mess = null;
        $this->last_error_code = null;

        $response = curl_exec($this->ch);
        $info = curl_getinfo($this->ch);

        do_action('hji_api_request', $info, $response);

        // If license key invalid, reset user session
        if ($info['http_code'] == '401')
        {
            UserSession::destroy();
        }

        if ($info['content_type'] == 'application/pdf')
        {
            return $response;
        }

        @list($header, $response_body) = explode("\r\n\r\n", $response, 2);

        // Debug request

        $this->_debugRequest($post_body, $header, $response_body, $info, $uri);

        // Returns result or false

        if ($object === true)
        {
            $result = $this->_handleObject($response_body, $info, $uri, $is_auth_request);
        }
        else
        {
            $result = $this->_handleResponse($response_body, $info, $uri, $is_auth_request);
        }

        if ($result == true)
        {
            return $result;
        }
        else if (($a_retry == false) && ($is_auth_request == false) && (($info['http_code'] == '403') || ($info['http_code'] == '404') || ($info['http_code'] == '500')))
        {
            $this->apiController->authenticate(true);
            $return = $this->makeAPIRequest($method, $uri, $args, $data, $is_auth_request, $a_retry = true);

            return $return;
        }
        else
        {
            return false;
        }

    }


    /**
     * Returns National Reports API Url (supports test and production)
     *
     * @todo - this should never be protected or public
     *       - the whole model should be re-written correctly
     *       - NR API should extend Slipstream API and override URLs
     *       - it also should be invoked on its own, instead of sharing with Slipstream.
     */
    private function getNRApiUrl()
    {
        if (Environment::is(Environment::DEV))
        {
            return self::$nr_api_base_test;
        }

        return self::$nr_api_base;
    }

}
