<?php


namespace hji\membership\models;

/**
 * Class DependencyPluginEntity
 * @package hji\membership\models
 */
class DependencyPluginEntity extends DependencyEntity
{

    /**
     * @var bool
     */
    public $activated;

    /**
     * @var array
     */
    protected $data = [];

    /**
     * DependencyPluginEntity constructor.
     * @param $name
     * @param $version
     * @param $type
     */
    function __construct($name, $version, $type)
    {
        parent::__construct($name, $version, $type);
        $this->data = $this->getPluginByName($this->name);
    }

    /**
     * @return mixed|string
     */
    protected function getPluginSlug()
    {
        return $this->data['slug'] ?? sanitize_title($this->name);
    }

    /**
     * @return mixed|null
     */
    public function getPluginPath()
    {
        return $this->data['pluginPath'] ?? null;
    }

    /**
     * @return bool|mixed
     */
    public function getPluginVersion()
    {
        return $this->data['Version'] ?? false;
    }

    /**
     * @return string|void
     */
    public function getPluginActivationLink()
    {
        global $status, $page, $s;
        if (current_user_can('activate_plugin', $this->data['pluginPath'])) {

            return sprintf(
                '<a href="%s" id="activate-%s" class="edit" aria-label="%s">%s</a>',
                wp_nonce_url(
                    'plugins.php?action=activate&amp;plugin=' . urlencode($this->data['pluginPath']) .
                    '&amp;plugin_status=' . $status . '&amp;paged=' . $page . '&amp;s=' . $s,
                    'activate-plugin_' . $this->data['pluginPath']
                ),
                esc_attr($this->getPluginSlug()),
                esc_attr(sprintf(_x('Activate %s', 'plugin'), $this->name)),
                __('Activate')
            );
        }

        return __('not activated');
    }

    /**
     * @return string|void
     */
    public function getPluginInstallLink()
    {
        if (current_user_can('install_plugins') || current_user_can('update_plugins')) {
            $pluginSlug = $this->getPluginSlug();
            $url = wp_nonce_url(self_admin_url('update.php?action=install-plugin&plugin=' . $pluginSlug), 'install-plugin_' . $pluginSlug) . '&isHJPlugin=1';

            return sprintf(
                '<a class="edit" data-slug="%s" href="%s" aria-label="%s" data-name="%s">%s</a>',
                esc_attr($pluginSlug),
                esc_url($url),
                esc_attr(sprintf(_x('Install %s now', 'plugin'), $this->name)),
                esc_attr($this->name),
                __('Install')
            );
        }

        return __('is not installed');
    }

    /**
     * @return array[]
     */
    protected function getPlugins()
    {
        if ( ! function_exists( 'get_plugins' ) ) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }

        return get_plugins();
    }

    /**
     * @param $name
     * @return array
     */
    protected function getPluginByName($name)
    {
        $plugins = $this->getPlugins();

        foreach ($plugins as $pluginKey => $plugin)
        {
            if (isset($plugin['Name']) && $plugin['Name'] == $name)
            {
                $plugin['pluginPath'] = $pluginKey;

                return $plugin;
            }
        }
    }
}