<?php


namespace hji\membership\models;


class MostPopulatedCity
{

    const OPTION = 'hji_most_populated_city';

    protected $cityObject;

    /**
     * MostPopulatedCity constructor.
     */
    public function __construct()
    {
        $this->cityObject = get_option(self::OPTION);
        if (empty($this->cityObject)) {
            $this->cityObject = $this->getDataFromApi();
            update_option(self::OPTION, $this->cityObject);
        }
    }

    /**
     * @return array
     */
    public function getCityObject()
    {
        return $this->cityObject;
    }

    /**
     * @return mixed
     */
    public function getId()
    {
        return $this->cityObject['id'];
    }

    /**
     * @return mixed
     */
    public function getName()
    {
        return $this->cityObject['name'];
    }

    /**
     * @return mixed
     */
    public function getCentroid()
    {
        return $this->cityObject['centroid'];
    }

    public function jsVar()
    {
        $jsVar = new \stdClass();
        $jsVar->id = $this->getId();
        $jsVar->name = $this->getName();
        $jsVar->centroid = $this->getCentroid();

        return $jsVar;
    }

    /**
     * @return array or bool
     */
    protected function getDataFromApi()
    {
        $membership = \hji\membership\Membership::getInstance();
        $markets = $membership->getMarkets();
        $_markets = [];
        foreach ($markets as $market) {
            $_markets[] = '@' . $market;
        }
        $params = [
            [
                'uri' => '/ws/areas/search',
                'parameters' => [
                    'polygon' => implode(',',$_markets),
                    'geoType' => 'area/census-place,area/postal-city',
                    'details' => true,
                    'sortField' => 'population',
                    'sortOrder' => 'desc',
                    'limit' => 1
                ]
            ],
            [
                'uri' => '/ws/areas/get',
                'parameters' => [
                    'id' => '{$.areas[0].id}',
                    'details' => true,
                ]
            ]
        ];
        $result = $membership->apiController->api->call($params);
        if (empty($result['responses'][1]['result']['areas'])) {
            return false;
        }

        return $result['responses'][1]['result']['areas'][0];
    }
}
