<?php
namespace hji\membership\observers;


use hji\membership\entities\Event;
use SplObserver;


class EventDispatcher implements \SplSubject
{
    private $observers;
    private $event;

    /**
     * (PHP 5 &gt;= 5.1.0)<br/>
     * Attach an SplObserver
     *
     * @link http://php.net/manual/en/splsubject.attach.php
     * @param SplObserver $observer <p>
     * The <b>SplObserver</b> to attach.
     * </p>
     * @return void
     */
    public function attach(\SplObserver $observer): void
    {
        if (!$this->hasObserver($observer))
        {
            $this->observers[] = $observer;
        }
    }


    public function hasObserver(\SplObserver $observer): bool
    {
        if (array_search($observer, (array)$this->observers) === false)
        {
            return false;
        }
        else
        {
            return true;
        }
    }


    /**
     * (PHP 5 &gt;= 5.1.0)<br/>
     * Detach an observer
     *
     * @link http://php.net/manual/en/splsubject.detach.php
     * @param SplObserver $observer <p>
     * The <b>SplObserver</b> to detach.
     * </p>
     * @return void
     */
    public function detach(\SplObserver $observer): void
    {
        $key = array_search($observer,$this->observers, true);

        if (false !== $key)
        {
            unset($this->observers[$key]);
        }
    }


    /**
     * (PHP 5 &gt;= 5.1.0)<br/>
     * Notify an observer
     *
     * @link http://php.net/manual/en/splsubject.notify.php
     * @return void
     */
    public function notify(): void
    {
        foreach ($this->observers as $value)
        {
            $value->update($this);
        }
    }


    private function _setEvent($event)
    {
        $this->event = $event;
    }


    public function getEvent()
    {
        return $this->event;
    }


    public function dispatchEvent(Event $event)
    {
        $this->_setEvent($event);
        $this->notify();
    }
}
