<?php
namespace hji\membership\rendering;

use hji\membership\Membership;
use Timber\Loader;
use Timber\Timber;


class twigRenderer implements TemplateRendererInterface
{
    protected static $timber;
    protected static $flushCache = false;


    public function __construct()
    {
        self::bootstrapTimber();
    }


    public static function setActionHooks()
    {
        // Sets cache location directory

        add_filter('timber/cache/location',     array(__CLASS__, 'getCacheDir'));

        // Sets "flush cache" indicator on all types of updates: plugin, theme, core, language

        add_action('upgrader_process_complete', array(__CLASS__, 'scheduleFlush'));
        add_action('after_switch_theme',        array(__CLASS__, 'scheduleFlush'));
        add_action('activated_plugin',          array(__CLASS__, 'scheduleFlush'));
        add_action('deactivated_plugin',        array(__CLASS__, 'scheduleFlush'));

        // Removes cache files on shutdown

        add_action('shutdown',                  array(__CLASS__, 'performScheduledFlush'));
    }


    public function render($template, array $vars = null)
    {
        if (!self::$timber)
            self::$timber = new Timber();

        if (defined('HJI_TIMBER_CACHE_ENABLE') && HJI_TIMBER_CACHE_ENABLE === false) {
            return Timber::compile($template, $vars, false, Loader::CACHE_NONE);
        }

        return Timber::compile($template, $vars);
    }


    public function setLocations(array $locations = null)
    {
        Timber::$locations = $locations;
    }


    /**
     * Added this public method instead of a direct anonymous function
     * within the filter hook  due to PHPUnit failure (Serialization of 'Closure' is not allowed)
     *
     * @return string
     */
    public static function getCacheDir()
    {
        return WP_CONTENT_DIR . '/hji-cache/twig';
    }


    protected static function bootstrapTimber()
    {
        if (!defined('TIMBER_LOC'))
            define('TIMBER_LOC', Membership::$dir . '/third-party/timber');

        Timber::$cache = true;
    }


    public static function scheduleFlush()
    {
        self::$flushCache = true;
    }


    public static function performScheduledFlush()
    {
        if (self::$flushCache === true)
        {
            self::flush();
            self::$flushCache = false;
        }
    }


    public static function flush()
    {
        self::bootstrapTimber();

        $loader = new Loader();
        $loader->clear_cache_timber();
        $loader->clear_cache_twig();
    }
}
