<?php

namespace hji\membership\services;

use Dice\Dice;
use hji\membership\Membership;
use hji\membership\services\diceBackwardCompatibility\Instance;
use hji\membership\services\diceBackwardCompatibility\Rule;

class DiceWrapper
{
    private $dice;

    public function __construct()
    {
        $this->dice = new Dice();
    }

    public function addRule(string $name, array $rule): self
    {
        $this->dice = $this->dice->addRule($name, $rule);

        return $this;
    }

    public function addRules($rules): self
    {
        $this->dice = $this->dice->addRules($rules);

        return $this;
    }

    public function getRule(string $name): array
    {
        return $this->dice->getRule($name);
    }

    public function create(string $name, array $args = [], array $share = [])
    {
        return $this->dice->create($name, $args, $share);
    }
}

class DependencyInjectorInterface
{
    private static $DI;

    public function __construct()
    {
        //TODO move to bootstrap
        require_once(Membership::$dir . '/third-party/Dice/Dice.php');
    }

    private function getDi()
    {
        if (!self::$DI) {
            self::$DI = new DiceWrapper();
        }

        return self::$DI;
    }

    /**
     * Creates new instance of an object
     *
     * @param       $component
     * @param array $args
     * @param array $share
     * @return object
     */
    public function create($component, array $args = [], array $share = [])
    {
        return ($this->getDi())->create($component, $args, $share);
    }


    /**
     * Returns an instance of an object
     * Singleton.
     */
    public function get($component, array $args = [], $share = [])
    {
        $di = $this->getDi();

        // Check if component already has a rule
        $component = trim($component, '\\');

        // Since getRule returns a new rule if there's no existing one
        // we should detect if it's just a blank rule
        $rule = $di->getRule($component);

        if (count($rule) === 0) {
            $di->addRule($component, ['shared' => true]);
        }

        return $di->create($component, $args, $share);
    }

    public function addRule($name, $rule)
    {
        if ($rule instanceof Rule) {
            $newRule = (array)$rule;
        } else {
            $newRule = $rule;
        }

        $di = $this->getDi();
        $di->addRule($name, $newRule);
    }

    /**
     * Creates new rule for dependency injection
     *
     * @return Rule
     */
    public function createRule()
    {
        return new Rule();
    }

    /**
     * @param $name
     * @return Instance
     */
    public function instance($name)
    {
        return new Instance($name);
    }
}
