<?php
namespace hji\membership\services;
use hji\membership\Membership;

/**
 * Required and recommended plugins
 * Uses TGM Plugin Activation Class
 * https://github.com/thomasgriffin/TGM-Plugin-Activation
 */
final class MembershipRequirePlugins
{

    protected static $menu_slug = 'install-required-plugins';

    function __construct()
    {
        if (is_admin()) {
            require_once dirname( __FILE__ ) . '/../includes/TGM_Plugin_Activation.php';

            $this->registerActions();
        }
    }

    /**
     *
     */
    protected function registerActions()
    {
        add_action( 'tgmpa_register', [$this, 'loadTGM'] );
    }


    /**
     * Loads TGM Plugin Activate/Recommended/Required Class
     */
    public function loadTGM()
    {
        // loads the tgm class
        tgmpa(self::registerPlugins(), self::TGMConfig());
    }


    /**
     * Register plugins that are recommended or required by the theme.
     * Add additional plugins to the $plugin array
     *
     * @return array $plugins
     */
    public static function registerPlugins()
    {
        /**
         * Array of plugin arrays. Required keys are name and slug.
         * If the source is NOT from the .org repo, then source is also required.
         */
        return [
            // gravity forms is required also
            [
                  'name'               => 'Gravity Forms',
                  'slug'               => 'gravityforms',
                  'source'             => 'https://github.com/wp-premium/gravityforms/archive/master.zip',
                  'required'           => true,
                  'external_url'       => '', // If set, overrides default API URL and points to an external URL.
            ],
        ];
    }

    /**
     * Sets the TGM Config
     *
     * @return array $config
     */
    public static function TGMConfig()
    {
        /**
         * Array of configuration settings. Amend each line as needed.
         * If you want the default strings to be available under your own theme domain,
         * leave the strings uncommented.
         * Some of the strings are added into a sprintf, so see the comments at the
         * end of each line for what each argument will be.
         */
        $config   = [
            'domain'           => 'hji-membership-textdomain',
            'id'               => 'hji-membership',
            'default_path'     => '',// Default absolute path to pre-packaged plugins
            'parent_slug'      => 'plugins.php',// Default parent menu slug
            'menu'             => self::$menu_slug,// Menu slug
            'has_notices'      => true,// Show admin notices or not
            'is_automatic'     => true,// Automatically activate plugins after installation or not
            'message'          => '',// Message to output right before the plugins table
            'strings'          => [
                'page_title'                      => __( 'Install Required Plugins', 'hji-membership-textdomain' ),
                'menu_title'                      => __( 'Install Plugins', 'hji-membership-textdomain' ),
                'installing'                      => __( 'Installing Plugin: %s', 'hji-membership-textdomain' ), // %1$s = plugin name
                'oops'                            => __( 'Something went wrong with the plugin API.', 'hji-membership-textdomain' ),
                'notice_can_install_required'     => _n_noop( 'HJI plugins require the following plugin: %1$s.', 'Some plugin requires the following plugins: %1$s.' ), // %1$s = plugin name(s)
                'notice_can_install_recommended'  => _n_noop( 'HJI plugins recommend the following plugin: %1$s.', 'Some plugin recommends the following plugins: %1$s.' ), // %1$s = plugin name(s)
                'notice_cannot_install'           => _n_noop( 'Sorry, but you do not have the correct permissions to install the %s plugin. Contact the administrator of this site for help on getting the plugin installed.', 'Sorry, but you do not have the correct permissions to install the %s plugins. Contact the administrator of this site for help on getting the plugins installed.' ), // %1$s = plugin name(s)
                'notice_can_activate_required'    => _n_noop( 'The following required plugin is currently inactive: %1$s.', 'The following required plugins are currently inactive: %1$s.' ), // %1$s = plugin name(s)
                'notice_can_activate_recommended' => _n_noop( 'The following recommended plugin is currently inactive: %1$s.', 'The following recommended plugins are currently inactive: %1$s.' ), // %1$s = plugin name(s)
                'notice_cannot_activate'          => _n_noop( 'Sorry, but you do not have the correct permissions to activate the %s plugin. Contact the administrator of this site for help on getting the plugin activated.', 'Sorry, but you do not have the correct permissions to activate the %s plugins. Contact the administrator of this site for help on getting the plugins activated.' ), // %1$s = plugin name(s)
                'notice_ask_to_update'            => _n_noop( 'The following plugin needs to be updated to its latest version to ensure maximum compatibility with this theme: %1$s.', 'The following plugins need to be updated to their latest version to ensure maximum compatibility with this theme: %1$s.' ), // %1$s = plugin name(s)
                'notice_cannot_update'            => _n_noop( 'Sorry, but you do not have the correct permissions to update the %s plugin. Contact the administrator of this site for help on getting the plugin updated.', 'Sorry, but you do not have the correct permissions to update the %s plugins. Contact the administrator of this site for help on getting the plugins updated.' ), // %1$s = plugin name(s)
                'install_link'                    => _n_noop( 'Begin installing plugin', 'Begin installing plugins' ),
                'activate_link'                   => _n_noop( 'Activate installed plugin', 'Activate installed plugins' ),
                'return'                          => __( 'Return to Required Plugins Installer', 'hji-membership-textdomain' ),
                'plugin_activated'                => __( 'Plugin activated successfully.', 'hji-membership-textdomain' ),
                'complete'                        => __( 'All plugins installed and activated successfully. %s', 'hji-membership-textdomain' ), // %1$s = dashboard link
                'nag_type'                        => 'notice-error' // Determines admin notice type - can only be 'updated' or 'error'
            ]
        ];

        return $config;
    }

}
