<?php
namespace hji\membership\services;


class Router extends \hji\membership\extension\abstracts\Component
{
    protected $table;
    protected $mounted = false;
    protected $request;


    public function __construct(Request $request)
    {
        // Important to call it here, because Requests will set itself up
        $this->request = $request;
    }


    /**
     * @codeCoverageIgnore
     */
    public function mount()
    {
        if (!$this->mounted)
        {
            add_filter('query_vars',    array($this, 'addQueryVars'));
            add_action('init',          array($this, 'registerRewriteRules'), 0);

            $this->mounted = true;
        }

//        add_action('pre_get_posts', array($this, 'swapQuery'));
//        add_filter('posts_request', array($this, 'clearQuery'));
    }


    public function add($model, $view, $controller, $rewriteRule = null, $expires = false)
    {
        $this->table[] = new Route($model, $view, $controller, $rewriteRule, $expires);
    }


    public function getRoute()
    {
        if (empty($this->request) || !$query = $this->request->getQuery())
        {
            trigger_error('Query is not set.', E_USER_ERROR);
        }
        else if (!empty($this->table)    &&
            $query->matched_rule    &&
            $query->matched_query   &&
            isset($query->query_vars['hji-controller']))
        {
            foreach ($this->table as $route)
            {
                if ($route->rewriteMatch == $query->matched_rule)
                {
                    return $route;
                }
            }
        }

        return false;
    }


    /**
     * @codeCoverageIgnore
     * @param $queryVars
     * @return array
     */
    function addQueryVars($queryVars)
    {
        $queryVars[] = 'hji-controller';
        $queryVars[] = 'hji-action';
        $queryVars[] = 'hji-params';

        return $queryVars;
    }


    function registerRewriteRules()
    {
        if (!empty($this->table))
        {
            foreach ($this->table as $route)
            {
                if ($route->rewriteMatch && $route->rewriteQuery)
                {
                    $this->_addRewriteRule($route->rewriteMatch, $route->rewriteQuery, 'top');
                }
            }
        }
    }


    /**
     * @codeCoverageIgnore
     * @param        $regex
     * @param        $redirect
     * @param string $after
     */
    protected function _addRewriteRule($regex, $redirect, $after = 'top')
    {
        add_rewrite_rule($regex, $redirect, $after);
    }


    /**
     * TODO: See if rIDX or any other plugins require this method
     *
     * @codeCoverageIgnore
     */
    public function swapQuery($query)
    {
        global $wp_query;

        if (!$wp_query || !is_array($wp_query->query) || !is_array($query->query) )
            return;

        if ( isset($wp_query->query["hji-controller"]) )
        {

            if ( !isset($query->query["hji-controller"]) )
            {
                $wp_query->query["hji-query-swap"] = $wp_query->query;
                $wp_query->query = null;
            }
            else
            {
                global $wp_version;

                //caller_get_posts deprecated since v.3.1
                if ( (version_compare($wp_version, '3.1.0') >= 0) )
                {
                    $query->query_vars["ignore_sticky_posts"] = true;
                }
                else
                {
                    $query->query_vars["caller_get_posts"] = true;
                }
            }
        }
    }


    /**
     * @codeCoverageIgnore
     * @param $query
     * @return string
     */
    function clearQuery($query)
    {
        global $wp_query;

        if (!is_array($wp_query->query) || !isset($wp_query->query["hji-query-swap"]))
        {
            return $query;
        }

        return "";
    }

}


/**
 * TODO: needs real implementation example
 *
 * @codeCoverageIgnore
 */
class CPT_Router extends Router
{
    protected $table;


    public function __construct()
    {
    }


    public function add($model, $view, $controller, $slug = null, $expires = false)
    {
        if ($slug)
        {
            $this->table[$slug] = new Route($model, $view, $controller);
        }
    }

}


class Route extends \hji\membership\extension\abstracts\Component
{

    /**
     * @codeCoverageIgnore
     * @param      $model
     * @param      $view
     * @param      $controller
     * @param null $rewriteRule
     * @param bool $expires
     */
    public function __construct($model, $view, $controller, $rewriteRule = null, $expires = false)
    {
        $parts = explode('\\', $controller);

        $this->controllerName   = (isset($parts[1]) ? array_pop($parts) : $controller);

        $this->modelClassName       = $model;
        $this->viewClassName        = $view;
        $this->controllerClassName  = $controller;

        if (isset($rewriteRule[0]) && isset($rewriteRule[1]))
        {
            $this->rewriteMatch          = $rewriteRule[0];
            $this->rewriteQuery          = $rewriteRule[1];
        }
        else
        {
            $this->rewriteMatch          = null;
            $this->rewriteQuery          = null;
        }

        $this->cache                = (bool)$expires;
        $this->cacheExpires         = $expires;

        $rule = [];
        $rule['shared'] = true;
        $this->di()->addRule($this->modelClassName, $rule);
    }


    function getController()
    {
        return $this->di()->create($this->controllerClassName);
    }


    function getView()
    {
        return $this->di()->create($this->viewClassName);
    }


    function getAction()
    {
        $request = $this->services()->getRequest();

        if ($action = $request->get('hji-action'))
        {
            return 'action_' . $action;
        }

        return null;
    }
}
