<?php
namespace hji\membership\services;


use hji\common\utils\Paths;
use hji\common\utils\Views;
use hji\membership\controllers\APIClient;
use hji\membership\controllers\Dispatcher;
use hji\membership\Membership;
use hji\membership\models\Customer;
use hji\membership\models\UserActionLogger;

class ServiceContainer extends DependencyInjector
{
    /**
     * Returns a current instance of models\Customer
     *
     * @since 2.2.0
     * @return mixed
     */
    function getCustomer()
    {
        return $this->di()->get(Customer::class);
    }


    /**
     * Returns a current authorized instance of hji\common\utils\APIClient
     *
     * @since 2.2.0
     * @return mixed
     */
    function getApiClient()
    {
        return ($this->di()->get(APIClient::class))->api;
    }


    /**
     * Returns a current instance of controllers\Dispatcher
     *
     * @since 2.2.0
     * @return mixed
     */
    function getDispatcher()
    {
        return $this->di()->get(Dispatcher::class);
    }


    /**
     * Returns a current instance of models\UserActionLogger
     *
     * @since 2.2.0
     * @return mixed
     */
    public function getUserActionLogger()
    {
        return $this->di()->get(UserActionLogger::class);
    }


    /**
     * @since 2.2.0
     * @param $templatesPath
     * @return object
     */
    public function getView($templatesPaths)
    {
        return $this->di()->create(Views::class, [$templatesPaths]);
    }


    /**
     * @since 2.2.2
     * @return object
     */
    public function getRequest()
    {
        return $this->di()->get(Request::class);
    }


    /**
     * @since 2.2.2
     * @return object
     */
    public function getPaths()
    {
        return $this->di()->get(Paths::class, [Membership::$file]);
    }


    /**
     * @since 2.2.2
     * @return object
     */
    public function getRouter()
    {
        return $this->di()->get(Router::class);
    }


    public function getRegistry()
    {
        return $this->di()->get(Registry::class);
    }
}
