<?php

namespace hji\membership\services\seo;

class HJIOpenGraph
{
    /**
     * @var int
     */
    private static $imageWidth = 1000;

    /**
     * @var int
     */
    private static $imageHeight = 750;

    /**
     * @var string
     */
    public static $openGraphType = 'place';



    public static function initActions(): void
    {
        add_action('hji_open_graph_image', [self::class, 'getOpenGraphImagesHtml']);
        add_action('hji_open_graph_title', [self::class, 'getOpenGraphSiteTitle']);
        add_action('hji_open_graph_description', [self::class, 'getOpenGraphDescription']);
        add_action('hji_open_graph_locale_meta', [self::class, 'getOpenGraphLocaleMeta']);
        add_action('hji_open_graph_type', [self::class, 'getOpenGraphMetaType']);
        add_action('hji_open_graph_location', [self::class, 'getOpenGraphPlaceLocationCoordinate']);
        add_action('hji_open_graph_url', [self::class, 'getOpenGraphPageUrl']);
        add_action('hji_open_graph_site_name', [self::class, 'getOpenGraphSiteName']);
    }

    /**
     * @param $post
     * @return void
     */
    public static function printOpenGraphInfo($post): void
    {
        do_action('hji_open_graph_image', $post);
        do_action('hji_open_graph_title', $post);
        do_action('hji_open_graph_description', $post);
        do_action('hji_open_graph_locale_meta', $post);
        do_action('hji_open_graph_type', $post);
        do_action('hji_open_graph_location', $post);
        do_action('hji_open_graph_url', $post);
        do_action('hji_open_graph_site_name', $post);
    }

    /**
     * @param $post
     * @return void
     */
    public static function getOpenGraphImagesHtml($post): void
    {
        $html = '';
        $width = self::getImageWidth();
        $imageGetParams = '';

        if (!empty(self::getImageWidth())) {
            $html .= '<meta property="og:image:width" content="' . self::getImageWidth() . '">';
            $imageGetParams = '?width=' . $width;
        }

        if (!empty(self::getImageHeight())) {
            $html .= '<meta property="og:image:height" content="' . self::getImageHeight() . '">';
        }

        $images = OpenGraphUtils::getImagesByListing($post);

        $imgOGDef = '<meta property="og:image" content="%s%s">';
        foreach ($images as $imageUrl) {
            $html .= sprintf($imgOGDef, $imageUrl, $imageGetParams);
        }

        echo $html;
    }

    /**
     * @param $post
     * @return bool
     */
    public static function getOpenGraphPlaceLocationCoordinate($post): bool
    {
        if (!property_exists($post, 'listing')) {
            return false;
        }

        if (isset($post->listing->coordinates)) {
            if (isset($post->listing->coordinates->latitude)) {
                $coords['latitude'] = $post->listing->coordinates->latitude;
            }

            if (isset($post->listing->coordinates->longitude)) {
                $coords['latitude'] = $post->listing->coordinates->longitude;
            }
        } else {
            if (isset($post->listing->coordinates_latitude)) {
                $coords['latitude'] = $post->listing->coordinates_latitude;
            }

            if (isset($post->listing->coordinates_longitude)) {
                $coords['longitude'] = $post->listing->coordinates_longitude;
            }
        }

        $html = '';
        if (!empty($coords['latitude'])) {
            $html .= '<meta property="place:location:latitude" content="' . $coords['latitude'] . '">';
        }

        if (!empty($coords['longitude'])) {
            $html .= '<meta property="place:location:longitude" content="' . $coords['longitude'] . '">';
        }

        echo $html;

        return true;
    }

    /**
     * @param $post
     * @return bool
     */
    public static function getOpenGraphSiteTitle($post): bool
    {
        if (
            is_object($post)
            && property_exists($post, 'listing')
            && !empty($post->listing->id)
            && !empty($post->listing->address)
        ) {
            $title = sprintf('MLS# %s - %s', $post->listing->id, $post->listing->address);

            echo '<meta property="og:title" content="' . $title . '">';

            return true;
        }

        echo '';

        return false;
    }

    /**
     * @param $post
     * @return bool
     */
    public static function getOpenGraphLocaleMeta($post): bool
    {
        $locale = self::getOpenGraphLocale('en_En', $post);

        if (!empty($locale)) {
            echo '<meta property="og:locale" content="' . $locale . '">';

            return true;
        }

        echo '';

        return false;
    }

    /**
     * @param $locale
     * @param $post
     * @return mixed|null
     */
    public static function getOpenGraphLocale($locale, $post)
    {
        return apply_filters('hji_open_graph_locale', $locale, $post);
    }

    /**
     * @param $post
     * @return bool
     */
    public static function getOpenGraphMetaType($post): bool
    {
        $type = self::getOpenGraphPageType($post);
        if (!empty($type)) {
            echo '<meta property="og:type" content="' . $type . '">';

            return true;
        }

        echo '';

        return false;
    }

    /**
     * @param $post
     * @return bool
     */
    public static function getOpenGraphDescription($post): bool
    {
        if (
            property_exists($post, 'listing')
            && isset($post->listing->description)
        ) {
            $meta = [];

            if ($post->listing->listPrice) {
                $listPrice = filter_var($post->listing->listPrice, FILTER_SANITIZE_NUMBER_INT);
                $meta[]    = '$' . number_format($listPrice);
            }

            if ($post->listing->beds) {
                $meta[] = $post->listing->beds . '-beds';
            }

            if ($post->listing->baths_total) {
                $meta[] = $post->listing->baths_total . '-baths';
            }

            $metaDesc = implode(' &#8226; ', $meta) . '.';

            if ($post->listing->description) {
                $metaDesc .= ' ' . $post->listing->description;
            }

            echo '<meta property="og:description" content="' . $metaDesc . '">';

            return true;
        }

        echo '';

        return false;
    }

    /**
     * @return bool
     */
    public static function getOpenGraphPageUrl(): bool
    {
        $url = home_url($_SERVER['REQUEST_URI']);
        if (!empty($url)) {
            echo '<meta property="og:url" content="' . $url . '">';

            return true;
        }

        echo '';

        return false;
    }

    /**
     * @return bool
     */
    public static function getOpenGraphSiteName(): bool
    {
        $siteName = get_bloginfo('name');

        if (!empty($siteName)) {
            echo '<meta property="og:site_name" content="' . $siteName . '">';

            return true;
        }

        echo '';

        return true;
    }

    /**
     * @return int
     */
    public static function getImageHeight(): int
    {
        return apply_filters('hji_open_graph_image_height', self::$imageHeight);
    }

    /**
     * @return int
     */
    public static function getImageWidth(): int
    {
        return apply_filters('hji_open_graph_image_width', self::$imageWidth);
    }

    /**
     * @param $post
     *
     * @return string
     */
    public static function getOpenGraphPageType($post): string
    {
        return apply_filters('hji_open_graph_page_type', self::$openGraphType, $post, 2);
    }
}
