<?php

namespace hji\membership\services\seo;

class OpenGraphUtils
{
    /**
     * @param $post
     *
     * @return array
     */
    public static function getImagesByListing($post): array
    {
        $result = [];

        if (
            $post
            && property_exists($post, 'listing')
            && property_exists($post->listing, 'images')
            && self::_isListingPost($post)
            && !empty($post->listing->images)
        ) {
            foreach ($post->listing->images->getAll() as $image) {
                $result[] = esc_url($image->src);
            }
        }

        return $result;
    }

    /**
     * @param $post
     *
     * @return bool
     */
    public static function _isListingPost($post)
    {
        return (
            $post
            && is_main_query()
            && isset($post->listing)
            && in_array($post->post_type, self::_getSupportedPostTypes(), false)
        );
    }


    // @TODO: we've added 'idx-listing' here because we're not always using a 'post' post-type for listings (think blvd)
    protected static function _getSupportedPostTypes()
    {
        // @TODO: remove the 'post' from this array once a post type is fully established for rIDX listings
        $supportedTypes = ['post', 'idx-listing', 'nc-listing'];
        $wpSeoOpt = self::_getWPseoOptions();

        if ( ! empty($wpSeoOpt['metadesc-nc-listing']) && ($key = array_search('nc-listing', $supportedTypes))) {
            unset($supportedTypes[$key]);
        }

        return $supportedTypes;
    }


    protected static function _getWPseoOptions()
    {
        $options = [];

        if (class_exists('\WPSEO_Options')) {
            $options = \WPSEO_Options::get_all();
        }

        return $options;
    }
}
