<?php
namespace hji\membership\services\seo;

use hji\common\utils\PluginHelper;
use hji\membership\services\seo\presenters\HJICoordinatesLatitudePresenter;
use hji\membership\services\seo\presenters\HJICoordinatesLongitudePresenter;
use hji\membership\services\seo\presenters\HJIImageHeightPresenter;
use hji\membership\services\seo\presenters\HJIImagePresenter;
use hji\membership\services\seo\presenters\HJIImageWidthPresenter;

/**
 * Currently just an entry point for future developments
 *
 * @package hji\membership\services
 */
class SocialGraph
{
    function __construct()
    {

    }


    function init()
    {
        $this->setActions();
    }


    protected function setActions()
    {
        if (PluginHelper::hji_is_plugin_active('wordpress-seo/wp-seo.php')) {
            add_filter('wpseo_canonical', [$this, 'setOGCanonical'], 90, 1);
            add_filter('wpseo_metadesc', [$this, 'filterWpSeoMetaDescription']);
            add_filter('wpseo_opengraph_type', [$this, 'filterOGtype']);
            add_filter('wpseo_opengraph_url', [$this, 'setOGUrl']);
            add_filter('wpseo_opengraph_desc', [$this, 'filterWpSeoMetaDescription']);
            add_filter('wpseo_opengraph_title', [$this, 'filterWpSeoMetaTitle']);

            add_filter('wpseo_frontend_presenters', [$this, 'addPresenters']);

            add_action('plugins_loaded', [$this, 'addOldVersionActions']);
        } else {
            HJIOpenGraph::initActions();
            add_action('wp_head', [$this,'printOpenGraphInfo']);
        }
    }

    public function printOpenGraphInfo()
    {
        global $post;

        if (OpenGraphUtils::_isListingPost($post)) {
            HJIOpenGraph::printOpenGraphInfo($post);
        }
    }

    function addPresenters($presenters)
    {
        global $post;

        $this->addOGplaceCoordinates();

        $images = OpenGraphUtils::getImagesByListing($post);

        $presenters[] = new HJICoordinatesLatitudePresenter();
        $presenters[] = new HJICoordinatesLongitudePresenter();
        $presenters[] = new HJIImageWidthPresenter(1000);
        $presenters[] = new HJIImageHeightPresenter(750);

        if ( ! empty($images)) {
            foreach ($images as $image) {
                $imageUrlParams = '';
                if ( ! empty($this->imageWidth) && $this->imageWidth > 0) {
                    $imageUrlParams = $image.'?width='.$this->imageWidth;
                }

                $presenters[] = new HJIImagePresenter($image.$imageUrlParams);
            }
        }

        return $presenters;
    }


    function filterWpSeoMetaDescription($metadesc)
    {
        global $post;

        if (OpenGraphUtils::_isListingPost($post)) {
            $meta = [];

            if ($post->listing->listPrice) {
                $listPrice = filter_var($post->listing->listPrice, FILTER_SANITIZE_NUMBER_INT);
                $meta[]    = '$'.number_format($listPrice);
            }

            if ($post->listing->beds) {
                $meta[] = $post->listing->beds.'-beds';
            }

            if ($post->listing->baths_total) {
                $meta[] = $post->listing->baths_total.'-baths';
            }

            $metadesc = implode(' &#8226; ', $meta).'.';

            if ($post->listing->description) {
                $metadesc .= ' '.$post->listing->description;
            }
        }

        $metadesc = apply_filters('hji_socialgraph_metadesc', $metadesc, $post);

        return $metadesc;
    }

    function filterWpSeoMetaTitle($title)
    {
        global $post;

        if (OpenGraphUtils::_isListingPost($post)) {
            $title = sprintf('MLS# %s - %s', $post->listing->id, $post->listing->address);
        }

        return $title;
    }


    public function filterOGtype($type)
    {
        global $post;

        if (OpenGraphUtils::_isListingPost($post)) {
            $type = 'place';
        }

        return $type;
    }

    private function getPostPlaceCoordinates($post)
    {
        if (property_exists($post, 'listing')) {
            if (isset($post->listing->coordinates_longitude)) {
                return [
                    'latitude'  => $post->listing->coordinates_latitude,
                    'longitude' => $post->listing->coordinates_longitude,
                ];
            }

            if (isset($post->listing->coordinates)) {
                return [
                    'latitude'  => $post->listing->coordinates->latitude,
                    'longitude' => $post->listing->coordinates->longitude,
                ];
            }
        }

        return null;
    }


    public function addOGplaceCoordinates()
    {
        global $post;

        if (OpenGraphUtils::_isListingPost($post)) {
            $coordinates = $this->getPostPlaceCoordinates($post);

            if (null === $coordinates) {
                return;
            }

            // Deprecated, YOAST v < 14.0.0
            if (isset($GLOBALS['wpseo_og'])) {
                $GLOBALS['wpseo_og']->og_tag('place:location:latitude', $coordinates['latitude']);
                $GLOBALS['wpseo_og']->og_tag('place:location:longitude', $coordinates['longitude']);
            }

            add_filter('wpseo_og_place_location_latitude', function () use ($coordinates) {
                return $coordinates['latitude'];
            });

            add_filter('wpseo_og_place_location_longitude', function () use ($coordinates) {
                return $coordinates['longitude'];
            });
        }
    }

    /**
     * Adds images from listings to open graph meta tags
     */
    public function addOGimages()
    {
        global $post;

        if (OpenGraphUtils::_isListingPost($post)) {
            if ($post->listing->images) {
                if (isset($GLOBALS['wpseo_og'])) {
                    foreach ($post->listing->images->getAll() as $image) {
                        $GLOBALS['wpseo_og']->og_tag('og:image', esc_url($image->src).'?width=1000');
                    }

                    $GLOBALS['wpseo_og']->og_tag('og:image:width', '1000');
                    $GLOBALS['wpseo_og']->og_tag('og:image:height', '750');
                }
            }
        }
    }

    public function setOGUrl($url)
    {
        return home_url($_SERVER['REQUEST_URI']);
    }

    public function setOGCanonical()
    {
        return home_url($_SERVER['REQUEST_URI']);
    }

    public function addOldVersionActions()
    {
        if ( ! function_exists('get_plugins')) {
            include_once ABSPATH.'wp-admin/includes/plugin.php';
        }

        $plugins = get_plugins();
        foreach ($plugins as $pluginFile => $plugin) {
            if ($plugin['Name'] === 'Yoast SEO' && version_compare($plugin['Version'], '14.0', '<=')) {
                // Deprecated YOAST v < 14.0.0
                add_action('wpseo_opengraph', [$this, 'addOGplaceCoordinates']);
                add_action('wpseo_opengraph', [$this, 'addOGimages']);
            }
        }
    }
}
