<?php

namespace hji\membership\utils;

use hji\common\utils\PluginHelper;

class URLHelper
{
    /**
     * @param $url
     *
     * @return false|int
     */
    public static function isValidURL($url)
    {
        $parsedUrl = parse_url($url);

        if (!is_array($parsedUrl) || !isset($parsedUrl['scheme'], $parsedUrl['host'])) {
            return false;
        }

        return $parsedUrl['scheme'] === 'http' || $parsedUrl['scheme'] === 'https';
    }

    public static function getPluginURIBySlug($slug)
    {
        if (is_multisite() && PluginHelper::hji_is_plugin_active('wordpress-mu-domain-mapping/domain_mapping.php')) {
            $siteUrlFromConst = URL::getDomainName(WP_PLUGIN_URL);
            $siteUrlOrig = URL::getDomainName(site_url());

            if ($siteUrlFromConst !== $siteUrlOrig) {
                $pluginURI = str_replace($siteUrlFromConst, $siteUrlOrig, WP_PLUGIN_URL);

                return $pluginURI . '/' . $slug;
            }

            return WP_PLUGIN_URL . '/' . $slug;
        }

        return WP_PLUGIN_URL . '/' . $slug;
    }

    /**
     * @return array|mixed|string|string[]
     */
    public static function getUploadDir()
    {
        $uploadDir = wp_upload_dir();

        if (is_multisite() && PluginHelper::hji_is_plugin_active('wordpress-mu-domain-mapping/domain_mapping.php')) {
            $siteUrlFromConst = URL::getDomainName($uploadDir['baseurl']);
            $siteUrlOrig = URL::getDomainName(site_url());

            if ($siteUrlFromConst !== $siteUrlOrig) {

                return str_replace($siteUrlFromConst, $siteUrlOrig, $uploadDir['baseurl']);
            }

            return $uploadDir['baseurl'];
        }

        return $uploadDir['baseurl'];
    }

    public static function unparseUrl(array $parsed)
    {
        $pass      = $parsed['pass'] ?? null;
        $user      = $parsed['user'] ?? null;
        $userinfo  = $pass !== null ? "$user:$pass" : $user;
        $port      = $parsed['port'] ?? 0;
        $scheme    = $parsed['scheme'] ?? "";
        $query     = $parsed['query'] ?? "";
        $fragment  = $parsed['fragment'] ?? "";
        $authority = (
            ($userinfo !== null ? "$userinfo@" : "") .
            ($parsed['host'] ?? "") .
            ($port ? ":$port" : "")
        );

        return (
            (\strlen($scheme) > 0 ? "$scheme:" : '') .
            (\strlen($authority) > 0 ? "//$authority" : '') .
            ($parsed['path'] ?? '') .
            (\strlen($query) > 0 ? "?$query" : '') .
            (\strlen($fragment) > 0 ? "#$fragment" : '')
        );
    }
}
