<?php

namespace hji\common;


use hji\common\plugin\PluginAPI;
use hji\membership\services\ServiceContainer;


abstract class Plugin
{
    public $name;
    public $slug;
    public $file; // absolute path to the plugin's main file
    protected $pluginAPI;
    protected $pluginInfo;

    /** @return mixed - Called by Membership when plugins loaded. */
    abstract public function init();


    public function __construct($name, $slug, $mainFilePath, PluginAPI $pluginAPI = null)
    {
        $this->name = $name;
        $this->slug = $slug;
        $this->file = $mainFilePath;
        $this->pluginAPI = $pluginAPI;
        $this->pluginInfo = get_plugin_data($this->file, false, false);
    }


    public function api()
    {
        if (!$this->pluginAPI instanceof PluginAPI)
            $this->pluginAPI = $this->createAPI();

        return $this->pluginAPI;
    }


    /**
     * Runs on plugin activation
     * If the plugin App class contains a method called doOnPluginActivation() it will also be called.
     */
    public function doOnPluginActivation($networkWide)
    {
        flush_rewrite_rules();
    }

    public function doOnNetworkPluginActivation()
    {
        // do something on network plugin activation
    }

    /**
     * Runs on plugin deactivation
     * If the plugin App class contains a method called doOnPluginDeactivation() it will also be called.
     */
    public function doOnPluginDeactivation()
    {
        flush_rewrite_rules();
    }

    public function doOnNetworkPluginDeactivation()
    {
        // do something on network plugin deactivation
    }


    protected function createAPI()
    {
        return new PluginAPI(new ServiceContainer());
    }

    /**
     * @return string|null
     */
    public function getPluginName()
    {
        return $this->pluginInfo['Name'] ?? null;
    }

    /**
     * @return string|null
     */
    public function getPluginTextDomain()
    {
        return $this->pluginInfo['TextDomain'] ?? null;
    }
}
