<?php

namespace hji\common\gravityform\import;

class FormImportContext
{
    private $existingForms;
    private $insertedForms;
    private $updatedForms;
    private $preImportedForms;

    public function __construct(array $existingForms, array $preImportedForms)
    {
        $this->existingForms = $existingForms;
        $this->insertedForms = [];
        $this->updatedForms = [];
        $this->preImportedForms = $preImportedForms;
    }

    public function addInsertedForm(string $slug, $id, $form, $pathToMainPluginFile)
    {
        $this->insertedForms[$slug] = [
            'id' => $id,
            'pluginPath' => $pathToMainPluginFile,
            'formVersion' => $form['hji_form_version'] ?? '',
            'formKey' => $form['hji_form_key'] ?? '',
            'formTitle' => $form['title'],
            ]
        ;
    }
    
    public function addPreImportedForm(array  $preImportedForms, string $pluginSlug, string $filePath, array $form)
    {
        if (!isset($form['hji_form_key'])) {
            $form['hji_form_key'] = basename($filePath, '.json');
        }
        $formSlug = $pluginSlug . ':' . $form['hji_form_key'];
        $this->preImportedForms[$pluginSlug . ':' . $form['hji_form_key']] = [
            'slug' => $formSlug,
            'filePath' => $filePath,
            'formVersion' => $form['hji_form_version'] ?? ''
            ]
        ;
    }

    public function addUpdatedForm(string $slug, $id, $form)
    {
        $this->insertedForms[$slug] = [
            'id' => $id,
            'slug' => $slug,
            'formVersion' => $form['hji_form_version'] ?? '',
            'formKey' => $form['hji_form_key'] ?? ''
        ]
        ;
    }

    public function getExistingForms()
    {
        return $this->existingForms;
    }

    public function getInsertedForms()
    {
        return $this->insertedForms;
    }
    
    public function getPreImportedForms()
    {
        return $this->preImportedForms;
    }

    public function getUpdatedForms()
    {
        return $this->updatedForms;
    }
    
    public function getActualForms()
    {
        //Existed + Inserted
        return array_merge($this->getExistingForms(), $this->getInsertedForms());
    }
    
    public function getMissingForms()
    {
        //existing - inserted - updated
        return array_diff($this->getExistingForms(), $this->getActualForms());
    }
}
