<?php

namespace hji\common\gravityform\import;


class FormImporter
{
    /**
     * @var FormImportContext
     */
    private $context;

    public function __construct(FormImportContext $context)
    {
        $this->context = $context;
    }

    public function importDirectory(string $nameSpace, string $pathToMainPluginFile)
    {
        $dir = rtrim(dirname($pathToMainPluginFile), '/') . '/resources/gravity-forms';
        $dirIterator = new \DirectoryIterator($dir);

        $gfForms = \GFAPI::get_forms();

        foreach ($dirIterator as $item) {
            if ($item->isDot() || $item->isDir() || !preg_match('/.\.(json)$/', $item->getFilename())) {
                continue;
            }

            $this->importFile($nameSpace, $item->getPathname(), $gfForms, $pathToMainPluginFile);
        }
    }

    public function importFile(string $nameSpace, string $file, array $gfForms, string $pathToMainPluginFile)
    {
        $addedForms = [];
        $importHook = static function ($forms) use (&$addedForms) {
            $addedForms = $forms;
        };

        $fileContent = file_get_contents($file);
        
        if ($fileContent === false) {
            throw new \RuntimeException("File '{$file}' not exist or empty");
        }
        
        $newForm = json_decode($fileContent, true);
        if (!is_array($newForm)) {
            throw new \RuntimeException("Importing '{$file}' is empty or have not data in json format");
        }
        
        $newForm = $newForm[0];
        $existingForms = $this->context->getExistingForms();
        $importedFormId = 0;

        $importFormStatus = $this->importingFormStatus($existingForms, $newForm, $gfForms, $nameSpace);

        switch ($importFormStatus) {
            case 0 :
                add_action('gform_forms_post_import', $importHook);
                \GFExport::import_file($file);
                remove_action('gform_forms_post_import', $importHook);
    
                if (($count = count($addedForms)) !== 1) {
                    throw new \RuntimeException("Expected that only one form should be imported from '{$file}', but actually imported {$count} forms");
                }

                $formSlug = $nameSpace . ':' . $newForm['hji_form_key'];
                $this->context->addInsertedForm($formSlug, array_values($addedForms)[0]['id'], $newForm, $pathToMainPluginFile);
                $importedFormId = array_values($addedForms)[0]['id'];
                break;
            case 1 :
                $preImportedForms = $this->context->getPreImportedForms();
                $this->context->addPreImportedForm($preImportedForms, $nameSpace, $file, $newForm);
                break;
            case 2 :
                break;
        }

        return $importedFormId;
    }

    public function updateFormFromFile(string $nameSpace, string $file, $formId)
    {
        $fileContent = file_get_contents($file);

        if ($fileContent === false) {
            throw new \RuntimeException("File '{$file}' not exist or empty");
        }

        $newForm = json_decode($fileContent, true);
        if (!is_array($newForm)) {
            throw new \RuntimeException("Importing '{$file}' is empty or have not data in json format");
        }

        $newForm = $newForm[0];
        $form = \GFAPI::get_form($formId);
        foreach ($newForm as $key => $field) {
            if ($key === 'id') {
                continue;
            }

            $form[$key] = $field;
        }

        \GFAPI::update_form($form);
    }
    
    private function importingFormStatus(array $existingForms, array $newForm, array $gfForms, string $namespace): int
    {
        if (!(is_array($newForm) && isset($newForm['hji_form_key'], $newForm['hji_form_version']))) {
            return 2;
        }

        $formSlug = $namespace . ':' . $newForm['hji_form_key'];
        if (!empty($existingForms[$formSlug])) {
            foreach ($gfForms as $form) {
                if (isset($form['hji_form_key']) && $form['hji_form_key'] === $newForm['hji_form_key']) {
                    if (isset($form['hji_form_version'])) {
                        $versionCompare = version_compare($form['hji_form_version'], $newForm['hji_form_version']);
                        if ($versionCompare === -1) {
                            return 1;
                        }

                        return 2;
                    }
                }
            }

            return 0;
        }

        return 0;
    }
}
