<?php
namespace hji\common\gravityform\import;

use GFAPI;
use hji\common\utils\GFHelper;
use hji\common\utils\PluginHelper;
use hji\membership\advancedOptions\AdvancedOptions;
use hji\membership\advancedOptions\GravityFormsMappingOptions;
use hji\membership\Membership;

class GravityFormImport
{

    const HJI_GF_MAPPING_OPTIONS = 'hji_gf_mapping';
    /**
     *
     */
    public function initHooks()
    {
        add_shortcode('hji_gravity_forms', [$this, 'renderGFFormByFormKey']);
        add_action('gform_after_delete_form', [$this, 'updateMappingOptionsAfterDeleteForm']);
        add_action('gform_post_form_deactivated', [$this, 'updateMappingOptionsAfterDeactivateForm'], 10, 2);
        add_action( 'gform_post_form_trashed', [$this, 'updateMappingOptionsAfterDeactivateForm'], 10, 1 );
        add_action('redux/options/advanced_options/saved', [$this, 'reduxOptionsSaved']);
        add_action('wp_ajax_hji_gf_form_update', [$this, 'importUpdateResponse']);
    }

    /**
     * @param string $pathToMainPluginFile
     * @param string $namespace
     * @param $networkWide
     * @return bool
     */
    public function import(string $pathToMainPluginFile, string $namespace, $networkWide): bool
    {
        if (!Membership::getInstance()->gravityFormsActive) {
            return false;
        }

        $this->updatePluginPath($pathToMainPluginFile, $namespace);

        if (!GFHelper::gravityFormsExists()) {
            //throw new \RuntimeException('Gravity Forms plugin is not active');
            error_log('Wordpress Notice: Gravity Forms plugin is not active');
            add_action('admin_notices', function () {
                echo '<div class="notice notice-error  is-dismissible"><p>The following required plugin is currently inactive: <em>Gravity Forms</em></p></div>';
            }, 40);
            return false;
        }

        if (is_multisite() && !empty($networkWide)) {
            $currentBlogId = get_current_blog_id();
            if (is_main_site($currentBlogId)) {
                $sites = get_sites(['fields' => 'ids']);
                foreach ($sites as $blogId) {
                    switch_to_blog($blogId);
                    if (!MappingOldForms::isNormalizeExistedForms($namespace)) {
                        MappingOldForms::normalizeAndMapping($namespace);
                    }
                    $this->importForSingleInstance($pathToMainPluginFile, $namespace);
                }

                switch_to_blog($currentBlogId);
            } else {
                if (!MappingOldForms::isNormalizeExistedForms($namespace)) {
                    MappingOldForms::normalizeAndMapping($namespace);
                }
                $this->importForSingleInstance($pathToMainPluginFile, $namespace);
            }
        } else {
            if (!MappingOldForms::isNormalizeExistedForms($namespace)) {
                MappingOldForms::normalizeAndMapping($namespace);
            }
            $this->importForSingleInstance($pathToMainPluginFile, $namespace);
        }

        return true;
    }

    public function updatePluginPath(string $pathToMainPluginFile, string $pluginSlug): void
    {
        $gfMappingOptions = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, []);
        if (strpos($pathToMainPluginFile, WP_PLUGIN_DIR)!== false) {
            if (!file_exists($pathToMainPluginFile)) {
                return;
            }
        } else {
            return;
        }

        $pluginData = get_plugin_data($pathToMainPluginFile, false, false);
        $pluginNameByPath = PluginHelper::getPluginSlug($pluginData['Name']);
        $flag = false;

        foreach ($gfMappingOptions as $key => $option) {
            $slugArray = explode(':', $key);
            $pluginNameByDB = $slugArray[0] ?? '';
            if ($pluginNameByDB === $pluginNameByPath) {

                if (!empty($option['pluginPath'])) {
                    $pathArray = explode('/', $option['pluginPath']);
                    $countPathParts = count($pathArray);
                    if ($countPathParts > 2) {
                        $pathArray = explode('/', $pathToMainPluginFile);
                        $countPathParts = count($pathArray);
                        $pluginPath = $pathArray[$countPathParts - 2] . '/' . $pathArray[$countPathParts - 1];
                        $gfMappingOptions[$key]['pluginPath'] = $pluginPath;
                        $flag = true;
                    }
                } else {
                    $pathArray = explode('/', $pathToMainPluginFile);
                    $countPathParts = count($pathArray);
                    $pluginPath = $pathArray[$countPathParts - 2] . '/' . $pathArray[$countPathParts - 1];
                    $gfMappingOptions[$key]['pluginPath'] = $pluginPath;
                    $flag = true;
                }
            }
        }

        if ($flag) {
            update_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, $gfMappingOptions);
        }

        $gfMappingOptions = get_option(Membership::HJI_GF_MAPPING_PREIMPORTED_OPTIONS_KEY, []);
        if (strpos($pathToMainPluginFile, WP_PLUGIN_DIR) !== false) {
            if (!file_exists($pathToMainPluginFile)) {
                return;
            }
        } else {
            return;
        }

        $flag = false;
        $pathArray = explode('/', $pathToMainPluginFile);
        $countPathParts = count($pathArray);
        $pluginPath = $pathArray[$countPathParts - 2] . '/' . $pathArray[$countPathParts - 1];
        $pluginDirName = basename($pluginPath);

        foreach ($gfMappingOptions as $key => $option) {
            $slugArray = explode(':', $key);
            $pluginNameByDB = $slugArray[0] ?? '';
            if ($pluginNameByDB === $pluginNameByPath) {

                if (!empty($option['filePath'])) {
                    $pathArray = explode($pluginDirName, $option['filePath']);
                    $countPathParts = count($pathArray);
                    if ($countPathParts > 1 ) {
                        $pathArray[0] = '/';
                        $gfMappingOptions[$key]['filePath'] = implode($pluginDirName, $pathArray);
                        $flag = true;
                    }
                }
            }
        }

        if ($flag) {
            update_option(Membership::HJI_GF_MAPPING_PREIMPORTED_OPTIONS_KEY, $gfMappingOptions);
        }
    }

    private function importForSingleInstance($pathToMainPluginFile, $namespace)
    {
        $contextFactory = new FormImportContextFactory();
        $context  = $contextFactory->create($namespace);
        $importer = new FormImporter($context);
        $importer->importDirectory($namespace, $pathToMainPluginFile);

        $actualForms = $context->getActualForms();

        if (count($context->getActualForms()) > 0) {
            $this->setMappingOptions($actualForms);
            $this->setReduxOptions($context->getActualForms());
        }

        if (count($context->getPreimportedForms()) > 0) {
            $this->setPreImportedOptions($context->getPreimportedForms());
        }
    }

    public function getPreimportedOptions()
    {
        return get_option('hji_gf_mapping_preimported');
    }

    public function setPreImportedOptions($options)
    {
        return update_option('hji_gf_mapping_preimported', $options, false);
    }

    public function setMappingOptions($options)
    {
        return update_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, $options, false);
    }

    private function setReduxOptions(array $options)
    {
        $reduxOptions = [];

        if (!class_exists('Redux')) {
            require_once  Membership::$dir . '/third-party/redux-framework/redux-framework.php';
            require_once  Membership::$dir . '/third-party/redux-framework/redux-core/inc/classes/class-redux-api.php';
            require_once  Membership::$dir . '/third-party/redux-framework/redux-core/inc/classes/class-redux-functions-ex.php';
        }

        foreach ($options as $key => $value) {
            $reduxOptions[] = \Redux::set_option('advanced_options', 'gf_form_' . str_replace(':', '_', $key), $value['id']);
        }


        return $reduxOptions;
    }

    public function importUpdateResponse()
    {
        $updateFormField = ['description', 'notifications', 'confirmations'];
        $slug = $_POST['hji_gf_form_slug'] ?? null;
        $answers = $_POST['hji_gf_form_answer'] ?? null;

        if ($slug === null || $answers === null) {
            wp_send_json_error('400', '400');
        }

        $slugArray = explode(':', $slug);
        $optionsFormKey = $slugArray[1];

        $formExist = false;
        $gfForms = GFAPI::get_forms();
        $originalOldForm = [];
        foreach ($gfForms as $form) {
            if (isset($form['hji_form_key']) && $optionsFormKey === $form['hji_form_key']) {
                $formExist = true;
                $originalOldForm = $form;
            }
        }

        $mappingOptions = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, null);
        $preImportedOptions = get_option('hji_gf_mapping_preimported', null);
        $newForm = false;

        if (isset($preImportedOptions[$slug]['filePath']) && file_exists($preImportedOptions[$slug]['filePath'])) {
            $newFormJson = file_get_contents($preImportedOptions[$slug]['filePath']);
            if ($newFormJson === false) {
                wp_send_json_error('400', '400');
            }

            $newForm = json_decode($newFormJson, true);
            $newForm = $newForm[0];
        }

        if ((int)$answers === 0 && is_array($newForm) && $formExist !== false) {
            foreach ($updateFormField as $key) {
                $newForm[$key] = $originalOldForm[$key];
            }

            $newForm['is_active'] = '1';
            $newForm['is_trash'] = '0';

            \GFAPI::update_form($newForm, $originalOldForm['id']);
            unset($preImportedOptions[$slug]);
            update_option('hji_gf_mapping_preimported', $preImportedOptions, false);
            $this->setReduxOption($slug, $originalOldForm['id']);

            wp_send_json_success(json_encode($slug), 200);
        }

        if ((int)$answers === 1 || (int)$answers === 2 || $formExist === false) {
            if (is_array($newForm)) {
                $importHook = static function ($forms) use (&$addedForms) {
                    $addedForms = $forms;
                };

                $addedForms = [];
                add_action('gform_forms_post_import', $importHook);
                \GFExport::import_file($preImportedOptions[$slug]['filePath']);
                remove_action('gform_forms_post_import', $importHook);

                if (isset($addedForms[0]['id']) && $addedForms[0]['id'] > 0) {
                    $mappingOptions[$slug]['id'] = $addedForms[0]['id'];
                    update_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, $mappingOptions, false);

                    unset($preImportedOptions[$slug]);
                    update_option('hji_gf_mapping_preimported', $preImportedOptions, false);
                    $this->setReduxOption($slug, $addedForms[0]['id']);

                    wp_send_json_success(json_encode($slug), 200);
                }
            }
        }

        wp_send_json_error('400', '400');
    }

    /**
     * @param $slug
     * @param $gravityFormId
     * @return bool
     */
    protected function setReduxOption($slug, $gravityFormId)
    {
        $advancedOptions =  AdvancedOptions::getInstance();
        $formSlugArray = explode(':', $slug);

        return \Redux::set_option($advancedOptions->reduxOptionName, GravityFormsMappingOptions::OPTION_PREFIX . $formSlugArray[0] . '_' . $formSlugArray[1], $gravityFormId);
    }

    public function reduxOptionsSaved($values)
    {
        $options = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, []);

        foreach ($options as $key => $option) {
            $reduxKey = 'gf_form_' . str_replace(':', '_', $key);
            if (isset($values[$reduxKey]) && (int)$option['id'] !== (int)$values[$reduxKey]) {
                $form = GFAPI::get_form($values[$reduxKey]);

                if (!is_array($form)) {
                    continue;
                }

                $options[$key]['id'] = $values[$reduxKey];

                $updateFormFlag = false;
                if (empty($form['hji_form_key']) && isset($form['title'])) {
                    $form['hji_form_key'] = strtolower(str_replace(' ', '_', $form['title']));
                    $updateFormFlag = true;
                }

                if (empty($form['hji_form_version'])) {
                    $form['hji_form_version'] = '0.0.1';
                    $updateFormFlag = true;
                }

                $options[$key]['formKey'] = $form['hji_form_key'];
                $options[$key]['formVersion'] = $form['hji_form_version'];

                if ($updateFormFlag && !empty($form['id']) && isset($form)) {
                    GFAPI::update_form($form, $form['id']);
                }
            }
        }

        return $this->setMappingOptions($options);
    }

    public function renderGFFormByFormKey(array $atts, $content, $tag)
    {

        $form = $this->getGFFormByFormKey($atts['form_key']);

        if (empty($form['id'])) {
            return '';
        }

        $fieldValues = [];
        if (!empty($atts['field_values'])) {
            $fieldValuesTemp = str_replace('field_values', '', $atts['field_values']);
            $paramsArray = explode('&', $fieldValuesTemp);
            foreach ($paramsArray as $param) {
                $values = explode('=', $param);
                if (is_array($values) && count($values) === 2) {
                    $fieldValues[$values[0]] = $values[1];
                }
            }
        }

        foreach ($atts as $key => $value) {
            if ($value === 'false') {
                $atts[$key] = false;
            } elseif ($value === true) {
                $atts[$key] = true;
            }
        }

        $formData = [
            'display_title' => $atts['title'] ?? true,
            'display_description' => $atts['description'] ?? true,
            'display_inactive' => $atts['inactive'] ?? false,
            'field_values' => $fieldValues,
            'ajax' => $atts['ajax'] ?? false,
            'tabindex' => $atts['tabindex'] ?? '',
        ];

        $formData = apply_filters('gravity_form_call_options', $formData, $atts['form_key'], $form);

        return gravity_form(
            $form['id'],
            $formData['display_title'],
            $formData['display_description'],
            $formData['display_inactive'] ,
            $formData['field_values'],
            $formData['ajax'],
            $formData['tabindex'],
            false
        );
    }

    /**
     * @param $formKey
     * @return false|mixed|null
     */
    public function getGFFormByFormKey($formKey)
    {
        if (!GFHelper::gravityFormsExists()) {
            return null;
        }
        $options = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, []);

        foreach ($options as $key => $option) {
            if (!empty($option['id']) && strpos($key, ':' . $formKey) !== false) {

                return GFAPI::get_form($option['id']);
            }
        }

        return null;
    }

    public function updateMappingOptionsAfterDeleteForm($formId):bool
    {
        $options = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, []);

        foreach ($options as $key => $value) {
            if ($formId == $value['id']) {
                $options[$key]['id'] = '';
                \Redux::set_option('advanced_options', 'gf_form_' . str_replace(':', '_', $key), '');
            }
        }

        return $this->setMappingOptions($options);
    }

    public function updateMappingOptionsAfterDeactivateForm($formId)
    {
        $form = GFAPI::get_form($formId);

        if (!is_array($form)) {
            return false;
        }

        if (isset($form['is_active'], $form['is_trash'])) {
            if ($form['is_active'] == 0 || $form['is_trash'] == 1) {
                $options = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, []);

                foreach ($options as $key => $item) {
                    if ($item['id'] == $formId) {
                        $options[$key]['id'] ='';
                        $reduxOption = \Redux::get_option('advanced_options', 'gf_form_' . str_replace(':', '_', $key));
                        if ($formId == $reduxOption) {
                            \Redux::set_option('advanced_options', 'gf_form_' . str_replace(':', '_', $key), '');
                        }
                    }
                }

                return $this->setMappingOptions($options);
            }
        }

        return false;
    }
}
