<?php

namespace hji\common\gravityform\import;

use GFAPI;
use hji\common\utils\GFHelper;

class Notification
{
    const HJI_GF_MAPPING_OPTIONS = 'hji_gf_mapping';

    /**
     * @param string $formMappingKey
     * @return bool
     */
    public function updateNotificationMessage(string $formMappingKey): bool
    {
        if (!GFHelper::gravityFormsExists()) {
            return false;
        }

        $mappingOptions = get_option(self::HJI_GF_MAPPING_OPTIONS, null);
        if ($mappingOptions === null || empty($mappingOptions[$formMappingKey]['id'])) {
            return false;
        }

        $tourTheHomeFormId = $mappingOptions[$formMappingKey]['id'];

        $form = GFAPI::get_form($tourTheHomeFormId);

        if ($form === false) {
            return false;
        }

        if (empty($form['notifications']) || count($form['notifications']) === 0) {
            return false;
        }

        foreach ($form['notifications'] as $key => $notification) {
            if (empty($notification['message'])) {
                continue;
            }

            if ($notification['message'] !== '{all_fields}') {
                continue;
            }

            $form['notifications'][$key]['message'] = $this->getMessage();

            GFAPI::update_form($form, $tourTheHomeFormId);
        }
        return true;
    }

    /**
     * @return string
     */
    private function getMessage(): string
    {
        $message = '
            {:1}<br>
            Name: {Full Name:6}<br>
            Email: {Email:8}<br>
            Phone: {Phone:7}<br>
            Request Type: {Request Type:15}<br>
            Date: {Pick Your Date:2}<br>
            Time: {Pick Your Time:5}<br>
            Request Type: {Request Type:15}<br>
            Message: {Message:9}<br>
            URL: {embed_url}
        ';

        return $message;
    }
}
