<?php

namespace hji\common\gravityform\import\mapping;

use hji\membership\Membership;
use hji\ResponsiveIDX\controllers\Settings;

class ExistedForms
{
    private $namespace;

    public function __construct($namespace)
    {
        $this->namespace = $namespace;
    }

    private function getGFFormIdFromShortCode(string $shortCode)
    {
        preg_match('~\[gravityform(.+)\]~s', $shortCode, $mm);

        $shortCodeAtts = shortcode_parse_atts($mm[1]);

        return $shortCodeAtts['id'] ?? null;
    }

    private function normalizeExistingGFForm(array $forms): void
    {
        foreach ($forms as $key => $formData) {
            if (empty($formData['form']['hji_form_key'])) {
                $formData['form']['hji_form_key'] = $key;
            }

            if (empty($formData['form']['hji_form_version'])) {
                $formData['form']['hji_form_version'] = '0.0.1';
            }

            if (!empty($formData['form']['id'])) {
                \GFAPI::update_form($formData['form'], $formData['form']['id']);
            }
        }
    }

    private function updateGFMappingOptions(array $forms): bool
    {
        $options = get_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, []);

        foreach ($forms as $key => $form) {
            if (!isset($form['form']['hji_form_key'])) {
                $slugArray = explode(':', $key);
                $formSlug = $slugArray[1] ?? '';
            } else {
                $formSlug = $form['form']['hji_form_key'];
            }

            $pluginPath = '';
            if (!empty($form['pluginPath'])) {
                $pathArray = explode('/', $form['pluginPath']);
                $countPathParts = count($pathArray);
                if ($countPathParts > 2) {
                    $pluginPath = $pathArray[$countPathParts - 2] . '/' . $pathArray[$countPathParts - 1];
                }
            }

            $options[$this->namespace . ':' . $key] = [
                'id' => $form['form']['id'] ?? '',
                'pluginPath' => $pluginPath,
                'formVersion' => $form['form']['hji_form_version'] ?? '0.0.1',
                'formKey' => $formSlug,
                'formTitle' => $form['title'],
            ];
        }

        return update_option(Membership::HJI_GF_MAPPING_OPTIONS_KEY, $options, false);
    }

    public function normalizeGFMapping($existedForms): bool
    {
        if (!is_array($existedForms) || count($existedForms) === 0) {
            return false;
        }

        foreach ($existedForms as $key => $formData) {
            $formId = $this->getGFFormIdFromShortCode($formData['shortcode']);
            if (empty($formId)) {
                $existedForms[$key]['form'] = [];
                continue;
            }

            $existedForms[$key]['form'] = \GFAPI::get_form($formId);
        }

        $this->normalizeExistingGFForm($existedForms);

        return $this->updateGFMappingOptions($existedForms);
    }
}
