<?php
namespace hji\common\models;

use hji\common\utils\GFHelper;

/**
 * Class GForms - HJI GForms Model
 *
 * @package hji\common\models
 */


class GForms
{
    protected $cache = array();

    /**
     * @var string - gravity form settings key to identify HJI forms
     */
    const HJI_FORM_KEY = 'hji_form_key';

    /**
     * @var string - single field setting key to identify the field by
     */
    const HJI_FIELD_KEY = 'hji_field_key';
    
    const  HJI_FORM_VERSION = 'hji_form_version';


    /**
     * Determines if a given form is an HJI form
     *
     * @param array|object $form
     * @return bool
     */
    function isHJIform($form, $hjiFormKey = null)
    {
        // If form is an object, we need to request form meta
        if (is_object($form))
        {
            $form = $this->getFormsMetaById($form->id);
        }

        if (isset($form[self::HJI_FORM_KEY]) && !empty($form[self::HJI_FORM_KEY]))
        {
            if ($hjiFormKey && ($form[self::HJI_FORM_KEY] != $hjiFormKey))
            {
                return false;
            }

            return true;
        }

        return false;
    }


    function isHJIfield($field, $fieldKey = null)
    {
        if (!(is_array($field) || (GFHelper::gravityFormsExists() && $field instanceof \GF_Field))) {
            return false;
        }

        if (empty($field[self::HJI_FIELD_KEY])) {
            return false;
        }

        if (isset($fieldKey) && ($field[self::HJI_FIELD_KEY] !== $fieldKey)) {
            return false;
        }

        return true;
    }


    function getFormByKey($hjiFormKey)
    {
        if (!$forms = $this->getForms())
        {
            return false;
        }

        $ids = array();

        foreach ($forms as $form)
        {
            $ids[] = $form->id;
        }

        if (!$formsMeta = $this->getFormsMetaById($ids))
        {
            return false;
        }

        foreach ($formsMeta as $form)
        {
            if ($this->isHJIform($form, $hjiFormKey))
            {
                return (array)$form;
            }
        }

        return false;
    }


    public function getForms()
    {
        if (!isset($this->cache['forms']))
        {
            $this->cache['forms'] = \GFFormsModel::get_forms( null, 'title' );
        }

        return $this->cache['forms'];
    }


    /**
     * @param $id mixed - array or integer
     * @return array
     */
    public function getFormsMetaById($id)
    {
        if (!isset($this->cache['formsMeta']))
        {
            $this->cache['formsMeta'] = \GFFormsModel::get_form_meta_by_id($id);
        }

        return (array)$this->cache['formsMeta'];
    }

}
