<?php

namespace hji\common\postType;


class CustomPostType
{
    /**
     * Custom Post Type Definition
     *
     * Required Properties:
     * $post_type
     *
     * Display in REST API
     * $show_in_rest = true;
     *
     * For all available properties
     * @see PostTypeDefinition::setDefaults()
     * or
     * @see register_post_type()
     * @link https://codex.wordpress.org/Function_Reference/register_post_type#Arguments
     *
     */

    /** @param string   $post_type singular post type name - lower case, no spaces (- _ allowed) */
    public $post_type;

    public $singular_name;

    public $plural_name;

    public $public = true;

    public $supports = ['title', 'editor', 'thumbnail'];

    public $show_in_rest = false;

    /** @param string   $enter_title_here title field placeholder text */
    public $enter_title_here;

    /** @param string   $featured_image name of the featured image i.e. Agent Headshot */
    public $featured_image;

    /** @var string  $fieldPrefix meta field key prefix i.e. _hji_my_custom_field */
    public $fieldPrefix = '_hj_';


    /** @param array $fieldsSchema schema of metaboxes and their fields
     * @link https://github.com/CMB2/CMB2/wiki/Basic-Usage#create-a-metabox
     *
     * Supports custom params:
     *       @property array rest_field_args
     *                                get_callback
     *                                update_callback
     *                                schema
     * @link https://developer.wordpress.org/reference/functions/register_rest_field/
     */
    public $fieldsSchema = [];

    /** @param array $taxonomiesSchema
     *  taxonomyName => [$args]
     *
     * @see register_taxonomy() $args
     * @link https://codex.wordpress.org/Function_Reference/register_taxonomy#Arguments
     */
    public $taxonomiesSchema = [];

    /** @var array $adminColsSchema
     * @link https://github.com/johnbillion/extended-cpts/wiki/Admin-columns#example
     */
    public $adminColsSchema = [];

    /** @var array $adminFiltersSchema
     * @link https://github.com/johnbillion/extended-cpts/wiki/Admin-filters#example
     */
    public $adminFiltersSchema = [];
}