<?php

namespace hji\common\utils;


use hji\membership\extension\abstracts\Component;
use hji\membership\Membership;

class Compliance extends Component
{

    /**
     * Checks if a market or group of markets is authorized on the current license key
     * Given an array of markets it will return true if any of those markets are authorized.
     *
     * @since 0.7.0
     * @param $markets array
     * @return bool
     */
    public static function isMarketAuthorized(array $markets)
    {
        if (class_exists('\hji\membership\Membership', false))
        {
            foreach ($markets as $market)
            {
                if (in_array($market, self::getAuthorizedMarkets()))
                {
                    return true; break;
                }
            }
        }

        return false;
    }


    /**
     * Gets an array of authorized MLS markets from membership
     *
     * @since 0.7.0
     * @return mixed Authorized Markets
     */
    public static function getAuthorizedMarkets()
    {
        if (class_exists('\hji\membership\Membership', false))
        {
            return Membership::getInstance()->getMarkets();
        }

        return false;
    }


    /**
     * Returns single authorized market
     *
     * @since 0.7.0
     * @return string First market from authorized markets
     */
    public static function getFirstMarket()
    {
        $markets = self::getAuthorizedMarkets();

        return $markets[0];
    }


    /**
     * Try to get a market
     * If the market isn't authorized it will return the first authorized market.
     *
     * @since 0.13.0
     * @param bool $market
     * @return bool|string
     */
    public static function tryMarket($market = false)
    {
        if ($market && self::isMarketAuthorized(array($market)))
        {
            return $market;
        }
        else
        {
            return self::getFirstMarket();
        }
    }


    /**
     * Tells us if a market does not allow days on market
     *
     * @since 1.0.0
     * @param $market
     * @return bool
     */
    public static function allowsDaysOnMarket($market)
    {

        $restrictions = [
            'bareis',
        ];

        if (in_array($market, $restrictions))
        {
            add_filter('blvd/rIDX/secondary_fields', function ($map){
                unset($map['daysOnHJI']);

                return $map;
            });

            return false;
        }

        return true;
    }

}
