<?php
/**
 * @since 2.5.1
 * COOKIEPATH, COOKIE_DOMAIN are WP Core constants
 * defined in wp-includes/default-constants.php
 */

namespace hji\common\utils;


class Cookies
{
    const COOKIEPATH = COOKIEPATH;
    const COOKIE_DOMAIN = COOKIE_DOMAIN;


    public static function setCookie($name, $value = null, $expire = 0, $path = '', $domain = '', $secure = false, $httponly = false)
    {
        if (!$domain) {
            $domain = self::getCookieDomain();
        }

        if (!isset($path)) {
            $path = self::getCookiePath();
        }

        if ($secure === null) {
            $secure = false;
        }

        if ($httponly === null) {
            $httponly = false;
        }

        setcookie($name, $value, $expire, $path, $domain, $secure, $httponly);
    }


    public static function getCookie($name)
    {
        return (isset($_COOKIE[$name])) ? $_COOKIE[$name] : null;
    }


    public static function deleteCookie($name, $path = null, $domain = null)
    {
        if (!$domain) $domain = self::getCookieDomain();
        if (!isset($path)) {
            $path = self::getCookiePath();
        }

        self::setCookie($name, '', time() - 3600, $path, $domain);
    }


    public static function getCookieDomain()
    {
        return (is_multisite()) ? get_blog_details('domain') : Cookies::COOKIE_DOMAIN;
    }

    public static function getCookiePath()
    {
        return defined('COOKIEPATH') ? Cookies::COOKIEPATH : preg_replace('|https?://[^/]+|i', '', get_blog_details('home') . '/');
    }
}
