<?php

namespace hji\common\utils;


class DataFormatter
{
    public $sections = array();


    /**
     * Adds a section of data to the model
     *
     * @since 1.0.0
     * @param string $title
     * @param array  $data
     * @param bool   $label_value use true for data models of label => value. use false for single string values
     */
    public function addSection($title, $data, $label_value = true)
    {
        $this->sections[$title] = array();
        $this->addData($data, $title, $label_value);
    }


    /**
     * Adds data to a section
     *
     * @since 1.0.0
     * @param array  $data
     * @param string $section
     * @param bool   $label_value use true for data models of label => value. use false for single string values
     */
    public function addData($data, $section = '', $label_value = true)
    {
        if ($label_value)
        {
            $this->sections[$section] = $this->_processAsLabelValue($data);
        }
        else
        {
            $this->sections[$section] = $this->_processAsString($data);
        }
    }


    /**
     * Gets the formatted data
     *
     * @since 1.0.0
     * @return array
     */
    public function getFormatted()
    {
        foreach ($this->sections as $id => $section)
        {
            if (empty($section))
            {
                unset($this->sections[$id]);
            }
        }
        return $this->sections;
    }


    private function _processAsLabelValue($data)
    {
        $data_array = array();

        foreach ($data as $label => $value)
        {
            $formatLabel = trim(ucfirst(preg_replace('/([a-z])(?<!\ )([A-Z])/', '$1 $2', $label)));
            $id = $this->_dehumanize($label);

            if ($value)
            {
                $data_array[$id] = array(
                    'name'  => $id,
                );
                if (is_bool($value)) // if it's a boolean we don't need a label
                {
                    $data_array[$id]['value'] = $formatLabel;
                }
                else
                {
                    $data_array[$id]['label'] = $formatLabel;
                    $data_array[$id]['value'] = $value;
                }
            }
        }

        return $data_array;
    }


    private function _processAsString($data)
    {
        foreach ($data as $value)
        {
            $value_as_array = explode(':', $value, 2);
            if (isset($value_as_array[1]))
            {
                $data_array[] = array('label' => trim($value_as_array[0]), 'value' => trim($value_as_array[1]));
            }
            else
            {
                $data_array[] = array('value' => trim($value_as_array[0]));
            }
        }

        return $data_array;
    }


    private function _dehumanize($string)
    {
        $forbidden_characters = array('(', ')', '[', ']');
        $dehumanized = trim(strtolower($string));
        $dehumanized = trim(str_replace(' ', '_', $dehumanized));

        return trim(str_replace($forbidden_characters, '', $dehumanized));
    }


}