<?php
namespace hji\common\utils;


class GFHelper
{
    public static function isGravityFormExportInitialized()
    {
        if (!class_exists('\GFExport')) {
            if (file_exists(WP_CONTENT_DIR . '/plugins/gravityforms/export.php')) {
                require_once WP_CONTENT_DIR . '/plugins/gravityforms/export.php';

                return  true;
            }

            return false;
        }

        return true;
    }

    public static function gravityFormsExists()
    {
        return class_exists('\GFForms');
    }

    public static function gravityFormsAPIEnable()
    {
        return class_exists('\GFAPI');
    }


    public static function importGravityForm($file)
    {
        require_once(\GFCommon::get_base_path() . "/export.php");

        return \GFExport::import_file($file);
    }


    public static function getFormByTitle($title)
    {
        $forms = \GFFormsModel::get_forms(null, 'title');

        foreach ($forms as $form)
        {
            if (trim($form->title) == $title)
            {
                return $form;
            }
        }
    }


    /**
     * Doesn't work with file upload fields
     *
     * @param $field_id
     * @param $form
     * @return array|mixed|string|void
     */
    public static function getFieldValue($field_id, $form)
    {
        if ($field = \GFFormsModel::get_field($form, $field_id))
        {
            return \GFFormsModel::get_field_value($field);
        }
    }


    /**
     * Returns array of forms that match given HJI Form Key
     *
     * @param $key
     * @return array
     */
    public static function getFormsByKey($key)
    {
        $forms = \GFFormsModel::get_forms(null, 'id');

        $list = array();
        $result = array();

        foreach ($forms as $form)
        {
            $list[$form->id] = $form;
        }

        $meta = \RGFormsModel::get_form_meta_by_id(array_keys($list));

        foreach ($meta as $form)
        {
            if (isset($form['hji_form_key']) && stristr($form['hji_form_key'], $key . ':'))
            {
                $result[] = $form;
            }
            else if (isset($form['hji_form_key']) && $form['hji_form_key'] == $key)
            {
                $result[] = $form;
            }
        }

        return $result;
    }


    public static function getFormsListByKey($key)
    {
        $forms = self::getFormsByKey($key);

        $list = array();

        if (!empty($forms))
        {
            foreach ($forms as $form)
            {
                $list[$form['id']] = $form['title'];
            }
        }

        return $list;
    }


    public static function getFieldByKey($form, $fieldKey)
    {
        foreach ($form['fields'] as $field)
        {
            if (isset($field['hji_field_key']) && $field['hji_field_key'] == $fieldKey)
            {
                return $field;
            }
        }
    }

    public static function getFormFieldValueFromShortcode(string $shortcode, string $fieldName)
    {
        if (empty($shortcode) || empty($fieldName)) {
            return null;
        }

        preg_match('~\[gravityform(.+)\]~s', $shortcode, $mm);

        $shortCodeAtts = shortcode_parse_atts($mm[1]);

        return $shortCodeAtts[$fieldName] ?? null;
    }

    public static function getGFFormIdFromShortCode(string $shortCode)
    {

        return self::getFormFieldValueFromShortcode($shortCode, 'id') ?? null;
    }
}
