<?php
/**
 * Wrapper class for CMB2 (third-party library)
 * @since 2.0.0
 */
namespace hji\common\utils;

use \hji\membership\Membership;

class MetaBoxes
{
    private static $_instance;

    // Metaboxes settings collection

    private $metaBoxes = array();

    private function __construct()
    {
        $this->loadLibraries();

        add_action('cmb2_admin_init', array($this, 'registerMetaBoxes'));
    }


    private static function getInstance()
    {
        if (!(self::$_instance instanceof self))
        {
            self::$_instance = new self();
        }

        return self::$_instance;
    }


    private function loadLibraries()
    {
        require_once(Membership::$dir . '/third-party/cmb2/init.php');

        // @link https://github.com/mustardBees/cmb-field-select2
        require_once(Membership::$dir . '/third-party/cmb2-field-select2/cmb-field-select2.php');
        // @link https://github.com/mustardBees/cmb-field-gallery
        // @TODO: deprecated field -- see link above for more information
        require_once(Membership::$dir . '/third-party/cmb2-field-gallery/cmb-field-gallery.php');
        // @link https://github.com/alexis-magina/cmb2-field-post-search-ajax
        require_once(Membership::$dir . '/third-party/cmb2-field-post-search-ajax/cmb-field-post-search-ajax.php');
    }


    public function registerMetaBoxes()
    {
        return self::get();
    }


    /**
     *
     *
     * @param     $metaBoxes array of metabox settings
     * @param int $priority
     */
    public static function add(array $metaBoxes, $priority = 10)
    {
        $instance = self::getInstance();

        if (!empty($metaBoxes))
        {
            $instance->set($metaBoxes, $priority);
        }
    }


    private static function set($metaBoxes, $priority)
    {
        $instance = self::getInstance();
        $set = array(
            'data'      => $metaBoxes,
            'priority'  => $priority
        );

        array_push($instance->metaBoxes, $set);
    }


    private function prioritize()
    {
        if (empty($this->metaBoxes))
        {
            return $this->metaBoxes;
        }

        $priority = array();
        $data = array();

        foreach ($this->metaBoxes as $k => $set)
        {
            $priority[$k] = $set['priority'];
            $data[$k] = $set['data'];
            $keys[$k] = $k;
        }

        // Sorting by keys assures that same priority items
        // are returned in the order they were added.

        array_multisort($priority, SORT_ASC, $keys, SORT_ASC, $this->metaBoxes);
    }


    /**
     * Global metaboxes getter
     *
     * @return array metaboxes settings
     */
    public static function get($object_type = false)
    {
        $instance = self::getInstance();
        $instance->prioritize();

        $results = $metaboxes =  array();

        // TODO: needs refactoring
	    // TODO: this method is part of the CMB2 hook, but the hook does not expect a return
	    // TODO: either fall back on the old format where we return cmb_config array,
	    //      or create new_cmb_box() in a separate method.
        if ($object_type)
        {
            foreach ($instance->metaBoxes as $box)
            {
                if ($box['data']['object_type'] == $object_type)
                {
                    $results = array_merge($results, $box['data']);
                }
            }

        }
        else
        {
            foreach ($instance->metaBoxes as $box)
            {
                foreach ($box['data'] as $meta => $key)
                {
                    $box['data'][$meta]['id'] = $meta;
                }

                $results = array_merge($results, $box['data']);

            }

            foreach ($results as $result)
            {
                 $metaboxes[] =  new_cmb2_box($result);

            }

            return $metaboxes;
        }
    }


    public static function renderField($fieldType, $callback)
    {
        if (is_callable($callback))
        {
            add_action('cmb2_render_' . $fieldType, $callback, 10, 5);
        }
    }


    public static function validateField($fieldType, $callback)
    {
        if (is_callable($callback))
        {
            add_filter('cmb2_sanitize_' . $fieldType, $callback, 10, 2);
        }
    }
}