<?php

namespace hji\common\utils;

use hji\membership\extension\abstracts\Component;

/**
 * Mortgage Utility
 * Creates helper functions for show mortgage data on property pages and throughout the site.
 *
 * @package hji\blvd\common
 * @since 0.2.0
 */
class MortgageInfo extends Component
{
    public static $rateTransient = 'hji_mortgage_rates';

    protected static $interestRate;
    protected static $downPayment = 20;
    protected static $paymentTerm = 30;
    protected static $loanAmount;
    protected static $mortgageRates;

    protected static $rateApiURL = 'https://www.quandl.com/api/v3/datasets/FMAC/WRMORTG.json';


    public function __construct()
    {

    }


    public static function get_ratesTransient()
    {
        return self::$rateTransient;
    }


    public static function getRates()
    {

        self::$mortgageRates = get_transient(self::$rateTransient);

        if ( self::$mortgageRates === false )
        {
            self::$mortgageRates = wp_remote_get(self::$rateApiURL);
            $apiCall = wp_remote_retrieve_body(self::$mortgageRates);

            set_transient(self::$rateTransient, $apiCall, 2 * WEEK_IN_SECONDS);

            return $apiCall;
        }

        return self::$mortgageRates;
    }

    public static function getCurrentRate()
    {
        $currentRate = 3.7;

        $currentRates = self::getRates();
        if ($currentRates !== '')
        {
            $currentRates_obj = json_decode($currentRates);
            if (isset($currentRates_obj->dataset) && isset($currentRates_obj->dataset->data[0][1]))
            {
                $currentRate = $currentRates_obj->dataset->data[0][1];
            }
        }

        return $currentRate;
    }

    public static function calculatePayment($salePrice)
    {
        $currentRate = 3.7;
        $salePrice = preg_replace( "/([^0-9\\.])/i", "", $salePrice);
        $currentRates = self::getRates();
        if ($currentRates !== '')
        {
            $currentRates_obj = json_decode($currentRates);
            if (isset($currentRates_obj->dataset) && isset($currentRates_obj->dataset->data[0][1]))
            {
                $currentRate = $currentRates_obj->dataset->data[0][1];
            }
        }
        $apr    = $currentRate / 100;
        $dwnPmt = round($salePrice * self::$downPayment / 100 * 100) / 100;

        $r = $apr  / 12;
        $t = self::$paymentTerm * 12;
        $p = round(($salePrice - $dwnPmt) * 100) / 100;

        $b = 1 + $r;
        $c = $t * (-1);
        $a = 1 - pow($b, $c);

        $payment = $p * ( $r / $a );

        return '$' . number_format($payment);
    }

    /**
     * @return int
     */
    public static function getPaymentTerm(): int
    {
        return self::$paymentTerm;
    }

    /**
     * @return int
     */
    public static function getDownPayment(): int
    {
        return self::$downPayment;
    }
}
