<?php

namespace hji\common\utils;


/**
 * Class ShortcodeBinding
 * Used to bind shortcodes to Twig Object Models
 *
 * @package hji\blvd\utils
 */
class ShortcodeBinding
{
    private $shortcode_id;
    private $filter_type;

    public function __construct($filter_id = null)
    {
        $this->filter_type = $filter_id;
    }


    public function __toString()
    {
        return do_shortcode($this->run());
    }


    public function __call($name, $arguments)
    {
        if (isset($arguments[0]))
        {
            $this->{$name} = $arguments[0];
        }
        return $this;
    }


    public function setShortcodeID($key)
    {
        $this->shortcode_id = apply_filters('blvd/sc_binding/' . $this->filter_type . '/id', $key);
        return $this;
    }


    /**
     * @return string Constructed shortcode with all parameters
     */
    protected function run()
    {
        $output_params = '';
        $vars = get_object_vars($this);
        $filter = $this->filterVars($vars);

        foreach ($filter as $key => $value)
        {
            if ($value !=='' && $key != 'shortcode_id' && $key != 'filter_id')
            {
                $output_params .= " {$key}=\"{$value}\"";
            }
        }

        return "[{$this->shortcode_id}{$output_params}]";
    }

    /**
     * Method adds to or modifies shortcode parameters based on
     * model_type and shortcode_id
     * 
     *
     * @param array $vars filters to be added to shortcode
     * @return string Constructed shortcode with all parameters
     */
    protected function filterVars($vars)
    {
        if (!$this->filter_type)
        {
            return $vars;
        }
        else
        {
            $vars = apply_filters('blvd/sc_binding/' . $this->filter_type, $vars);
        }

        return $vars;

    }
}