<?php

namespace hji\common\utils;

class StringUtil
{
    static function truncate($s, $length)
    {
        if (!empty($s) && is_string($s))
        {
            if (strlen($s) > $length)
            {
                $s = substr($s, 0, $length) . '...';
            }
        }
        
        return $s;
    }
    
    static function toBoolean($s)
    {
        $b = false;
        
        if (isset($s) && ($s !== false))
        {
            if ($s === true)
            {
                $b = true;
            }
            else
            {
                $s = strtolower(trim($s));
        
                if (ctype_digit($s))
                {
                    $b = (intval($s) != 0) ? true : false;
                }
                else if (($s == 'true') || ($s == 'yes') || ($s == 'y') || ($s == 'on') || ($s == 't'))
                {
                    $b = true;
                }
            }
        }
        
        return $b;
    }

    static function toInteger($s)
    {
        $i = 0;

        if (!is_array($s) && !is_object($s))
        {
            $i = (int)preg_replace("/[^0-9\\.\\/]/", "", $s);
        }

        return !is_nan($i) && is_int($i) ? $i : 0;
    }


    static function moneyToInteger($s)
    {
        $i = 0;

        if (!is_array($s) && !is_object($s))
        {
            $i = (int)preg_replace("/([^0-9\\.])/i", "", $s);
        }

        return !is_nan($i) ? $i : 0;
    }


    static function toArray($s)
    {
        $array = array();

        $s = trim($s);

        // Pipes and commas are usual delimiters

        if (strstr($s, '|'))
        {
            $array = explode('|', $s);
        }
        else if (strstr($s, ','))
        {
            $array = explode(',', $s);
        }
        else
        {
            $array[] = $s;
        }

        foreach ($array as &$value)
        {
            $value = trim($value);
        }

        return array_filter($array);
    }


    static function applyPrefix(string $string, string $prefix)
    {
        return (!self::hasPrefix($string, $prefix)) ? $prefix . $string : $string;
    }


    static function removePrefix(string $string, string $prefix)
    {
        return (self::hasPrefix($string)) ?
            str_replace($prefix, '', $string) : $string;
    }


    static function hasPrefix(string $string, string $prefix = '')
    {
        return (substr($string, 0, strlen($prefix)) === $prefix);
    }

    static function  fullBathBedRoom($text)
    {
        $patterns = ['/1Br/i',  '/1Ba/i'];
        $replacements = ['1 bath', '1 bedroom'];
        $text = preg_replace($patterns, $replacements, $text);

        $patterns = ['/(\d)(Br)/i',  '/(\d)(Ba)/i'];
        $replacements = ['$1 baths', '$1 bedrooms'];
        $text = preg_replace($patterns, $replacements, $text);

        return $text;
    }
}