<?php

namespace hji\common\utils;

use \hji\membership\interfaces\Product;
use \hji\membership\Membership;
use hji\membership\models\VersionDependency;
use \hji\membership\models\WpCatalog;


class Updater
{
    /**
     * Product entity object
     *
     * @var object
     */
    private $product;

    /**
     * Product type - plugin|theme
     *
     * @var string
     */
    private $type;

    /**
     * Fully qualified path to the product's file
     *
     * @var
     */
    private $file;

    /**
     * Product slug
     *
     * @var string
     */
    private $slug;

    /**
     * API Url to get plugin update info
     *
     * @var bool|string
     */
    private $infoUrl;


    public $checker;


    public function __construct($product)
    {
        $this->product = $product;

        // For backward compatibility when file path was passed into the constructor

        if (is_string($product))
        {
            $this->file = $product;
            $this->type = 'plugin';
            $this->slug = basename(dirname($this->file));
        }
        else if ($product instanceof Product)
        {
            $this->file = $product->getFile();
            $this->type = $product->getType();
            $this->slug = $product->getSlug();
        }

        $this->infoUrl = WpCatalog::getInfoUrl($this->slug);

        $this->run();
    }


    /**
     * Provides get access to all private/protected class properties
     *
     * @param $property
     * @return mixed
     */
    function __get($property)
    {
        if (property_exists(__CLASS__, $property))
        {
            return $this->$property;
        }
    }


    public function run()
    {
        if ($this->infoUrl)
        {
            switch($this->type)
            {
                case 'plugin':
                    $this->_setPluginUpdater();
                    break;

                case 'theme':
                    $this->_setThemeUpdater();
                    break;
            }
        }
    }


    /**
     * Sets plugin updater based on currently constructed object
     */
    protected function _setPluginUpdater()
    {
        require_once(Membership::$dir . '/third-party/updater/plugin-update-checker.php');

        $this->checker = new \PluginUpdateChecker($this->infoUrl, $this->file, $this->slug, 24);

        add_filter('puc_request_info_query_args-' . $this->slug, array($this, 'modifyRequestInfo'));
    }


    /**
     * Sets theme updater based on currently constructed object
     */
    protected function _setThemeUpdater()
    {
        require_once(Membership::$dir . '/third-party/updater/theme-update-checker.php');

        $this->checker = new \ThemeUpdateChecker($this->slug, $this->infoUrl);

        add_filter('tuc_request_update_query_args-' . $this->slug, array($this, 'modifyRequestInfo'));
    }


    /**
     * Injecting a slug parameter into the update checker's request,
     * so we can identify it during pre_http_request hook implementation
     *
     * @since 2.0.0
     * @param $args
     * @return mixed
     */
    function modifyRequestInfo($args)
    {
        // Pass slug, so we can easily identify this request
        // during pre_http_request interception

        $args['slug'] = $this->slug;

        return $args;
    }

}