<?php
/**
 * This class first looks in theme's folder
 * then in the views, therefore allowing overwriting
 * of templates.
 */

namespace hji\common\utils;

use hji\membership\rendering\Renderer;


class Views
{
    protected $vars = array();
    protected $templatesDir = array();
    protected $fileName = null;
    protected $textdomain = 'hji-textdomain';

    /** @var  Renderer */
    protected $templateRenderer;


    function __construct($templatesDir = null)
    {
        if ($templatesDir)
        {
           $this->templatesDir = array_merge($this->templatesDir, (array)$templatesDir);
        }

        $this->templateRenderer = new Renderer();
        $this->templateRenderer->setLocations($this->templatesDir);
    }


    /**
     * Render method for rendering a .phtml or .twig template
     *
     * @param bool|false       $fileName
     * @param array|bool|false $vars
     * @return bool|string
     */
    public function render($fileName = false, $vars = null)
    {
        list($template, $vars) = $this->_prepare($fileName, $vars);

        $ss = $this->templateRenderer->render($template, $vars);
        return $ss;
    }


    /**
     * Sets the text domain variable for templates
     *
     * @param $textdomain
     */
    public function setTextdomain($textdomain)
    {
        $this->textdomain = $textdomain;
    }


    /**
     * Sets the filename for the template
     *
     * @param $fileName
     */
    public function setFileName($fileName)
    {
        $this->fileName = $fileName;
    }


    function getFileName()
    {
        return $this->fileName;
    }


    /**
     * Setter for template variables array
     *
     * @param $name
     * @param $value
     */
    public function set($name, $value)
    {
        $this->vars[$name] = $value;
    }


    /**
     * Getter for template variables array
     *
     * @param $name
     * @return null
     */
    public function get($name)
    {
        return (isset($this->vars[$name])) ? $this->vars[$name] : null;
    }


    /**
     * Validates that a filename was passed correctly when calling render method
     *
     * @param $fileName
     * @return null|void
     */
    protected function _validateFile($fileName)
    {
        if (!$fileName)
        {
            if (!$this->fileName)
            {
                trigger_error("Template file name is required.", E_USER_ERROR);

                return;
            }
            else
            {
                $fileName = $this->fileName;
            }
        }

        return $fileName;
    }


    /**
     * Prepares and validates we've passed template variables
     *
     * @param $vars
     * @return bool|void
     */
    protected function _prepareVars()
    {
        $vars = $this->vars;

        if (!isset($vars['textdomain']) || empty($vars['textdomain']))
        {
            $vars['textdomain'] = $this->textdomain;
        }

        return $vars;
    }


    protected function _prepare($fileName, $vars)
    {
        if ($fileName)
        {
            $this->setFileName($fileName);
        }

        if (is_array($vars))
        {
            $this->vars = array_merge($this->vars, $vars);
        }

        $template   = $this->_validateFile($this->getFileName());
        $vars       = $this->_prepareVars();

        return array($template, $vars);
    }


    /**
     * Sets the twig cache path
     *
     * @param $twig
     * @return mixed
     *
     * @deprecated since 2.4.8 in favor of timber/cache/location filter
     */
    public function setTwigCachePath($twig)
    {
        _deprecated_function(__FUNCTION__, '2.4.8', null);

        if (!file_exists(WP_CONTENT_DIR . '/hji-cache/twig'))
        {
            mkdir(WP_CONTENT_DIR . '/hji-cache/twig', 0700, true);
        }

        $twig->setCache(WP_CONTENT_DIR . '/hji-cache/twig');

        return $twig;
    }
}
